/* --------------------------------------------------------------------------------------------
 * Copyright (c) Jan Dolejsi. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for license information.
 * ------------------------------------------------------------------------------------------ */
'use strict';
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.PlannerAsyncService = void 0;
const PlannerService_1 = require("./PlannerService");
/** Wraps the `/request` planning web service interface. */
class PlannerAsyncService extends PlannerService_1.PlannerService {
    constructor(plannerUrl, asyncPlannerConfiguration, providerConfiguration) {
        super(plannerUrl, asyncPlannerConfiguration, providerConfiguration);
        this.asyncPlannerConfiguration = asyncPlannerConfiguration;
        this.timeout = PlannerAsyncService.DEFAULT_TIMEOUT; //this default is overridden by info from the configuration!
        this.asyncMode = false;
        this.planTimeScale = 1;
    }
    getTimeout() {
        return this.timeout;
    }
    createUrl() {
        return this.plannerPath + '?async=' + this.asyncMode;
    }
    // eslint-disable-next-line @typescript-eslint/no-explicit-any
    createRequestBody(domainFileInfo, problemFileInfo) {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            const configuration = this.asyncPlannerConfiguration;
            if (!configuration) {
                return null;
            }
            configuration.planFormat = (_a = configuration.planFormat) !== null && _a !== void 0 ? _a : 'JSON';
            if (("timeout" in configuration) && configuration.timeout !== undefined) {
                this.timeout = configuration.timeout;
            }
            this.planTimeScale = PlannerAsyncService.getPlanTimeScale(configuration);
            return {
                'domain': {
                    'name': domainFileInfo.name,
                    'format': 'PDDL',
                    'content': domainFileInfo.getText()
                },
                'problem': {
                    'name': problemFileInfo.name,
                    'format': 'PDDL',
                    'content': problemFileInfo.getText()
                },
                'configuration': configuration
            };
        });
    }
    // eslint-disable-next-line @typescript-eslint/no-explicit-any
    static getPlanTimeScale(configuration) {
        const planTimeUnit = configuration['planTimeUnit'];
        switch (planTimeUnit) {
            case "MINUTE":
                return 60;
            case "MILLISECOND":
                return 1 / 1000;
            case "HOUR":
                return 60 * 60;
            case "DAY":
                return 24 * 60 * 60;
            case "WEEK":
                return 7 * 24 * 60 * 60;
            case "SECOND":
            default:
                return 1;
        }
    }
    // eslint-disable-next-line @typescript-eslint/no-explicit-any
    processServerResponseBody(responseBody, planParser, callbacks, resolve, reject) {
        return __awaiter(this, void 0, void 0, function* () {
            let _timedOut = false;
            const responseStatus = responseBody['status']['status'];
            if (["STOPPED", "SEARCHING_BETTER_PLAN"].includes(responseStatus)) {
                _timedOut = responseBody['status']['reason'] === "TIMEOUT";
                if (responseBody['plans'].length > 0) {
                    const plansJson = responseBody['plans'];
                    try {
                        // eslint-disable-next-line @typescript-eslint/no-explicit-any
                        const parserPromises = plansJson.map((plan) => this.parsePlan(plan, planParser));
                        yield Promise.all(parserPromises);
                    }
                    catch (err) {
                        reject(err);
                    }
                    const plans = planParser.getPlans();
                    if (plans.length > 0) {
                        callbacks.handleOutput(plans[0].getText() + '\n');
                    }
                    else {
                        callbacks.handleOutput('No plan found.');
                    }
                    resolve(plans);
                    return;
                }
                else {
                    // todo: no plan found yet. Poll again later.
                    resolve([]);
                    return;
                }
            }
            else if (responseStatus === "FAILED") {
                const error = responseBody['status']['error']['message'];
                reject(new Error(error));
                return;
            }
            else if (["NOT_INITIALIZED", "INITIATING", "SEARCHING_INITIAL_PLAN"].includes(responseStatus)) {
                _timedOut = true;
                const error = `After timeout ${this.timeout} the status is ${responseStatus}`;
                reject(new Error(error));
                return;
            }
            console.log(_timedOut);
        });
    }
    // eslint-disable-next-line @typescript-eslint/no-explicit-any
    parsePlan(plan, planParser) {
        return __awaiter(this, void 0, void 0, function* () {
            const makespan = plan['makespan'];
            const metric = plan['metricValue'];
            const searchPerformanceInfo = plan['searchPerformanceInfo'];
            const statesEvaluated = searchPerformanceInfo['statesEvaluated'];
            const elapsedTimeInSeconds = parseFloat(searchPerformanceInfo['timeElapsed']) / 1000;
            planParser.setPlanMetaData(makespan, metric, statesEvaluated, elapsedTimeInSeconds, this.planTimeScale);
            const planFormat = plan['format'];
            if ((planFormat === null || planFormat === void 0 ? void 0 : planFormat.toLowerCase()) === 'json') {
                const planSteps = JSON.parse(plan['content']);
                this.parsePlanSteps(planSteps, planParser);
                planParser.onPlanFinished();
            }
            else if ((planFormat === null || planFormat === void 0 ? void 0 : planFormat.toLowerCase()) === 'tasks') {
                const planText = plan['content'];
                planParser.appendLine(planText);
                planParser.onPlanFinished();
            }
            else if ((planFormat === null || planFormat === void 0 ? void 0 : planFormat.toLowerCase()) === 'xplan') {
                const planText = plan['content'];
                yield planParser.appendXplan(planText); // must await the underlying async xml parsing
            }
            else {
                throw new Error('Unsupported plan format: ' + planFormat);
            }
        });
    }
    static createDefaultConfiguration(timeout) {
        return {
            planFormat: "JSON",
            timeout: timeout
        };
    }
}
exports.PlannerAsyncService = PlannerAsyncService;
PlannerAsyncService.DEFAULT_TIMEOUT = 60;
//# sourceMappingURL=PlannerAsyncService.js.map