/* --------------------------------------------------------------------------------------------
 * Copyright (c) Jan Dolejsi. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for license information.
 * ------------------------------------------------------------------------------------------ */
'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
exports.NormalizedPlanDocumentContentProvider = void 0;
const vscode_1 = require("vscode");
const pddl_workspace_1 = require("pddl-workspace");
const workspaceUtils_1 = require("../workspace/workspaceUtils");
const ai_planning_val_1 = require("ai-planning-val");
const PlanningDomains_1 = require("../catalog/PlanningDomains");
const Catalog_1 = require("../catalog/Catalog");
const utils_1 = require("../utils");
/**
 * Normalizes the plan at that URI and outputs the normal representation so it can be used for diffing.
 */
class NormalizedPlanDocumentContentProvider {
    constructor(pddlWorkspace, configuration, includeFinalValues) {
        this.pddlWorkspace = pddlWorkspace;
        this.configuration = configuration;
        this.includeFinalValues = includeFinalValues;
        this._onDidChange = new vscode_1.EventEmitter();
        this.changingUris = [];
    }
    get onDidChange() {
        return this._onDidChange.event;
    }
    dispose() {
        this._onDidChange.dispose();
        if (this.timeout) {
            clearTimeout(this.timeout);
        }
    }
    planChanged(uri) {
        if (this.timeout) {
            clearTimeout(this.timeout);
        }
        if (!this.changingUris.some(uri1 => uri1.toString() === uri.toString())) {
            this.changingUris.push(uri);
        }
        this.timeout = setTimeout(() => this.updateChangedPlans(), 1000);
    }
    updateChangedPlans() {
        this.timeout = undefined;
        this.changingUris.forEach(uri => this._onDidChange.fire(uri));
        this.changingUris = [];
    }
    provideTextDocumentContent(uri, token) {
        if (token.isCancellationRequested) {
            return "Canceled";
        }
        const newScheme = (uri.authority === PlanningDomains_1.PlanningDomains.AUTHORITY) ? Catalog_1.HTTPLAN : 'file';
        const fileUri = uri.with({ scheme: newScheme });
        return vscode_1.workspace.openTextDocument(fileUri)
            .then(document => document.getText())
            .then(documentText => this.normalize(fileUri, documentText));
    }
    async normalize(uri, origText) {
        let normalizedPlan = new pddl_workspace_1.parser.NormalizingPddlPlanParser(this.configuration.getEpsilonTimeStep()).normalize(origText, '\n');
        if (this.includeFinalValues) {
            try {
                const planValuesAsText = await this.evaluate(uri, origText);
                if (planValuesAsText) {
                    normalizedPlan = `${normalizedPlan}\n\n;; Modified state values:\n\n${planValuesAsText}`;
                }
            }
            catch (err) {
                vscode_1.window.showWarningMessage(err.toString());
            }
        }
        return normalizedPlan;
    }
    async evaluate(uri, origText) {
        const planMetaData = pddl_workspace_1.parser.PddlPlanParser.parsePlanMeta(origText);
        if (planMetaData.domainName !== pddl_workspace_1.parser.UNSPECIFIED_DOMAIN && planMetaData.problemName !== pddl_workspace_1.parser.UNSPECIFIED_DOMAIN) {
            const planInfo = new pddl_workspace_1.parser.PddlPlanParser().parseText(origText, this.configuration.getEpsilonTimeStep(), utils_1.toURI(uri));
            const context = workspaceUtils_1.getDomainAndProblemForPlan(planInfo, this.pddlWorkspace);
            const valStepPath = await this.configuration.getValStepPath();
            const verbose = this.configuration.getValStepVerbose();
            const planValues = await new ai_planning_val_1.PlanEvaluator()
                .evaluate(context.domain, context.problem, planInfo, { valStepPath: valStepPath, verbose });
            const planValuesAsText = planValues === null || planValues === void 0 ? void 0 : planValues.sort((a, b) => a.getVariableName().localeCompare(b.getVariableName())).map(value => `; ${value.getVariableName()}: ${value.getValue()}`).join("\n");
            return planValuesAsText !== null && planValuesAsText !== void 0 ? planValuesAsText : '';
        }
        else {
            return "";
        }
    }
}
exports.NormalizedPlanDocumentContentProvider = NormalizedPlanDocumentContentProvider;
//# sourceMappingURL=NormalizedPlanDocumentContentProvider.js.map