/* --------------------------------------------------------------------------------------------
 * Copyright (c) Jan Dolejsi. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for license information.
 * ------------------------------------------------------------------------------------------ */
'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
exports.SuggestionDetails = exports.Suggestion = exports.AbstractCompletionItemProvider = void 0;
const vscode_1 = require("vscode");
const pddl_workspace_1 = require("pddl-workspace");
const utils_1 = require("../utils");
class AbstractCompletionItemProvider {
    constructor() {
        this.suggestionDetails = new Map();
    }
    addSuggestionDocumentation(label, detail, documentation, kind) {
        const details = new SuggestionDetails(label, detail, documentation, kind);
        this.suggestionDetails.set(details.label, details);
    }
    insideDefine(domainInfo, currentNode, context) {
        const defineNode = domainInfo.syntaxTree.getDefineNode();
        if (context.triggerKind === vscode_1.CompletionTriggerKind.Invoke) {
            return currentNode.getParent() === defineNode;
        }
        else {
            const enclosingScope = currentNode.expand();
            return enclosingScope.getToken().tokenText.match(/^\(\s*:?$/) !== null
                && enclosingScope.getParent() === defineNode;
        }
    }
    insideRequirements(domainInfo, currentNode, _context) {
        const defineNode = domainInfo.syntaxTree.getDefineNode();
        const requirementsNode = defineNode.getFirstChild(pddl_workspace_1.parser.PddlTokenType.OpenBracketOperator, /:\s*requirements/i);
        if (!requirementsNode) {
            return false;
        }
        return currentNode.getParent() === requirementsNode;
    }
    createSnippetCompletionItem(suggestion, snippet, range, _context, index) {
        var _a;
        if (suggestion === null) {
            return null;
        }
        const suggestionDetail = this.suggestionDetails.get(suggestion.sectionName);
        const completionItem = new vscode_1.CompletionItem(suggestion.sectionName, (_a = suggestionDetail === null || suggestionDetail === void 0 ? void 0 : suggestionDetail.kind) !== null && _a !== void 0 ? _a : vscode_1.CompletionItemKind.Keyword);
        completionItem.insertText = new vscode_1.SnippetString(snippet);
        if (range) {
            completionItem.range = range;
        }
        if (suggestionDetail) {
            completionItem.detail = suggestionDetail.detail;
            completionItem.documentation = suggestionDetail.documentation;
        }
        completionItem.filterText = suggestion.filterText;
        completionItem.sortText = 'item' + index;
        return completionItem;
    }
    addConstraintsDocumentation() {
        this.addSuggestionDocumentation(':constraints', 'Constraints', 'Constraints that all plans must satisfy.');
    }
    addRequirementsDocumentation() {
        this.addSuggestionDocumentation(':requirements', 'Requirements', 'Required planning engine features.');
    }
    createRequirementsCompletionItems(document, currentNode, context) {
        const range = context.triggerCharacter && ['(', ':'].includes(context.triggerCharacter)
            ? utils_1.nodeToRange(document, currentNode) : null;
        const requirements = ['strips', 'typing', 'negative-preconditions', 'disjunctive-preconditions', 'equality', 'existential-preconditions', 'universal-preconditions', 'quantified-preconditions', 'conditional-effects', 'fluents', 'numeric-fluents', 'object-fluents', 'adl', 'durative-actions', 'duration-inequalities', 'continuous-effects', 'derived-predicates', 'derived-functions', 'timed-initial-literals', 'timed-effects', 'preferences', 'constraints', 'action-costs', 'timed-initial-fluents', 'time'];
        return requirements.map(r => ':' + r)
            .map((r, index) => {
            const suggestion = Suggestion.from(r, context.triggerCharacter, '');
            if (suggestion) {
                return this.createSnippetCompletionItem(suggestion, r, range, context, index);
            }
            else {
                return null;
            }
        })
            .filter(s => !!s)
            .map(s => s);
    }
    addIconGallery() {
        // Object.keys(CompletionItemKind).map(key => CompletionItemKind[key]);
        return [
            new vscode_1.CompletionItem("Class", vscode_1.CompletionItemKind.Class),
            new vscode_1.CompletionItem("Text", vscode_1.CompletionItemKind.Text),
            new vscode_1.CompletionItem("Method", vscode_1.CompletionItemKind.Method),
            new vscode_1.CompletionItem("Function", vscode_1.CompletionItemKind.Function),
            new vscode_1.CompletionItem("Constructor", vscode_1.CompletionItemKind.Constructor),
            new vscode_1.CompletionItem("Field", vscode_1.CompletionItemKind.Field),
            new vscode_1.CompletionItem("Variable", vscode_1.CompletionItemKind.Variable),
            new vscode_1.CompletionItem("Class", vscode_1.CompletionItemKind.Class),
            new vscode_1.CompletionItem("Interface", vscode_1.CompletionItemKind.Interface),
            new vscode_1.CompletionItem("Module", vscode_1.CompletionItemKind.Module),
            new vscode_1.CompletionItem("Property", vscode_1.CompletionItemKind.Property),
            new vscode_1.CompletionItem("Unit", vscode_1.CompletionItemKind.Unit),
            new vscode_1.CompletionItem("Value", vscode_1.CompletionItemKind.Value),
            new vscode_1.CompletionItem("Enum", vscode_1.CompletionItemKind.Enum),
            new vscode_1.CompletionItem("Keyword", vscode_1.CompletionItemKind.Keyword),
            new vscode_1.CompletionItem("Snippet", vscode_1.CompletionItemKind.Snippet),
            new vscode_1.CompletionItem("Color", vscode_1.CompletionItemKind.Color),
            new vscode_1.CompletionItem("Reference", vscode_1.CompletionItemKind.Reference),
            new vscode_1.CompletionItem("File", vscode_1.CompletionItemKind.File),
            new vscode_1.CompletionItem("Folder", vscode_1.CompletionItemKind.Folder),
            new vscode_1.CompletionItem("EnumMember", vscode_1.CompletionItemKind.EnumMember),
            new vscode_1.CompletionItem("Constant", vscode_1.CompletionItemKind.Constant),
            new vscode_1.CompletionItem("Struct", vscode_1.CompletionItemKind.Struct),
            new vscode_1.CompletionItem("Event", vscode_1.CompletionItemKind.Event),
            new vscode_1.CompletionItem("Operator", vscode_1.CompletionItemKind.Operator),
            new vscode_1.CompletionItem("TypeParameter", vscode_1.CompletionItemKind.TypeParameter)
        ];
    }
}
exports.AbstractCompletionItemProvider = AbstractCompletionItemProvider;
class Suggestion {
    constructor(sectionName, filterTextPrefix) {
        this.sectionName = sectionName;
        this.filterText = filterTextPrefix + sectionName;
    }
    static from(sectionName, triggerCharacter, filterTextPrefix) {
        if (triggerCharacter === ':' && !sectionName.startsWith(':')) {
            return null;
        }
        else {
            return new Suggestion(sectionName, filterTextPrefix);
        }
    }
}
exports.Suggestion = Suggestion;
class SuggestionDetails {
    constructor(label, detail, documentation, kind) {
        this.label = label;
        this.detail = detail;
        this.documentation = documentation;
        this.kind = kind;
    }
}
exports.SuggestionDetails = SuggestionDetails;
//# sourceMappingURL=AbstractCompletionItemProvider.js.map