/* --------------------------------------------------------------------------------------------
 * Copyright (c) Jan Dolejsi. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for license information.
 * ------------------------------------------------------------------------------------------ */
'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
exports.DiscreteEffectCompletionItemProvider = void 0;
const vscode_1 = require("vscode");
const pddl_workspace_1 = require("pddl-workspace");
const pddl_workspace_2 = require("pddl-workspace");
const AbstractCompletionItemProvider_1 = require("./AbstractCompletionItemProvider");
const Delegate_1 = require("./Delegate");
const DomainCompletionItemProvider_1 = require("./DomainCompletionItemProvider");
class DiscreteEffectCompletionItemProvider extends AbstractCompletionItemProvider_1.AbstractCompletionItemProvider {
    constructor() {
        super();
        const discreteEffectHint = 'Use this either in instantaneous `:action`\'s `:effect`, or in `:durative-action`\'s `(at start ...)` or `(at end ...)` effect.';
        const requiresFluents = DomainCompletionItemProvider_1.requires([':fluents']);
        this.addSuggestionDocumentation('not', 'Assigns `false` to a predicate', new vscode_1.MarkdownString('Makes predicate false:')
            .appendCodeblock('(not (at ?location))', pddl_workspace_1.PDDL)
            .appendMarkdown(discreteEffectHint), vscode_1.CompletionItemKind.Function);
        this.addSuggestionDocumentation('assign', 'Numeric assign effect', new vscode_1.MarkdownString('Assigns value to the function, for example:')
            .appendCodeblock('(assign (function1) 3.14)', pddl_workspace_1.PDDL)
            .appendMarkdown(discreteEffectHint)
            .appendMarkdown(requiresFluents), vscode_1.CompletionItemKind.Method);
        this.addSuggestionDocumentation('increase', 'Discrete numeric increase effect', new vscode_1.MarkdownString('For example to increment a function value by `3.14`, use')
            .appendCodeblock('(increase (function1) 3.14)', pddl_workspace_1.PDDL)
            .appendMarkdown(discreteEffectHint)
            .appendMarkdown(requiresFluents), vscode_1.CompletionItemKind.Method);
        this.addSuggestionDocumentation('decrease', 'Discrete numeric decrease effect', new vscode_1.MarkdownString('For example to decrement a function value by `3.14`, use')
            .appendCodeblock('(decrease (function1) 3.14)', pddl_workspace_1.PDDL)
            .appendMarkdown(discreteEffectHint)
            .appendMarkdown(requiresFluents), vscode_1.CompletionItemKind.Method);
        this.addSuggestionDocumentation('forall', 'For all effect', new vscode_1.MarkdownString('Effect that shall be applied to all objects of specified type. For example:')
            .appendCodeblock('(forall (?p - product) (sold_out ?p))', pddl_workspace_1.PDDL), vscode_1.CompletionItemKind.TypeParameter);
        this.addSuggestionDocumentation('when', 'Conditional effect', new vscode_1.MarkdownString('Effect that shall only be applied when a condition is met. For example:')
            .appendCodeblock('(when (at ?location) (not (at ?location)))', pddl_workspace_1.PDDL)
            .appendMarkdown(discreteEffectHint)
            .appendMarkdown(DomainCompletionItemProvider_1.requires([':conditional-effects'])), vscode_1.CompletionItemKind.Method);
    }
    static inside(currentNode) {
        return pddl_workspace_2.ModelHierarchy.isInsideEffect(currentNode)
            && (pddl_workspace_2.ModelHierarchy.isInsideActionOrEvent(currentNode)
                || pddl_workspace_2.ModelHierarchy.isInsideDurativeActionDiscreteEffect(currentNode));
    }
    provide(domainInfo, context, range) {
        if (context.triggerKind !== vscode_1.CompletionTriggerKind.Invoke && context.triggerCharacter !== '(') {
            return [];
        }
        const functions = domainInfo.getFunctions();
        const functionNamesCsv = Delegate_1.Delegate.toTypeLessNamesCsv(functions);
        const predicates = domainInfo.getPredicates();
        const predicateNamesCsv = Delegate_1.Delegate.toTypeLessNamesCsv(predicates);
        return [
            this.createSnippetCompletionItem(AbstractCompletionItemProvider_1.Suggestion.from("not", context.triggerCharacter, '('), "(not (" + DomainCompletionItemProvider_1.toSelection(1, predicateNamesCsv, "new_predicate") + "))$0", range, context, 0),
            this.createSnippetCompletionItem(AbstractCompletionItemProvider_1.Suggestion.from("assign", context.triggerCharacter, '('), "(assign (" + DomainCompletionItemProvider_1.toSelection(1, functionNamesCsv, "new_function") + ") ${2:0})$0", range, context, 1),
            this.createSnippetCompletionItem(AbstractCompletionItemProvider_1.Suggestion.from("increase", context.triggerCharacter, '('), "(increase (" + DomainCompletionItemProvider_1.toSelection(1, functionNamesCsv, "new_function") + ") ${2:1})$0", range, context, 2),
            this.createSnippetCompletionItem(AbstractCompletionItemProvider_1.Suggestion.from("decrease", context.triggerCharacter, '('), "(decrease (" + DomainCompletionItemProvider_1.toSelection(1, functionNamesCsv, "new_function") + ") ${2:1})$0", range, context, 3),
            this.createSnippetCompletionItem(AbstractCompletionItemProvider_1.Suggestion.from("forall", context.triggerCharacter, '('), "(forall ($1) $2)$0", range, context, 4),
            this.createSnippetCompletionItem(AbstractCompletionItemProvider_1.Suggestion.from("when", context.triggerCharacter, '('), "(when ${1:condition} ${2:effect})$0", range, context, 5)
        ];
    }
}
exports.DiscreteEffectCompletionItemProvider = DiscreteEffectCompletionItemProvider;
//# sourceMappingURL=DiscreteEffectCompletionItemProvider.js.map