/* --------------------------------------------------------------------------------------------
 * Copyright (c) Jan Dolejsi. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for license information.
 * ------------------------------------------------------------------------------------------ */
'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
exports.toSelection = exports.requires = exports.DomainCompletionItemProvider = void 0;
const vscode_1 = require("vscode");
const pddl_workspace_1 = require("pddl-workspace");
const pddl_workspace_2 = require("pddl-workspace");
const utils_1 = require("../utils");
const AbstractCompletionItemProvider_1 = require("./AbstractCompletionItemProvider");
const DiscreteEffectCompletionItemProvider_1 = require("./DiscreteEffectCompletionItemProvider");
const ContinuousEffectCompletionItemProvider_1 = require("./ContinuousEffectCompletionItemProvider");
const DurativeActionEffectCompletionItemProvider_1 = require("./DurativeActionEffectCompletionItemProvider");
const DurativeActionConditionCompletionItemProvider_1 = require("./DurativeActionConditionCompletionItemProvider");
const pddl_workspace_3 = require("pddl-workspace");
const pddl_workspace_4 = require("pddl-workspace");
class DomainCompletionItemProvider extends AbstractCompletionItemProvider_1.AbstractCompletionItemProvider {
    constructor() {
        super();
        this.DURATION_SNIPPET = ":duration ${2|(= ?duration 1),(> ?duration 0),(<= ?duration 10),(and (>= ?duration 1)(<= ?duration 2))|}";
        this.addRequirementsDocumentation();
        this.addSuggestionDocumentation(':types', 'Types', new vscode_1.MarkdownString('Types of objects and their hierarchy. Example:').appendCodeblock('car - vehicle', pddl_workspace_1.PDDL));
        this.addSuggestionDocumentation(':constants', 'Constants', new vscode_1.MarkdownString('Constant objects that will be part of all problems defined for this domain in addition to the objects defined in the `:objects` section.'));
        this.addSuggestionDocumentation(':predicates', 'Predicates', 'Predicates are things that are either true or false.');
        this.addSuggestionDocumentation(':functions', 'Functions', 'Functions are used to define numeric values.');
        this.addConstraintsDocumentation();
        this.addSuggestionDocumentation(':derived', 'Derived predicate/function', new vscode_1.MarkdownString('Derived predicate/function can be defined to simplify action declaration. Example derived predicate:').appendCodeblock('(:derived (p_and_q) \n\t(and (p) (q))\n)', pddl_workspace_1.PDDL).appendText('Example derived function: ').appendCodeblock('(:derived (c) (+ (a) (b))', pddl_workspace_1.PDDL));
        this.addSuggestionDocumentation(':action', 'Instantaneous action', new vscode_1.MarkdownString('Actions that change state of the world. Example:').appendCodeblock('(:action action_name\n\t:parameters (?t - type1)\n\t:precondition (and (p ?t))\n\t:effect (and (q ?t))\n)', pddl_workspace_1.PDDL));
        this.addSuggestionDocumentation(':durative-action', 'Durative action', 'Actions that change the state of the world when they start, then they last for a defined duration period, while changing the world continuously and finally change the state when they end.');
        this.addSuggestionDocumentation(':process', 'PDDL+ Process', new vscode_1.MarkdownString('Process is activated and continues running when its condition is met. It may only have continuous effects. Example:').appendCodeblock(['(:process HEAT',
            '    :parameters (?r - room)',
            '    :precondition (and',
            '        (too_cold ?r)',
            '        (< (temperature ?r) 22)',
            '    )',
            '    :effect (and',
            '        (increase (temperature ?b) (* #t 3))',
            '    )',
            ')'].join('\n'), pddl_workspace_1.PDDL).appendMarkdown('Note that `:process` and `:event` require the `:time` requirement.'), vscode_1.CompletionItemKind.Struct);
        this.addSuggestionDocumentation(':event', 'PDDL+ Effect', new vscode_1.MarkdownString('Effect is triggered when its condition is met. It may only have continuous effects. Example:').appendCodeblock(['(:event BOUNCE',
            '    :parameters (?b - ball)',
            '    :precondition (and',
            '        (not (held ?b))',
            '        (<= (distance-to-floor ?b) 0)',
            '    )',
            '    :effect (and',
            '        (assign (velocity ?b) (* -0.8 (velocity ?b)))',
            '    )',
            ')'].join('\n'), pddl_workspace_1.PDDL).appendMarkdown('Note that `:process` and `:event` require the `:time` requirement.'), vscode_1.CompletionItemKind.Event);
        /* ACTION KEYWORDS */
        this.addSuggestionDocumentation(':parameters', 'Action parameters', new vscode_1.MarkdownString('Parameters such as:').appendCodeblock(':parameters (?v - vehicle ?from ?to - place)'));
        this.addSuggestionDocumentation(':precondition', 'Instantaneous action precondition', '');
        this.addSuggestionDocumentation(':effect', 'Action effect', '');
        /* DURATIVE ACTION KEYWORDS */
        this.addSuggestionDocumentation(':duration', 'Durative action duration', new vscode_1.MarkdownString('Examples:').appendCodeblock(':duration (= ?duration 1)\n:duration (and (>= ?duration (min_duration))(<= ?duration (max_duration)))'));
        this.addSuggestionDocumentation(':condition', 'Durative action condition', '');
    }
    async provide(document, domainInfo, position, context) {
        let currentNode = domainInfo.syntaxTree.getNodeAt(document.offsetAt(position));
        if (currentNode.isType(pddl_workspace_2.parser.PddlTokenType.Comment)) {
            return [];
        }
        if (this.insideDefine(domainInfo, currentNode, context)) {
            // inside the 'define' bracket
            if (context.triggerCharacter) {
                currentNode = currentNode.expand();
            }
            const supportedSectionsHere = pddl_workspace_2.parser.PddlStructure.getSupportedSectionsHere(currentNode, currentNode, pddl_workspace_2.parser.PddlTokenType.OpenBracketOperator, pddl_workspace_2.parser.PddlStructure.PDDL_DOMAIN_SECTIONS, pddl_workspace_2.parser.PddlStructure.PDDL_DOMAIN_STRUCTURES);
            const range = context.triggerCharacter && ['(', ':'].includes(context.triggerCharacter)
                ? utils_1.nodeToRange(document, currentNode) : null;
            const suggestions = supportedSectionsHere
                .map(s => AbstractCompletionItemProvider_1.Suggestion.from(s, context.triggerCharacter, '('))
                .filter(s => !!s).map(s => s);
            return suggestions
                .map((suggestion, index) => this.createDefineCompletionItem(currentNode, suggestion, range, context, index))
                .filter(item => !!item).map(item => item); // filter out nulls
        }
        else if (this.insideRequirements(domainInfo, currentNode, context)) {
            return this.createRequirementsCompletionItems(document, currentNode, context);
        }
        else if (this.insideAction(currentNode)) {
            const nearestPrecedingKeyword = pddl_workspace_2.parser.PddlStructure.getPrecedingKeywordOrSelf(currentNode);
            const supportedSectionsHere = pddl_workspace_2.parser.PddlStructure.getSupportedSectionsHere(nearestPrecedingKeyword, currentNode, pddl_workspace_2.parser.PddlTokenType.Keyword, pddl_workspace_2.parser.PddlStructure.PDDL_ACTION_SECTIONS, []);
            const range = context.triggerCharacter && ['(', ':'].includes(context.triggerCharacter)
                ? utils_1.nodeToRange(document, currentNode) : null;
            const suggestions = supportedSectionsHere
                .map(s => AbstractCompletionItemProvider_1.Suggestion.from(s, context.triggerCharacter, ''))
                .filter(s => !!s).map(s => s);
            return suggestions
                .map((suggestion, index) => this.createActionCompletionItem(currentNode, suggestion, range, context, index))
                .filter(item => !!item).map(item => item); // filter out nulls
        }
        else if (this.insideDurativeAction(currentNode)) {
            const nearestPrecedingKeyword = pddl_workspace_2.parser.PddlStructure.getPrecedingKeywordOrSelf(currentNode);
            const supportedSectionsHere = pddl_workspace_2.parser.PddlStructure.getSupportedSectionsHere(nearestPrecedingKeyword, currentNode, pddl_workspace_2.parser.PddlTokenType.Keyword, pddl_workspace_2.parser.PddlStructure.PDDL_DURATIVE_ACTION_SECTIONS, []);
            const range = context.triggerCharacter && ['(', ':'].includes(context.triggerCharacter)
                ? utils_1.nodeToRange(document, currentNode) : null;
            const suggestions = supportedSectionsHere
                .map(s => AbstractCompletionItemProvider_1.Suggestion.from(s, context.triggerCharacter, ''))
                .filter(s => !!s).map(s => s);
            return suggestions
                .map((suggestion, index) => this.createDurativeActionCompletionItem(currentNode, suggestion, range, context, index))
                .filter(item => !!item).map(item => item); // filter out nulls
        }
        else if (pddl_workspace_4.ModelHierarchy.isInsideEffect(currentNode)) {
            const completions = [];
            const range = context.triggerCharacter === '(' ? utils_1.nodeToRange(document, currentNode) : null;
            if (DurativeActionEffectCompletionItemProvider_1.DurativeActionEffectCompletionItemProvider.inside(currentNode)) {
                completions.push(...new DurativeActionEffectCompletionItemProvider_1.DurativeActionEffectCompletionItemProvider().provide(domainInfo, context, range));
            }
            if (DiscreteEffectCompletionItemProvider_1.DiscreteEffectCompletionItemProvider.inside(currentNode)) {
                completions.push(...new DiscreteEffectCompletionItemProvider_1.DiscreteEffectCompletionItemProvider().provide(domainInfo, context, range));
            }
            if (ContinuousEffectCompletionItemProvider_1.ContinuousEffectCompletionItemProvider.inside(currentNode)) {
                completions.push(...await new ContinuousEffectCompletionItemProvider_1.ContinuousEffectCompletionItemProvider().provide(domainInfo, context, range));
            }
            // completions.push(...this.addIconGallery());
            return completions
                .filter(c => !!c).map(c => c);
        }
        else if (DurativeActionConditionCompletionItemProvider_1.DurativeActionConditionCompletionItemProvider.inside(currentNode)) {
            const range = context.triggerCharacter === '(' ? utils_1.nodeToRange(document, currentNode) : null;
            return new DurativeActionConditionCompletionItemProvider_1.DurativeActionConditionCompletionItemProvider()
                .provide(domainInfo, context, range)
                .filter(c => !!c).map(c => c);
        }
        if (context.triggerCharacter === '?') {
            const scopes = currentNode.findAllParametrisableScopes();
            const range = utils_1.nodeToRange(document, currentNode);
            const parameterNamesFromAllScopes = scopes
                .map(s => s.getParameterDefinition())
                .filter(paramNode => !!paramNode).map(paramNode => paramNode)
                .map(paramNode => this.getParameterNames(paramNode));
            return pddl_workspace_3.utils.Util.flatMap(parameterNamesFromAllScopes)
                .map((paramName, index) => this.createSnippetCompletionItem(AbstractCompletionItemProvider_1.Suggestion.from(paramName, context.triggerCharacter, ''), paramName, range, context, index))
                .filter(c => !!c).map(c => c);
        }
        return [];
    }
    getParameterNames(parametersNode) {
        return parametersNode.getChildrenOfType(pddl_workspace_2.parser.PddlTokenType.Parameter, /.*/)
            .map(n => n.getText());
    }
    insideAction(currentNode) {
        const pattern = /^\(\s*:action$/i;
        return this.insideScope(currentNode, pattern);
    }
    insideDurativeAction(currentNode) {
        const pattern = /^\(\s*:durative-action$/i;
        return this.insideScope(currentNode, pattern);
    }
    insideScope(currentNode, pattern) {
        const parentScope = currentNode.expand();
        return parentScope.isType(pddl_workspace_2.parser.PddlTokenType.OpenBracketOperator) && parentScope.getToken().tokenText.match(pattern) !== null;
    }
    createDefineCompletionItem(_currentNode, suggestion, range, context, index) {
        if (!suggestion) {
            return null;
        }
        switch (suggestion.sectionName) {
            case pddl_workspace_2.parser.PddlStructure.DOMAIN:
                return this.createSnippetCompletionItem(suggestion, "(domain ${1:domain_name})", range, context, index);
            case pddl_workspace_2.parser.PddlStructure.REQUIREMENTS:
                return this.createSnippetCompletionItem(suggestion, "(:requirements :strips $0)", range, context, index);
            case pddl_workspace_2.parser.PddlStructure.TYPES:
            case pddl_workspace_2.parser.PddlStructure.CONSTANTS:
            case pddl_workspace_2.parser.PddlStructure.PREDICATES:
            case pddl_workspace_2.parser.PddlStructure.FUNCTIONS:
                return this.createSnippetCompletionItem(suggestion, "(" + suggestion.sectionName + "\n\t$0\n)", range, context, index);
            case pddl_workspace_2.parser.PddlStructure.CONSTRAINTS:
                return this.createSnippetCompletionItem(suggestion, "(" + suggestion.sectionName + " (and\n\t$0\n))", range, context, index);
            case pddl_workspace_2.parser.PddlStructure.ACTION:
                return this.createSnippetCompletionItem(suggestion, ["(:action ${1:action_name}",
                    "    :parameters ($0)",
                    "    :precondition (and )",
                    "    :effect (and )",
                    ")",
                    ""
                ].join('\n'), range, context, index);
            case pddl_workspace_2.parser.PddlStructure.DURATIVE_ACTION:
                return this.createSnippetCompletionItem(suggestion, [
                    "(:durative-action ${1:action_name}",
                    "    :parameters ($0)",
                    "    " + this.DURATION_SNIPPET,
                    "    :condition (and ",
                    "        (at start (and ",
                    "        ))",
                    "        (over all (and ",
                    "        ))",
                    "        (at end (and ",
                    "        ))",
                    "    )",
                    "    :effect (and ",
                    "        (at start (and ",
                    "        ))",
                    "        (at end (and ",
                    "        ))",
                    "    )",
                    ")",
                    ""
                ].join('\n'), range, context, index);
            case pddl_workspace_2.parser.PddlStructure.PROCESS:
                return this.createSnippetCompletionItem(suggestion, ["(:process ${1:process_name}",
                    "    :parameters ($0)",
                    "    :precondition (and",
                    "        ; activation condition",
                    "    )",
                    "    :effect (and",
                    "        ; continuous effect(s)",
                    "    )",
                    ")",
                    ""
                ].join('\n'), range, context, index);
            case pddl_workspace_2.parser.PddlStructure.EVENT:
                return this.createSnippetCompletionItem(suggestion, ["(:event ${1:event_name}",
                    "    :parameters ($0)",
                    "    :precondition (and",
                    "        ; trigger condition",
                    "    )",
                    "    :effect (and",
                    "        ; discrete effect(s)",
                    "    )",
                    ")",
                    ""
                ].join('\n'), range, context, index);
            default:
                return null;
        }
    }
    createActionCompletionItem(_currentNode, suggestion, range, context, index) {
        switch (suggestion.sectionName) {
            case pddl_workspace_2.parser.PddlStructure.PARAMETERS:
                return this.createParametersCompletionItem(suggestion, range, context, index);
            case pddl_workspace_2.parser.PddlStructure.PRECONDITION:
            case pddl_workspace_2.parser.PddlStructure.EFFECT:
                return this.createSnippetCompletionItem(suggestion, suggestion.sectionName + " (and \n\t$0\n)", range, context, index);
            default:
                return null;
        }
    }
    createParametersCompletionItem(suggestion, range, context, index) {
        return this.createSnippetCompletionItem(suggestion, ":parameters ($0)", range, context, index);
    }
    createDurativeActionCompletionItem(_currentNode, suggestion, range, context, index) {
        switch (suggestion.sectionName) {
            case pddl_workspace_2.parser.PddlStructure.PARAMETERS:
                return this.createParametersCompletionItem(suggestion, range, context, index);
            case pddl_workspace_2.parser.PddlStructure.DURATION:
                return this.createSnippetCompletionItem(suggestion, this.DURATION_SNIPPET, range, context, index);
            case pddl_workspace_2.parser.PddlStructure.CONDITION:
            case pddl_workspace_2.parser.PddlStructure.EFFECT:
                return this.createSnippetCompletionItem(suggestion, suggestion.sectionName + " (and \n\t$0\n)", range, context, index);
            default:
                return null;
        }
    }
}
exports.DomainCompletionItemProvider = DomainCompletionItemProvider;
function requires(requirements) {
    const requirementsCsv = requirements.map(r => '`' + r + '`').join(', ');
    return `\n\nThis language feature requires ${requirementsCsv}.`;
}
exports.requires = requires;
function toSelection(tabstop, optionsCsv, orDefault) {
    if (optionsCsv.length) {
        return "${" + tabstop + "|" + optionsCsv + "|}";
    }
    else {
        return "${" + tabstop + ":" + orDefault + "}";
    }
}
exports.toSelection = toSelection;
//# sourceMappingURL=DomainCompletionItemProvider.js.map