/* --------------------------------------------------------------------------------------------
 * Copyright (c) Jan Dolejsi. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for license information.
 * ------------------------------------------------------------------------------------------ */
'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
exports.ProblemInitDelegate = void 0;
const vscode_1 = require("vscode");
const ContextDelegate_1 = require("./ContextDelegate");
const Delegate_1 = require("./Delegate");
class ProblemInitDelegate extends ContextDelegate_1.ContextDelegate {
    constructor(items, context) {
        super(context);
        this.completions = items;
    }
    createProblemInitCompletionItems(problemFileInfo, domainFiles) {
        if (domainFiles.length === 1) {
            // there is a single associated domain file
            this.createSymmetricInit(problemFileInfo, domainFiles[0]);
            this.createTimedInitialLiteral(problemFileInfo, domainFiles[0]);
            this.createTimedInitialNegativeLiteral(problemFileInfo, domainFiles[0]);
            this.createTimedInitialFluent(problemFileInfo, domainFiles[0]);
        }
    }
    createSymmetricInit(problemFileInfo, domainFile) {
        const allTypeObjects = domainFile.getConstants().merge(problemFileInfo.getObjectsTypeMap());
        const symmetricPredicates = this.getSymmetricPredicates(domainFile);
        const symmetricFunctions = this.getSymmetricFunctions(domainFile);
        if (symmetricPredicates.length && symmetricFunctions.length) {
            this.createSymmetricPredicateAndFunctionInitItem(symmetricPredicates, symmetricFunctions, domainFile, allTypeObjects);
        }
        if (symmetricPredicates.length) {
            this.createSymmetricPredicateInitItem(symmetricPredicates, domainFile, allTypeObjects);
            symmetricPredicates.forEach(predicate1 => {
                this.createSequencePredicateInitItem(predicate1, allTypeObjects);
            });
        }
        if (symmetricFunctions.length) {
            this.createSymmetricFunctionInitItem(symmetricFunctions, domainFile, allTypeObjects);
        }
    }
    createSymmetricPredicateAndFunctionInitItem(symmetricPredicates, symmetricFunctions, domainFile, allTypeObjects) {
        const typesInvolved = this.getTypesInvolved(symmetricPredicates.concat(symmetricFunctions), domainFile);
        const objectsDefined = this.getObjects(allTypeObjects, typesInvolved);
        const objectNames = objectsDefined
            .join(',');
        if (!objectsDefined.length) {
            return;
        }
        const item = new vscode_1.CompletionItem('Initialize a symmetric predicate and function', vscode_1.CompletionItemKind.Snippet);
        item.insertText = new vscode_1.SnippetString("(${1|" + Delegate_1.Delegate.toNamesCsv(symmetricPredicates) + "|} ${2|" + objectNames + "|} ${3|" + objectNames + "|}) (${1} ${3} ${2})\n" +
            "(= (${4|" + Delegate_1.Delegate.toNamesCsv(symmetricFunctions) + "|} ${2|" + objectNames + "|} ${3|" + objectNames + "|}) ${5:1}) (= (${4} ${3} ${2}) ${5})");
        item.documentation = new vscode_1.MarkdownString()
            .appendText("Inserts a predicate and function initialization for predicates and functions with two parameters of the same type.")
            .appendCodeblock("(road A B) (road B A)\n(= (distance A B) 1) (= (distance B A) 1)", "PDDL")
            .appendMarkdown("Use the `Tab` and `Enter` keys on your keyboard to cycle through the selection of the predicate, objects, function and the function value.");
        this.completions.push(item);
    }
    createSymmetricPredicateInitItem(symmetricPredicates, domainFile, allTypeObjects) {
        const typesInvolved = this.getTypesInvolved(symmetricPredicates, domainFile);
        const objectsDefined = this.getObjects(allTypeObjects, typesInvolved);
        const objectNames = objectsDefined
            .join(',');
        if (!objectsDefined.length) {
            return;
        }
        const symmetricPredicateNames = Delegate_1.Delegate.toNamesCsv(symmetricPredicates);
        const item = new vscode_1.CompletionItem('Initialize a symmetric predicate', vscode_1.CompletionItemKind.Snippet);
        item.insertText = new vscode_1.SnippetString("(${1|" + symmetricPredicateNames + "|} ${2|" + objectNames + "|} ${3|" + objectNames + "|}) (${1} ${3} ${2})");
        item.documentation = new vscode_1.MarkdownString()
            .appendText("Inserts a predicate initialization for predicates with two parameters of the same type.")
            .appendCodeblock("(road A B) (road B A)", "pddl")
            .appendMarkdown("Use the `Tab` and `Enter` keys on your keyboard to cycle through the selection of the predicate and objects.");
        this.completions.push(item);
    }
    createSymmetricFunctionInitItem(symmetricFunctions, domainFile, allTypeObjects) {
        const typesInvolved = this.getTypesInvolved(symmetricFunctions, domainFile);
        const objectsDefined = this.getObjects(allTypeObjects, typesInvolved);
        const objectNames = objectsDefined
            .join(',');
        if (!objectsDefined.length) {
            return;
        }
        const symmetricFunctionNames = Delegate_1.Delegate.toNamesCsv(symmetricFunctions);
        const item = new vscode_1.CompletionItem('Initialize a symmetric function', vscode_1.CompletionItemKind.Snippet);
        item.insertText = new vscode_1.SnippetString("(= (${1|" + symmetricFunctionNames + "|} ${2|" + objectNames + "|} ${3|" + objectNames + "|}) ${4:1}) (= (${1} ${3} ${2}) ${4})");
        item.documentation = new vscode_1.MarkdownString()
            .appendText("Inserts a function initialization for functions with two parameters of the same type.")
            .appendCodeblock("(= (distance A B) 1) (= (distance B A) 1)", "pddl")
            .appendMarkdown("Use the `Tab` and `Enter` keys on your keyboard to cycle through the selection of the function, objects and the function value.");
        this.completions.push(item);
    }
    createSequencePredicateInitItem(symmetricPredicate, allTypeObjects) {
        // note for the sequence case we do not consider type inheritance
        const typeInvolved = symmetricPredicate.parameters[0].type;
        const objects = this.getObjects(allTypeObjects, [typeInvolved]);
        if (objects.length < 2) {
            return;
        }
        let textToInsert = '';
        for (let i = 0; i < objects.length - 1; i++) {
            textToInsert += `(${symmetricPredicate.name} ${objects[i]} ${objects[i + 1]})\n`;
        }
        const item = new vscode_1.CompletionItem('Initialize a sequence for ' + symmetricPredicate.declaredNameWithoutTypes, vscode_1.CompletionItemKind.Snippet);
        item.insertText = textToInsert;
        item.documentation = new vscode_1.MarkdownString()
            .appendText("Inserts a predicate initialization for predicates with two parameters of the same type with a sequence of object combinations.")
            .appendCodeblock(textToInsert, "pddl");
        this.completions.push(item);
    }
    createTimedInitialLiteral(_problemFileInfo, domainFile) {
        const predicates = domainFile.getPredicates();
        if (!predicates.length) {
            return;
        }
        const namesCsv = Delegate_1.Delegate.toTypeLessNamesCsv(predicates);
        const item = new vscode_1.CompletionItem('at <time> (predicate1)', vscode_1.CompletionItemKind.Snippet);
        item.filterText = 'at ';
        item.insertText = new vscode_1.SnippetString(this.enclose("at ${1:1.0} (${2|" + namesCsv + "|})"));
        item.detail = 'Timed initial literal';
        item.documentation = new vscode_1.MarkdownString()
            .appendMarkdown("Defines that selected predicated changes value to `true` at specified time.")
            .appendCodeblock("(at 42 (predicate1))", "PDDL")
            .appendMarkdown("Use the `Tab` and `Enter` keys on your keyboard to cycle through the snippet inputs.");
        this.completions.push(item);
    }
    createTimedInitialNegativeLiteral(_problemFileInfo, domainFile) {
        const predicates = domainFile.getPredicates();
        if (!predicates.length) {
            return;
        }
        const namesCsv = Delegate_1.Delegate.toTypeLessNamesCsv(predicates);
        const item = new vscode_1.CompletionItem('at <time> (not (predicate1))', vscode_1.CompletionItemKind.Snippet);
        item.filterText = 'at ';
        item.insertText = new vscode_1.SnippetString(this.enclose("at ${1:1.0} (not (${2|" + namesCsv + "|}))"));
        item.detail = 'Timed initial literal (negative)';
        item.documentation = new vscode_1.MarkdownString()
            .appendMarkdown("Defines that selected predicated changes value to `false` at specified time.")
            .appendCodeblock("(at 42 (not (predicate1)))", "PDDL")
            .appendMarkdown("Use the `Tab` and `Enter` keys on your keyboard to cycle through the snippet inputs.");
        this.completions.push(item);
    }
    createTimedInitialFluent(_problemFileInfo, domainFile) {
        const functions = domainFile.getFunctions();
        if (!functions.length) {
            return;
        }
        const item = new vscode_1.CompletionItem('at <time> (= (function1) <value>)', vscode_1.CompletionItemKind.Snippet);
        item.filterText = 'at ';
        item.insertText = new vscode_1.SnippetString(this.enclose("at ${1:1.0} (= (${2|" + Delegate_1.Delegate.toTypeLessNamesCsv(functions) + "|}) ${3:42})"));
        item.detail = 'Timed initial fluent';
        item.documentation = new vscode_1.MarkdownString()
            .appendText("Defines that selected function changes value at specified time.")
            .appendCodeblock("(at 42 (= (function1) 1.0))", "PDDL")
            .appendMarkdown("Use the `Tab` and `Enter` keys on your keyboard to cycle through the snippet inputs.");
        this.completions.push(item);
    }
}
exports.ProblemInitDelegate = ProblemInitDelegate;
//# sourceMappingURL=ProblemInitDelegate.js.map