/* --------------------------------------------------------------------------------------------
 * Copyright (c) Jan Dolejsi 2021. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for license information.
 * ------------------------------------------------------------------------------------------ */
/* Warning: This file is shared by source-code between the PDDL extension and at least one other extension. */
'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
exports.LongRunningPlannerProvider = exports.PlanningServiceExecution = void 0;
const path = require("path");
const vscode_1 = require("vscode");
const pddl_planning_service_client_1 = require("pddl-planning-service-client");
const pddl_workspace_1 = require("pddl-workspace");
const utils_1 = require("../utils");
/** Tracks a long-running planning service execution. */
class PlanningServiceExecution {
    constructor(servicePath, wasAlreadyRunning, execution) {
        this.servicePath = servicePath;
        this.wasAlreadyRunning = wasAlreadyRunning;
        this.execution = execution;
    }
    static alreadyRunning(servicePath) {
        return new PlanningServiceExecution(servicePath, true);
    }
    static justStarted(servicePath, execution) {
        return new PlanningServiceExecution(servicePath, false, execution);
    }
}
exports.PlanningServiceExecution = PlanningServiceExecution;
class LongRunningPlannerProvider {
    constructor(disposables) {
        this.planningServiceExecutions = new Map();
        vscode_1.tasks.onDidEndTask(this.handleTaskEnded, this, disposables);
    }
    handleTaskEnded(event) {
        const taskExecution = [...this.planningServiceExecutions.values()].find(e => e.execution === event.execution);
        if (taskExecution) {
            // yes, a relevant task ended. Clean-up.
            this.planningServiceExecutions.delete(taskExecution.servicePath);
        }
    }
    /**
     * Checks if the service was started.
     * @param configuration planner configuration to start
     * @returns true if the service was started by this provider
     */
    isServiceWasStarted(configuration) {
        return configuration.path !== undefined && this.planningServiceExecutions.has(configuration.path);
    }
    /**
     * Tests whether host name is local (this machine)
     * @param hostname host name from the configured url of the planning service
     * @returns true if the service is running locally
     */
    static isLocal(hostname) {
        return ['localhost', '127.0.0.1'].includes(hostname);
    }
    /**
     * Callback invoked when the planner fails. Get troubleshooting options.
     * @param failedPlanner planner that failed
     * @param reason reason to fail (most likely an `Error`)
     * @returns list of troubleshooting options
     */
    async troubleshoot(failedPlanner, reason) {
        let troubleShootingInfo = '';
        const troubleShootings = new Map();
        if (pddl_planning_service_client_1.instanceOfHttpConnectionRefusedError(reason)) {
            const configuration = failedPlanner.providerConfiguration.configuration;
            troubleShootingInfo += `Service ${configuration.url} cannot be reached.\n`;
            const isLocal = LongRunningPlannerProvider.isLocal(reason.address);
            const serviceExePath = configuration.path;
            if (isLocal && serviceExePath !== undefined) {
                const fileName = path.basename(serviceExePath);
                if (await utils_1.fileOrFolderExists(vscode_1.Uri.file(serviceExePath))) {
                    if (this.isServiceWasStarted(configuration)) {
                        troubleShootingInfo += `Service appears to be running, but not responding. Click 'Re-start the service'.\n`;
                        troubleShootings.set(`Re-start the service`, async () => this.startService(configuration));
                    }
                    else {
                        troubleShootingInfo += `Service does not appear to be running. Click 'Start the service' to execute '${fileName}'.\n`;
                        troubleShootings.set(`Start the service`, async () => this.startService(configuration));
                    }
                }
                else {
                    troubleShootingInfo += `Service does not appear to be running. The configured server '${fileName}' is not a valid file.\n`;
                }
            }
        }
        return {
            info: troubleShootingInfo,
            options: troubleShootings
        };
    }
    /**
     * Suggest to the user to start the planning service.
     * @param configuration planning configuration for a service to start
     */
    async suggestStartService(configuration) {
        const serviceExePath = configuration.path;
        if (!serviceExePath) {
            throw new Error(`Expected planner configuration with a 'path' specified`);
        }
        const fileName = path.basename(serviceExePath);
        const startService = 'Start the service';
        const answer = await vscode_1.window.showWarningMessage(`Service at '${configuration.url}' does not appear to be running. Click '${startService}' to execute '${fileName}'.\n`, startService);
        if (answer === startService) {
            this.startService(configuration);
        }
    }
    /**
     * Start the planning service.
     * @param configuration planning configuration for a service to start
     */
    startService(configuration) {
        var _a;
        const executablePath = configuration.path;
        if (!executablePath) {
            throw new Error(`Expected configuration with attribute: 'path'`);
        }
        const taskExecution = new vscode_1.ShellExecution(pddl_workspace_1.utils.Util.q(executablePath), {
            cwd: ((_a = configuration.cwd) !== null && _a !== void 0 ? _a : path.isAbsolute(executablePath)) ? path.dirname(executablePath) : undefined
        });
        const task = new vscode_1.Task(this.taskDefinition, vscode_1.TaskScope.Global, configuration.title, this.taskSource, taskExecution);
        task.presentationOptions = { reveal: vscode_1.TaskRevealKind.Always, echo: true };
        task.isBackground = true;
        vscode_1.tasks.executeTask(task).then(taskExecution => {
            this.planningServiceExecutions.set(executablePath, PlanningServiceExecution.justStarted(executablePath, taskExecution));
        });
    }
    get taskDefinition() {
        return {
            type: 'PlanningService:' + this.kind.kind
        };
    }
    get taskSource() {
        return 'PDDL';
    }
}
exports.LongRunningPlannerProvider = LongRunningPlannerProvider;
//# sourceMappingURL=plannerExecution.js.map