/* --------------------------------------------------------------------------------------------
 * Copyright (c) Jan Dolejsi. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for license information.
 * ------------------------------------------------------------------------------------------ */
'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
exports.HappeningsExecutor = void 0;
const vscode = require("vscode");
const vscode_1 = require("vscode");
const path_1 = require("path");
const ai_planning_val_1 = require("ai-planning-val");
/**
 * Executes sequence of plan happenings and decorates the happenings file with action effects.
 */
class HappeningsExecutor {
    /**
     * Constructs the executor.
     * @param editor happenings text editor
     * @param context debugging session context - the happenings, problem and domain files
     * @param pddlConfiguration user/workspace configuration
     */
    constructor(editor, context, pddlConfiguration) {
        this.editor = editor;
        this.context = context;
        this.pddlConfiguration = pddlConfiguration;
        this.decorations = [];
        this.seeNextLineDecoration = vscode_1.window.createTextEditorDecorationType({
            after: {
                contentText: '↓',
                textDecoration: "; opacity: 0.5; font-size: 10px; margin-left: 5px"
            }
        });
        this.seeNextLineRanges = [];
        this.valStep = new ai_planning_val_1.ValStep(context.domain, context.problem);
    }
    /**
     * Executes all the happenings and decorates the happenings file with action effects.
     */
    async execute() {
        if (this.context.happenings.getParsingProblems().length > 0) {
            vscode.commands.executeCommand('workbench.action.problems.show');
            vscode.window.showErrorMessage('The plan happenings file contains syntactic errors. Fix those first.');
            return [];
        }
        const valStepPath = await this.pddlConfiguration.getValStepPath();
        const valStepVerbose = this.pddlConfiguration.getValStepVerbose();
        if (!valStepPath) {
            return [];
        }
        const cwd = path_1.dirname(this.context.happenings.fileUri.fsPath);
        try {
            this.valStep.on(ai_planning_val_1.ValStep.NEW_HAPPENING_EFFECTS, (happenings, values) => this.showValues(happenings, values));
            await this.valStep.executeIncrementally(this.context.happenings.getHappenings(), { valStepPath, cwd, verbose: valStepVerbose });
            this.decorations.push(this.seeNextLineDecoration);
        }
        catch (err) {
            if (err instanceof ai_planning_val_1.ValStepError) {
                vscode_1.window.showErrorMessage(err.message);
            }
            else if (err instanceof ai_planning_val_1.ValStepExitCode) {
                vscode_1.window.showInformationMessage(err.message);
            }
            else {
                vscode_1.window.showErrorMessage(err);
            }
        }
        return this.decorations;
    }
    showValues(happenings, values) {
        const decoration = this.createDecorationText(values);
        this.decorate(decoration, happenings);
    }
    createDecorationText(values) {
        const positiveEffects = values.filter(v => v.getValue() === true).sort((a, b) => a.getVariableName().localeCompare(b.getVariableName()));
        const negativeEffects = values.filter(v => v.getValue() === false).sort((a, b) => a.getVariableName().localeCompare(b.getVariableName()));
        const numericEffects = values.filter(v => v.getValue() !== true && v.getValue() !== false).sort((a, b) => a.getVariableName().localeCompare(b.getVariableName()));
        const decorations = [];
        if (positiveEffects.length > 0) {
            const decoration = 'Sets: ' + positiveEffects.map(v => `(${v.getVariableName()})`).join(', ');
            decorations.push(decoration);
        }
        if (negativeEffects.length > 0) {
            const decoration = 'Unsets: ' + negativeEffects.map(v => `(${v.getVariableName()})`).join(', ');
            decorations.push(decoration);
        }
        if (numericEffects.length > 0) {
            const decoration = 'Assigns: ' + numericEffects.map(v => `(${v.getVariableName()}):=${v.getValue()}`).join(', ');
            decorations.push(decoration);
        }
        return decorations.length > 0 ? decorations.join(', ') : 'Has no effects.';
    }
    decorate(decorationText, happenings) {
        const decorationType = vscode_1.window.createTextEditorDecorationType({
            after: {
                contentText: decorationText,
                textDecoration: "; opacity: 0.5; font-size: 10px; margin-left: 5px"
            }
        });
        const lastHappening = happenings[happenings.length - 1];
        const range = this.createRange(lastHappening);
        this.editor.setDecorations(decorationType, [range]);
        this.decorations.push(decorationType);
        for (let index = 0; index < happenings.length - 1; index++) {
            const range = this.createRange(happenings[index]);
            this.seeNextLineRanges.push(range);
            this.editor.setDecorations(this.seeNextLineDecoration, this.seeNextLineRanges);
        }
    }
    createRange(happening) {
        var _a;
        const line = (_a = happening.lineIndex) !== null && _a !== void 0 ? _a : 0;
        return new vscode.Range(line, 0, line, 100);
    }
}
exports.HappeningsExecutor = HappeningsExecutor;
//# sourceMappingURL=HappeningsExecutor.js.map