/* --------------------------------------------------------------------------------------------
 * Copyright (c) Jan Dolejsi. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for license information.
 * ------------------------------------------------------------------------------------------ */
'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
exports.ValidatorService = void 0;
const vscode_1 = require("vscode");
const validator_1 = require("./validator");
const pddl_workspace_1 = require("pddl-workspace");
const request = require("request");
class ValidatorService extends validator_1.Validator {
    constructor(path, useAuthentication, authentication) {
        super(path);
        this.useAuthentication = useAuthentication;
        this.authentication = authentication;
    }
    validate(domainInfo, problemFiles, onSuccess, onError) {
        // eslint-disable-next-line @typescript-eslint/no-explicit-any
        let requestHeader = {};
        if (this.useAuthentication && this.authentication.getToken()) {
            requestHeader = {
                "Authorization": "Bearer " + this.authentication.getToken()
            };
        }
        const requestBody = {
            "domain": domainInfo.getText(),
            "problems": problemFiles.map(pf => pf.getText())
        };
        request.post({ url: this.path, headers: requestHeader, body: requestBody, json: true }, (err, httpResponse, responseBody) => {
            var _a;
            if (err) {
                onError.apply(this, [err.message]);
                return;
            }
            if (this.useAuthentication) {
                if (httpResponse) {
                    if (httpResponse.statusCode === 400) {
                        const message = "Authentication failed. Please login or update tokens.";
                        onError.apply(this, [message]);
                        return;
                    }
                    else if (httpResponse.statusCode === 401) {
                        const message = "Invalid token. Please update tokens.";
                        onError.apply(this, [message]);
                        return;
                    }
                }
            }
            if (httpResponse && httpResponse.statusCode !== 200) {
                const notificationMessage = `PDDL Language Parser returned code ${httpResponse.statusCode} ${httpResponse.statusMessage}`;
                //let notificationType = MessageType.Warning;
                onError.apply(this, [notificationMessage]);
                return;
            }
            const messages = responseBody;
            const diagnostics = this.createEmptyDiagnostics(domainInfo, problemFiles);
            for (let i = 0; i < messages.length; i++) {
                //&& diagnostics.length < this.maxNumberOfProblems; i++) {
                domainInfo.setStatus(pddl_workspace_1.FileStatus.Validated);
                problemFiles.forEach(p => p.setStatus(pddl_workspace_1.FileStatus.Validated));
                const location = messages[i].location;
                let fileUri;
                if (location === "DOMAIN") {
                    fileUri = domainInfo.fileUri;
                }
                else if (location.startsWith("PROBLEM")) {
                    const problemIdx = parseInt(location.substr("PROBLEM".length + 1));
                    fileUri = problemFiles[problemIdx].fileUri;
                }
                else {
                    console.log("Unsupported: " + location);
                    continue;
                }
                const severity = validator_1.Validator.toSeverity(messages[i].severity);
                const range = this.toRange(messages[i].position);
                if (fileUri !== undefined) {
                    (_a = diagnostics.get(fileUri.toString())) === null || _a === void 0 ? void 0 : _a.push(new vscode_1.Diagnostic(range, messages[i].message, severity));
                }
            }
            // Send the computed diagnostics to VSCode.
            onSuccess.apply(this, [diagnostics]);
        });
    }
    // eslint-disable-next-line @typescript-eslint/no-explicit-any
    toRange(position) {
        if (position === null || position === undefined) {
            return validator_1.Validator.createRange(0, 0);
        }
        const line = parseInt(position.line) - 1;
        const character = parseInt(position.character) - 1;
        return validator_1.Validator.createRange(line, character);
    }
}
exports.ValidatorService = ValidatorService;
//# sourceMappingURL=ValidatorService.js.map