/* --------------------------------------------------------------------------------------------
 * Copyright (c) Jan Dolejsi. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for license information.
 * ------------------------------------------------------------------------------------------ */
'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
exports.PddlOnTypeFormatter = void 0;
const vscode_1 = require("vscode");
const pddl_workspace_1 = require("pddl-workspace");
class PddlOnTypeFormatter {
    constructor(pddlWorkspace, testMode = false) {
        this.pddlWorkspace = pddlWorkspace;
        this.testMode = testMode;
    }
    async provideOnTypeFormattingEdits(document, position, ch, options, token) {
        var _a;
        const fileInfo = await ((_a = this.pddlWorkspace) === null || _a === void 0 ? void 0 : _a.upsertAndParseFile(document));
        if (token.isCancellationRequested) {
            return undefined;
        }
        const offset = document.offsetAt(position);
        if (fileInfo && (fileInfo.getLanguage() !== pddl_workspace_1.PddlLanguage.PDDL)) {
            return undefined;
        }
        let tree;
        if (fileInfo && (fileInfo instanceof pddl_workspace_1.DomainInfo)) {
            tree = fileInfo.syntaxTree;
        }
        else if (fileInfo && (fileInfo instanceof pddl_workspace_1.ProblemInfo)) {
            tree = fileInfo.syntaxTree;
        }
        else {
            tree = new pddl_workspace_1.parser.PddlSyntaxTreeBuilder(document.getText()).getTree();
        }
        const currentNode = tree.getNodeAt(offset);
        // console.log(`Formatting upon '${ch}', ${currentNode}, ${options}`);
        const parentIndent = this.getParentStartCharacterIndent(currentNode, document);
        if (parentIndent === null) {
            return [];
        }
        const previousIndent = this.getPreviousSiblingStartCharacterIndent(currentNode, document);
        const rangeBefore = new vscode_1.Range(position.with({ character: 0 }), position);
        const rangeAfter = new vscode_1.Range(position, position.with({ character: Number.MAX_VALUE }));
        if (ch === '\n' && currentNode.isType(pddl_workspace_1.parser.PddlTokenType.Whitespace)) {
            const insertBeforeText = previousIndent !== null ?
                PddlOnTypeFormatter.createIndent(previousIndent, 0, options) :
                PddlOnTypeFormatter.createIndent(parentIndent, +1, options);
            const insertAfterText = '\n' + PddlOnTypeFormatter.createIndent(previousIndent !== null && previousIndent !== void 0 ? previousIndent : parentIndent, 0, options);
            // todo: use createEolString(document)
            const trailingText = document.getText(rangeAfter).trim();
            if (trailingText && !trailingText.match(/\w/)) {
                const edits = [
                    vscode_1.TextEdit.replace(rangeBefore, insertBeforeText + insertAfterText),
                ];
                return this.applyAndMoveCursor(document, edits, { to: 'left', value: insertAfterText.length });
            }
            else {
                const edits = [
                    vscode_1.TextEdit.replace(rangeBefore, insertBeforeText),
                ];
                return edits;
            }
        }
        else if (ch === '(-disabled') { // disabled, because it cancels out the auto-completion pop-up
            const leadingText = document.getText(rangeBefore);
            if (leadingText.trim() === '(') {
                const insertBeforeText = PddlOnTypeFormatter.createIndent(parentIndent, +1, options) + '(';
                return [
                    vscode_1.TextEdit.replace(rangeBefore, insertBeforeText)
                ];
            }
            return [];
        }
        else {
            return [];
        }
    }
    async applyAndMoveCursor(document, edits, cursorMoveOptions) {
        if (this.testMode) {
            return edits;
        }
        else {
            const edit = new vscode_1.WorkspaceEdit();
            edit.set(document.uri, edits);
            await vscode_1.workspace.applyEdit(edit);
            await vscode_1.commands.executeCommand('cursorMove', cursorMoveOptions);
            return [];
        }
    }
    getParentStartCharacterIndent(currentNode, document) {
        var _a;
        const parent = (_a = currentNode.getParent()) === null || _a === void 0 ? void 0 : _a.expand();
        if (parent === undefined) {
            throw new Error(`Unexpected PDDL structure in ${currentNode.getText()}`);
        }
        if (parent.isDocument()) {
            return null;
        }
        else {
            const lineOfParent = document.lineAt(document.positionAt(parent.getStart()).line);
            const firstNonWhitespaceCharacter = lineOfParent.firstNonWhitespaceCharacterIndex;
            return lineOfParent.text.substr(0, firstNonWhitespaceCharacter);
        }
    }
    getPreviousSiblingStartCharacterIndent(currentNode, document) {
        const allPreviousSiblings = currentNode.getPrecedingSiblings(pddl_workspace_1.parser.PddlTokenType.OpenBracket)
            .concat(currentNode.getPrecedingSiblings(pddl_workspace_1.parser.PddlTokenType.OpenBracketOperator))
            .sort((n1, n2) => n1.getStart() - n2.getStart());
        if (allPreviousSiblings.length === 0) {
            return null;
        }
        const previous = allPreviousSiblings[allPreviousSiblings.length - 1];
        if (previous.isDocument()) {
            return null;
        }
        const lineOfPrevious = document.lineAt(document.positionAt(previous.getStart()).line);
        const firstNonWhitespaceCharacter = lineOfPrevious.firstNonWhitespaceCharacterIndex;
        return lineOfPrevious.text.substr(0, firstNonWhitespaceCharacter);
    }
    static createIndent(parentIndentText, levelIncrement, options) {
        const singleIndent = options.insertSpaces ? " ".repeat(options.tabSize) : "\t";
        return parentIndentText +
            singleIndent.repeat(levelIncrement);
    }
}
exports.PddlOnTypeFormatter = PddlOnTypeFormatter;
//# sourceMappingURL=PddlOnTypeFormatter.js.map