/* --------------------------------------------------------------------------------------------
 * Copyright (c) Jan Dolejsi. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for license information.
 * ------------------------------------------------------------------------------------------ */
'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
exports.ProblemConstraintsView = void 0;
const vscode_1 = require("vscode");
const path = require("path");
const pddl_workspace_1 = require("pddl-workspace");
const utils_1 = require("../utils");
const view_1 = require("./view");
const ProblemView_1 = require("./ProblemView");
const pddl_workspace_2 = require("pddl-workspace");
const CONTENT = path.join('views', 'modelView', 'static');
const PDDL_PROBLEM_CONSTRAINTS_PREVIEW_COMMAND = "pddl.problem.constraints.preview";
const PDDL_PROBLEM_CONSTRAINTS_INSET_COMMAND = "pddl.problem.constraints.inset";
class ProblemConstraintsView extends ProblemView_1.ProblemView {
    constructor(context, codePddlWorkspace) {
        super(context, codePddlWorkspace, new ProblemConstraintsRenderer(), {
            content: CONTENT,
            viewCommand: PDDL_PROBLEM_CONSTRAINTS_PREVIEW_COMMAND,
            insetViewCommand: PDDL_PROBLEM_CONSTRAINTS_INSET_COMMAND,
            insetHeight: 5,
            webviewType: 'problemConstraintsPreview',
            webviewHtmlPath: 'graphView.html',
            webviewOptions: {
                enableFindWidget: true,
                // enableCommandUris: true,
                retainContextWhenHidden: true,
                enableScripts: true,
                localResourceRoots: [
                    vscode_1.Uri.file(context.asAbsolutePath("views"))
                ]
            }
        }, {});
    }
    async provideCodeLenses(document, token) {
        if (token.isCancellationRequested) {
            return undefined;
        }
        const problem = await this.parseProblem(document);
        if (token.isCancellationRequested) {
            return undefined;
        }
        if (!problem) {
            return [];
        }
        const defineNode = problem.syntaxTree.getDefineNodeOrThrow();
        const constraintsNode = defineNode.getFirstChild(pddl_workspace_1.parser.PddlTokenType.OpenBracketOperator, /\s*:constraints/i);
        if (constraintsNode) {
            return [
                new view_1.DocumentCodeLens(document, utils_1.nodeToRange(document, constraintsNode))
            ];
        }
        else {
            return [];
        }
    }
    async resolveCodeLens(codeLens, token) {
        if (!(codeLens instanceof view_1.DocumentCodeLens)) {
            return undefined;
        }
        if (token.isCancellationRequested) {
            return undefined;
        }
        const domainAndProblem = await this.getProblemAndDomain(codeLens.getDocument());
        if (!domainAndProblem) {
            return undefined;
        }
        if (token.isCancellationRequested) {
            return undefined;
        }
        if (codeLens instanceof view_1.DocumentInsetCodeLens) {
            codeLens.command = { command: PDDL_PROBLEM_CONSTRAINTS_INSET_COMMAND, title: 'View inset', arguments: [codeLens.getDocument().uri, codeLens.getLine()] };
            return codeLens;
        }
        else {
            codeLens.command = { command: PDDL_PROBLEM_CONSTRAINTS_PREVIEW_COMMAND, title: 'View', arguments: [codeLens.getDocument().uri] };
            return codeLens;
        }
    }
    createPreviewPanelTitle(uri) {
        return `:constraints of '${path.basename(uri.fsPath)}'`;
    }
    async handleOnLoad(panel) {
        await panel.postMessage('setOptions', {
            "layout": {
                "hierarchical": {
                    "enabled": true,
                    "levelSeparation": 50,
                    "treeSpacing": 300,
                    "sortMethod": "directed"
                }
            },
        });
        return super.handleOnLoad(panel);
    }
}
exports.ProblemConstraintsView = ProblemConstraintsView;
class ProblemConstraintsRenderer {
    render(context, problem, domain, options) {
        const renderer = new ProblemConstraintsRendererDelegate(context, domain, problem, options);
        return {
            nodes: renderer.getNodes(),
            relationships: renderer.getRelationships()
        };
    }
}
class NamedConditionNode {
    constructor(id, name, definition) {
        this.id = id;
        this.name = name;
        this.definition = definition;
    }
}
class ProblemConstraintsRendererDelegate {
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    constructor(_context, domain, problem, _options) {
        this.domain = domain;
        this.problem = problem;
        this.nodes = new Map();
        this.relationships = [];
        const allConstraints = this.domain.getConstraints().concat(this.problem.getConstraints());
        this.namedConditionConstraints = allConstraints
            .filter(c => c instanceof pddl_workspace_2.NamedConditionConstraint)
            .map(c => c);
        this.afterConstraints = allConstraints
            .filter(c => c instanceof pddl_workspace_2.AfterConstraint)
            .map(c => c);
        this.namedConditionConstraints
            .forEach((c, index) => this.addNamedCondition(c, index));
        this.lastNodeIndex = this.namedConditionConstraints.length;
        this.afterConstraints.forEach(ac => {
            const predecessorId = this.upsertGoal(ac.predecessor);
            const successorId = this.upsertGoal(ac.successor);
            const label = ac instanceof pddl_workspace_2.StrictlyAfterConstraint ? 'strictly-after' : 'after';
            this.addEdge(predecessorId, successorId, label);
        });
        this.addTetheredGoal();
    }
    addNamedCondition(namedCondition, index) {
        var _a, _b, _c, _d, _e, _f;
        const key = (_c = (_a = namedCondition.name) !== null && _a !== void 0 ? _a : (_b = namedCondition.condition) === null || _b === void 0 ? void 0 : _b.getText()) !== null && _c !== void 0 ? _c : "unnamed";
        this.nodes.set(key, new NamedConditionNode(index, (_d = namedCondition.name) !== null && _d !== void 0 ? _d : "", (_f = (_e = namedCondition.condition) === null || _e === void 0 ? void 0 : _e.getText()) !== null && _f !== void 0 ? _f : "unspecified condition"));
        return index;
    }
    upsertGoal(namedCondition) {
        if (namedCondition.name) {
            const detail = this.nodes.get(namedCondition.name);
            if (detail) {
                return detail.id;
            }
            else {
                // this happens when the model is incomplete
                return this.addNamedCondition(namedCondition, this.lastNodeIndex++);
            }
        }
        else if (namedCondition.condition) {
            const conditionText = namedCondition.condition.getText();
            if (this.nodes.has(conditionText)) {
                return this.nodes.get(conditionText).id;
            }
            else {
                return this.addNamedCondition(namedCondition, this.lastNodeIndex++);
            }
        }
        else {
            throw new Error('Unexpected constraint: ' + namedCondition.toString());
        }
    }
    addTetheredGoal() {
        const pattern = /\((start|end)\s+of\s*(.+)\s*\)/i;
        [...this.nodes.keys()].forEach(nodeStartName => {
            const matchStart = pattern.exec(nodeStartName);
            if (matchStart) {
                if (matchStart[1] === "start") {
                    const actionName = matchStart[2];
                    [...this.nodes.keys()].find(nodeEndName => {
                        const matchEnd = pattern.exec(nodeEndName);
                        if (matchEnd &&
                            matchEnd[1] === "end" &&
                            matchEnd[2] === actionName) {
                            const startNode = this.nodes.get(nodeStartName);
                            const endNode = this.nodes.get(nodeEndName);
                            if (startNode && endNode) {
                                this.addEdge(startNode.id, endNode.id, "durative-action");
                            }
                        }
                    });
                }
            }
        });
    }
    addEdge(predecessorId, successorId, label) {
        this.relationships.push({ from: predecessorId, to: successorId, label: label });
    }
    getNodes() {
        return [...this.nodes.values()].map(entry => this.toNode(entry));
    }
    toNode(entry) {
        const shape = "box";
        // concatenate the name and definition if both are provided
        const label = [entry.name, entry.definition]
            .filter(element => element && element.length > 0)
            .join(': ');
        return { id: entry.id, label: label, shape: shape };
    }
    getRelationships() {
        return this.relationships;
    }
}
//# sourceMappingURL=ProblemConstraintsView.js.map