/* --------------------------------------------------------------------------------------------
 * Copyright (c) Jan Dolejsi. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for license information.
 * ------------------------------------------------------------------------------------------ */
'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
exports.ProblemInitView = void 0;
const vscode_1 = require("vscode");
const pddl_workspace_1 = require("pddl-workspace");
const path = require("path");
const utils_1 = require("../utils");
const view_1 = require("./view");
const ProblemView_1 = require("./ProblemView");
const DomainVisualization_1 = require("../planView/DomainVisualization");
const asSerializable = pddl_workspace_1.utils.serializationUtils.asSerializable;
const makeSerializable = pddl_workspace_1.utils.serializationUtils.makeSerializable;
const CONTENT = path.join('views', 'modelView', 'static');
const PDDL_PROBLEM_INIT_PREVIEW_COMMAND = "pddl.problem.init.preview";
const PDDL_PROBLEM_INIT_INSET_COMMAND = "pddl.problem.init.inset";
const DEFAULT_INSET_HEIGHT = 10;
class ProblemInitView extends ProblemView_1.ProblemView {
    constructor(context, codePddlWorkspace) {
        super(context, codePddlWorkspace, new ProblemInitRenderer(), {
            content: CONTENT,
            viewCommand: PDDL_PROBLEM_INIT_PREVIEW_COMMAND,
            insetViewCommand: PDDL_PROBLEM_INIT_INSET_COMMAND,
            insetHeight: DEFAULT_INSET_HEIGHT,
            webviewType: 'problemPreview',
            webviewHtmlPath: 'problemInitView.html',
            allowUnsafeEval: true,
            webviewOptions: {
                enableFindWidget: true,
                // enableCommandUris: true,
                retainContextWhenHidden: true,
                enableScripts: true,
                localResourceRoots: [
                    vscode_1.Uri.file(context.asAbsolutePath("views"))
                ]
            }
        }, { displayWidth: 100 });
    }
    async provideCodeLenses(document, token) {
        if (token.isCancellationRequested) {
            return null;
        }
        const problem = await this.parseProblem(document);
        if (token.isCancellationRequested) {
            return null;
        }
        if (!problem) {
            return [];
        }
        const defineNode = problem.syntaxTree.getDefineNodeOrThrow();
        const initNode = defineNode.getFirstChild(pddl_workspace_1.parser.PddlTokenType.OpenBracketOperator, /\s*:init/i);
        if (initNode) {
            return [
                new view_1.DocumentCodeLens(document, utils_1.nodeToRange(document, initNode))
            ];
        }
        else {
            return [];
        }
    }
    async resolveCodeLens(codeLens, token) {
        if (!(codeLens instanceof view_1.DocumentCodeLens)) {
            return null;
        }
        if (token.isCancellationRequested) {
            return null;
        }
        const domainAndProblem = await this.getProblemAndDomain(codeLens.getDocument());
        if (!domainAndProblem) {
            return null;
        }
        if (token.isCancellationRequested) {
            return null;
        }
        if (codeLens instanceof view_1.DocumentInsetCodeLens) {
            codeLens.command = { command: PDDL_PROBLEM_INIT_INSET_COMMAND, title: 'View inset', arguments: [codeLens.getDocument().uri, codeLens.getLine()] };
            return codeLens;
        }
        else {
            codeLens.command = { command: PDDL_PROBLEM_INIT_PREVIEW_COMMAND, title: 'View', arguments: [codeLens.getDocument().uri] };
            return codeLens;
        }
    }
    createPreviewPanelTitle(uri) {
        return `:init of '${path.basename(uri.fsPath)}'`;
    }
}
exports.ProblemInitView = ProblemInitView;
class ProblemInitRenderer {
    async render(context, problem, domain, options) {
        const renderer = new ProblemInitRendererDelegate(context, domain, problem, options);
        return {
            symmetricRelationshipGraph: {
                nodes: renderer.getNodes(),
                relationships: renderer.getRelationships()
            },
            typeProperties: asSerializable(renderer.getTypeProperties()),
            typeRelationships: asSerializable(renderer.getTypeRelationships()),
            scalarValues: asSerializable(renderer.getScalarValues()),
            customVisualization: !options.hideCustomViz ? makeSerializable(await renderer.getCustomVisualization()) : undefined
        };
    }
}
class ProblemInitRendererDelegate {
    constructor(_context, domain, problem, options) {
        this.domain = domain;
        this.problem = problem;
        this.options = options;
        this.nodes = new Map();
        this.relationships = [];
        this.typeProperties = new Map();
        this.typeRelationships = new Array();
        this.scalarValues = new Map();
        if (!options.hide2dGraph) {
            this.construct2dGraphData();
        }
        if (!options.hideScalarValues) {
            this.constructScalarValues();
        }
        if (!options.hideObjectProperties) {
            this.constructObjectProperties();
        }
        if (!options.hideObjectRelationships) {
            this.constructObjectRelationships();
        }
    }
    constructScalarValues() {
        const scalarVariables = this.domain.getPredicates().concat(this.domain.getFunctions())
            .filter(v => v.parameters.length === 0);
        const scalarInits = scalarVariables
            .map(v => this.toVariableInitialValueTuple(v));
        scalarInits.forEach(init => this.addScalar(init));
    }
    toVariableInitialValueTuple(v) {
        return { variable: v, value: this.getInitValue(v) };
    }
    addScalar(initValue) {
        var _a, _b;
        const variableName = initValue.variable.getFullName();
        if (!this.scalarValues.has(variableName)) { // this is where we are throwing away TILs/TIFs
            this.scalarValues.set(variableName, ((_a = initValue.value) === null || _a === void 0 ? void 0 : _a.getValue()) !== undefined ? (_b = initValue.value) === null || _b === void 0 ? void 0 : _b.getValue() : "undefined");
        }
    }
    construct2dGraphData() {
        const symmetric2dPredicates = this.domain.getPredicates()
            .filter(v => this.is2DGraphable(v));
        const symmetric2dFunctions = this.domain.getFunctions()
            .filter(v => this.is2DGraphable(v));
        const symmetric2dVariables = symmetric2dFunctions.concat(symmetric2dPredicates);
        const relatableTypes = pddl_workspace_1.utils.Util.distinct(pddl_workspace_1.utils.Util.flatMap(symmetric2dVariables
            .map(v => [v.parameters[0].type, v.parameters[1].type])));
        const relatableAndInheritedTypes = pddl_workspace_1.utils.Util.distinct(pddl_workspace_1.utils.Util.flatMap(relatableTypes.map(type => pddl_workspace_1.getTypesInheritingFromPlusSelf(type, this.domain.getTypeInheritance()))));
        relatableAndInheritedTypes.forEach(type => this.getObjects(type).forEach(obj => this.addNode(obj)));
        const symmetric2dInits = this.problem.getInits()
            .filter(init => init.isSupported)
            .filter(init => symmetric2dVariables.some(v => v.matchesShortNameCaseInsensitive(init.getLiftedVariableName())));
        symmetric2dInits.forEach(init => this.addRelationship(init));
    }
    is2DGraphable(v) {
        return ProblemInitRendererDelegate.is2D(v) &&
            (!this.options.graph2dSymmetricOnly || ProblemInitRendererDelegate.is2DSymmetric(v));
    }
    constructObjectProperties() {
        this.domain.getTypes().forEach(type => this.constructTypeProperties(type));
    }
    constructTypeProperties(type) {
        const typeObjects = this.domain.getConstants().merge(this.problem.getObjectsTypeMap()).getTypeCaseInsensitive(type);
        if (!typeObjects) {
            return;
        }
        const objects = typeObjects.getObjects();
        if (objects.length > 0) {
            const typeDetails = this.constructObjectsProperties(type, objects);
            if (typeDetails.propertyNames.length) {
                this.typeProperties.set(type, typeDetails);
            }
        }
    }
    constructObjectsProperties(type, objects) {
        const liftedVariables = this.domain.getPredicates().concat(this.domain.getFunctions())
            .filter(variable => this.isTypeProperty(type, variable));
        const objectsValues = new Map();
        objects.forEach(objectName => {
            const objectValues = new Map();
            liftedVariables.forEach(v => {
                const value = this.getInitValue(v.bind([new pddl_workspace_1.ObjectInstance(objectName, type)]));
                if (value) {
                    objectValues.set(v.name, value.getValue());
                }
            });
            objectsValues.set(objectName, objectValues);
        });
        return {
            propertyNames: liftedVariables.map(v => v.name),
            objects: objectsValues
        };
    }
    getInitValue(groundedVariable) {
        const firstInit = this.problem.getInits()
            .filter(init => init.isSupported)
            .filter(viv => viv.getVariableName().toLowerCase() === groundedVariable.getFullName().toLowerCase())
            .sort(viv => viv.getTime())
            .find(() => true);
        // todo: do not ignore viv.getTime()
        return firstInit;
    }
    isTypeProperty(type, variable) {
        const applicableTypes = pddl_workspace_1.getTypesInheritingFromPlusSelf(type, this.domain.getTypeInheritance());
        return variable.parameters.length === 1
            && applicableTypes.includes(variable.parameters[0].type);
    }
    getScalarValues() {
        return this.scalarValues;
    }
    getTypeProperties() {
        return this.typeProperties;
    }
    getTypeRelationships() {
        return this.typeRelationships;
    }
    constructObjectRelationships() {
        const binaryRelationships = this.domain.getPredicates().concat(this.domain.getFunctions())
            .filter(v => v.parameters.length === 2);
        const relationshipPerTypes = pddl_workspace_1.utils.Util.groupBy(binaryRelationships, r => r.parameters.map(p => p.type).join(','));
        relationshipPerTypes
            .forEach((relationships, typeNames) => this.constructTypesRelationships(typeNames.split(','), relationships));
    }
    constructTypesRelationships(types, relationships) {
        const typeObjectsMap = new Map();
        types.forEach(t => typeObjectsMap.set(t, this.getObjects(t)));
        const relationshipsMap = new Map();
        relationships.forEach(r => relationshipsMap.set(r.name, this.createTypeRelationships(r)));
        this.typeRelationships.push({
            types: typeObjectsMap,
            relationships: relationshipsMap
        });
    }
    createTypeRelationships(relationship) {
        const applicableInits = this.problem.getInits()
            .filter(init => relationship.matchesShortNameCaseInsensitive(init.getLiftedVariableName()));
        return applicableInits.map(init => this.createRelationshipValue(init));
    }
    createRelationshipValue(init) {
        const parameters = new Map();
        const liftedVariable = this.domain.getPredicates().concat(this.domain.getFunctions())
            .find(v => v.name.toLowerCase() === init.getLiftedVariableName().toLowerCase());
        if (liftedVariable) {
            liftedVariable.parameters
                .forEach((term, index) => parameters.set(term.name, init.getVariableName().split(' ')[index + 1]));
        }
        else {
            init.getVariableName().split(' ').slice(1)
                .forEach((term, index) => parameters.set(index.toString(), term));
        }
        return {
            parameters: parameters,
            value: init.getValue()
        };
    }
    getObjects(type) {
        return pddl_workspace_1.getObjectsInheritingFrom(this.domain.getConstants().merge(this.problem.getObjectsTypeMap()), type, this.domain.getTypeInheritance());
    }
    addNode(obj) {
        if (!this.nodes.has(obj)) {
            this.nodes.set(obj, this.nodes.size + 1);
        }
    }
    addRelationship(initialValue) {
        const edge = this.toEdge(initialValue);
        if (edge) {
            this.relationships.push(edge);
        }
    }
    getNodes() {
        return [...this.nodes.entries()].map(entry => this.toNode(entry));
    }
    toNode(entry) {
        const [entryLabel, entryId] = entry;
        return { id: entryId, label: entryLabel };
    }
    toEdge(initialValue) {
        const variableNameParts = initialValue.getVariableName().split(' ');
        const fromName = variableNameParts[1];
        const toName = variableNameParts[2];
        let label = variableNameParts[0];
        if (variableNameParts.length > 3) {
            // the variable had more than 2 parameters
            label += ' ' + variableNameParts.slice(3).join(' ');
        }
        if (!(initialValue.getValue() === true)) {
            // this is a fluent! include the value
            label += `=${initialValue.getValue()}`;
        }
        if (initialValue.getTime() > 0) {
            // this is a timed-initial literal/fluent
            label += ' @ ' + initialValue.getTime();
        }
        const fromId = this.nodes.get(fromName);
        const toId = this.nodes.get(toName);
        if (fromId !== undefined && toId !== undefined) {
            return { from: fromId, to: toId, label: label };
        }
        else {
            console.log(`One or more nodes not found: ${fromName}, ${toName}`);
            return null;
        }
    }
    getRelationships() {
        return this.relationships;
    }
    /**
     * Tests whether the predicate/function has two parameters of the same type.
     * @param variable predicate/function to test
     */
    static is2DSymmetric(variable) {
        return ProblemInitRendererDelegate.is2D(variable)
            && variable.parameters[0].type === variable.parameters[1].type;
    }
    static is2D(variable) {
        return variable.parameters.length >= 2;
    }
    getSerialisableDomain() {
        var _a;
        return (_a = this.serializableDomain) !== null && _a !== void 0 ? _a : (this.serializableDomain = pddl_workspace_1.DomainInfo.clone(makeSerializable(this.domain)));
    }
    getSerialisableProblem() {
        var _a;
        return (_a = this.serializableProblem) !== null && _a !== void 0 ? _a : (this.serializableProblem = pddl_workspace_1.ProblemInfo.clone(makeSerializable(this.problem)));
    }
    async getCustomVisualization() {
        const plan = new pddl_workspace_1.Plan([], makeSerializable(this.getSerialisableDomain()), this.getSerialisableProblem());
        const domainViz = await DomainVisualization_1.getDomainVisualizationConfigurationDataForPlan(plan);
        return {
            plan: plan,
            state: this.problem.getInits()
                .filter(init => init.isSupported)
                .map(init => init.getVariableValue()),
            customVisualizationScript: domainViz.customDomainVisualizationScript,
            displayWidth: domainViz.width
        };
    }
}
//# sourceMappingURL=ProblemInitView.js.map