/* --------------------------------------------------------------------------------------------
 * Copyright (c) Jan Dolejsi. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for license information.
 * ------------------------------------------------------------------------------------------ */
'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
exports.ProblemObjectsView = void 0;
const vscode_1 = require("vscode");
const path = require("path");
const pddl_workspace_1 = require("pddl-workspace");
const utils_1 = require("../utils");
const view_1 = require("./view");
const ProblemView_1 = require("./ProblemView");
const CONTENT = path.join('views', 'modelView', 'static');
const PDDL_PROBLEM_OBJECTS_PREVIEW_COMMAND = "pddl.problem.objects.preview";
const PDDL_PROBLEM_OBJECTS_INSET_COMMAND = "pddl.problem.objects.inset";
class ProblemObjectsView extends ProblemView_1.ProblemView {
    constructor(context, codePddlWorkspace) {
        super(context, codePddlWorkspace, new ProblemObjectsRenderer(), {
            content: CONTENT,
            viewCommand: PDDL_PROBLEM_OBJECTS_PREVIEW_COMMAND,
            insetViewCommand: PDDL_PROBLEM_OBJECTS_INSET_COMMAND,
            insetHeight: 5,
            webviewType: 'problemObjectsPreview',
            webviewHtmlPath: 'graphView.html',
            webviewOptions: {
                enableFindWidget: true,
                // enableCommandUris: true,
                retainContextWhenHidden: true,
                enableScripts: true,
                localResourceRoots: [
                    vscode_1.Uri.file(context.asAbsolutePath("views"))
                ]
            }
        }, {});
    }
    async provideCodeLenses(document, token) {
        if (token.isCancellationRequested) {
            return null;
        }
        const problem = await this.parseProblem(document);
        if (token.isCancellationRequested) {
            return null;
        }
        if (!problem) {
            return [];
        }
        const defineNode = problem.syntaxTree.getDefineNodeOrThrow();
        const objectsNode = defineNode.getFirstChild(pddl_workspace_1.parser.PddlTokenType.OpenBracketOperator, /\s*:objects/i);
        if (objectsNode) {
            return [
                new view_1.DocumentCodeLens(document, utils_1.nodeToRange(document, objectsNode))
            ];
        }
        else {
            return [];
        }
    }
    async resolveCodeLens(codeLens, token) {
        if (!(codeLens instanceof view_1.DocumentCodeLens)) {
            return null;
        }
        if (token.isCancellationRequested) {
            return null;
        }
        const domainAndProblem = await this.getProblemAndDomain(codeLens.getDocument());
        if (!domainAndProblem) {
            return null;
        }
        if (token.isCancellationRequested) {
            return null;
        }
        if (codeLens instanceof view_1.DocumentInsetCodeLens) {
            codeLens.command = { command: PDDL_PROBLEM_OBJECTS_INSET_COMMAND, title: 'Show hierarchy inset', arguments: [codeLens.getDocument().uri, codeLens.getLine()] };
            return codeLens;
        }
        else {
            codeLens.command = { command: PDDL_PROBLEM_OBJECTS_PREVIEW_COMMAND, title: 'Show hierarchy', arguments: [codeLens.getDocument().uri] };
            return codeLens;
        }
    }
    createPreviewPanelTitle(uri) {
        return `:objects of '${path.basename(uri.fsPath)}'`;
    }
    async handleOnLoad(panel) {
        await panel.postMessage('setInverted', { value: true });
        await panel.postMessage('setOptions', {
            groups: {
                object: {
                    color: {
                        background: 'lightgreen'
                    },
                    borderWidth: 0
                }
            }
        });
        return super.handleOnLoad(panel);
    }
}
exports.ProblemObjectsView = ProblemObjectsView;
class ProblemObjectsRenderer {
    render(context, problem, domain, options) {
        const renderer = new ProblemObjectsRendererDelegate(context, domain, problem, options);
        return {
            nodes: renderer.getNodes(),
            relationships: renderer.getRelationships()
        };
    }
}
class ProblemObjectsRendererDelegate {
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    constructor(_context, domain, problem, _options) {
        this.domain = domain;
        this.problem = problem;
        this.nodes = new Map();
        this.relationships = [];
        this.typeNames = new Set();
        this.objectsAndConstantsPerType = this.domain.getConstants().merge(this.problem.getObjectsTypeMap());
        domain.getTypesInclObject().forEach((t, index) => {
            this.nodes.set(t, index);
            this.typeNames.add(t);
        });
        domain.getTypeInheritance().getEdges().forEach(edge => this.addEdge(edge, 'extends'));
        this.lastIndex = domain.getTypesInclObject().length;
        domain.getTypes().forEach(t => this.addObjects(t));
    }
    addObjects(typeName) {
        const objectsOfType = this.objectsAndConstantsPerType.getTypeCaseInsensitive(typeName);
        if (objectsOfType) {
            const objects = objectsOfType.getObjects();
            objects.forEach((objectName, index) => {
                this.nodes.set(objectName, index + this.lastIndex);
                this.addEdge([objectName, typeName], '');
            });
            this.lastIndex += objects.length;
        }
    }
    addEdge(edge, label) {
        const networkEdge = this.toEdge(edge, label);
        if (networkEdge) {
            this.relationships.push(networkEdge);
        }
    }
    getNodes() {
        return [...this.nodes.entries()].map(entry => this.toNode(entry));
    }
    toNode(entry) {
        const [entryLabel, entryId] = entry;
        const isType = this.typeNames.has(entryLabel);
        const shape = isType ? "ellipse" : "box";
        const group = isType ? "type" : "object";
        return { id: entryId, label: entryLabel, shape: shape, group: group };
    }
    toEdge(edge, label) {
        const [from, to] = edge;
        const fromId = this.nodes.get(from);
        const toId = this.nodes.get(to);
        if (fromId !== undefined && toId !== undefined) {
            return { from: fromId, to: toId, label: label };
        }
        else {
            console.log(`One or more nodes not found: ${from}, ${to}`);
            return null;
        }
    }
    getRelationships() {
        return this.relationships;
    }
}
//# sourceMappingURL=ProblemObjectsView.js.map