/* --------------------------------------------------------------------------------------------
 * Copyright (c) Jan Dolejsi. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for license information.
 * ------------------------------------------------------------------------------------------ */
'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
exports.ProblemView = void 0;
const vscode_1 = require("vscode");
const vscode_extension_telemetry_wrapper_1 = require("vscode-extension-telemetry-wrapper");
const path = require("path");
const workspaceUtils_1 = require("../workspace/workspaceUtils");
const pddl_workspace_1 = require("pddl-workspace");
const utils_1 = require("../utils");
const ProblemViewPanel_1 = require("./ProblemViewPanel");
const view_1 = require("./view");
const webviewUtils_1 = require("../webviewUtils");
const VIEWS = "views";
const COMMON_FOLDER = path.join(VIEWS, "common");
/**
 * Base-class for different problem views.
 */
class ProblemView extends vscode_1.Disposable {
    constructor(context, codePddlWorkspace, renderer, options, rendererOptions) {
        super(() => this.dispose());
        this.context = context;
        this.codePddlWorkspace = codePddlWorkspace;
        this.renderer = renderer;
        this.options = options;
        this.rendererOptions = rendererOptions;
        this._onDidChangeCodeLenses = new vscode_1.EventEmitter();
        this.onDidChangeCodeLenses = this._onDidChangeCodeLenses.event;
        this.webviewPanels = new utils_1.UriMap();
        this.webviewInsets = new utils_1.UriMap();
        context.subscriptions.push(vscode_extension_telemetry_wrapper_1.instrumentOperationAsVsCodeCommand(options.viewCommand, async (problemUri) => {
            const problemDocument = await getProblemDocument(problemUri);
            if (problemDocument) {
                return this.revealOrCreatePreview(problemDocument, vscode_1.ViewColumn.Beside).catch(utils_1.showError);
            }
        }));
        context.subscriptions.push(vscode_extension_telemetry_wrapper_1.instrumentOperationAsVsCodeCommand(options.insetViewCommand, async (problemUri, line) => {
            if (vscode_1.window.activeTextEditor && problemUri && line) {
                if (problemUri.toString() === vscode_1.window.activeTextEditor.document.uri.toString()) {
                    this.showInset(vscode_1.window.activeTextEditor, line, options.insetHeight).catch(utils_1.showError);
                }
            }
        }));
        codePddlWorkspace.pddlWorkspace.on(pddl_workspace_1.PddlWorkspace.UPDATED, (fileInfo) => {
            if (fileInfo.isProblem()) {
                this.refreshProblem(fileInfo);
            }
            else if (fileInfo.isDomain()) {
                this.refreshDomainProblems(fileInfo);
            }
        });
    }
    async refreshDomainProblems(domainInfo) {
        const promises = this.codePddlWorkspace.pddlWorkspace
            .getProblemFiles(domainInfo)
            .map(async (problemInfo) => await this.refreshProblem(problemInfo, domainInfo));
        await Promise.all(promises);
    }
    async refreshProblem(problemInfo, domainInfo) {
        const problemUri = utils_1.toUri(problemInfo.fileUri);
        // if no panel was created, skip
        if (!this.webviewPanels.get(problemUri) && !this.webviewInsets.get(problemUri)) {
            return;
        }
        let domainInfoAssigned;
        let error;
        try {
            domainInfoAssigned = domainInfo !== null && domainInfo !== void 0 ? domainInfo : workspaceUtils_1.getDomainFileForProblem(problemInfo, this.codePddlWorkspace);
        }
        catch (ex) {
            error = ex;
        }
        let panelsToRefresh = [];
        // update the panel
        const panel = this.webviewPanels.get(problemUri);
        if (panel) {
            panelsToRefresh.push(panel);
        }
        // update all the insets
        const insets = this.webviewInsets.get(problemUri);
        if (insets) {
            panelsToRefresh = panelsToRefresh.concat([...insets.values()]);
        }
        panelsToRefresh.forEach(async (panel) => {
            if (!error) {
                panel.setDomainAndProblem(domainInfoAssigned, problemInfo);
            }
            else {
                panel.setError(error);
            }
            await this.refreshPanelContent(panel);
        });
        if (panelsToRefresh.length > 0) {
            this._onDidChangeCodeLenses.fire();
        }
    }
    refreshPanel(panel) {
        if (panel.getNeedsRebuild() && panel.getPanel().isVisible()) {
            this.refreshPanelContent(panel);
        }
    }
    async setup(previewPanel, problemInfo) {
        if (!previewPanel.getPanel().html) {
            previewPanel.getPanel().html = "Please wait...";
        }
        previewPanel.setNeedsRebuild(false);
        previewPanel.getPanel().html = await this.generateHtml(previewPanel);
        try {
            const domainInfo = workspaceUtils_1.getDomainFileForProblem(problemInfo, this.codePddlWorkspace);
            previewPanel.setDomainAndProblem(domainInfo, problemInfo);
        }
        catch (ex) {
            previewPanel.setError(ex);
        }
    }
    async refreshPanelContent(previewPanel) {
        previewPanel.setNeedsRebuild(false);
        return this.updateContentData(previewPanel.getDomain(), previewPanel.getProblem(), previewPanel);
    }
    async showInset(editor, line, height) {
        console.log(`todo: revealOrCreateInset ${editor} line ${line} height ${height}`);
    }
    async expandInset(panel) {
        console.log(`todo: expand inset ${panel.uri}`);
    }
    async revealOrCreatePreview(doc, displayColumn) {
        const problemInfo = await this.parseProblem(doc);
        if (!problemInfo) {
            return;
        }
        let previewPanel = this.webviewPanels.get(doc.uri);
        if (previewPanel && previewPanel.getPanel().canReveal()) {
            previewPanel.getPanel().reveal(displayColumn);
            await this.refreshProblem(problemInfo);
        }
        else {
            previewPanel = await this.createPreviewPanelForDocument(problemInfo, displayColumn);
            this.webviewPanels.set(doc.uri, previewPanel);
        }
    }
    async createPreviewPanelForDocument(problemInfo, displayColumn) {
        const problemUri = utils_1.toUri(problemInfo.fileUri);
        const previewTitle = this.createPreviewPanelTitle(problemUri);
        const webViewPanel = vscode_1.window.createWebviewPanel(this.options.webviewType, previewTitle, displayColumn, this.options.webviewOptions);
        webViewPanel.iconPath = vscode_1.Uri.file(this.context.asAbsolutePath("images/icon.png"));
        const panel = new ProblemViewPanel_1.ProblemViewPanel(problemUri, new view_1.WebviewPanelAdapter(webViewPanel));
        // when the user closes the tab, remove the panel
        webViewPanel.onDidDispose(() => this.webviewPanels.delete(problemUri), undefined, this.context.subscriptions);
        // when the pane becomes visible again, refresh it
        webViewPanel.onDidChangeViewState(() => this.refreshPanel(panel));
        webViewPanel.webview.onDidReceiveMessage(e => this.handleMessageCore(panel, e), undefined, this.context.subscriptions);
        await this.setup(panel, problemInfo);
        return panel;
    }
    async generateHtml(viewPanel) {
        const error = viewPanel.getError();
        if (error) {
            return error.message;
        }
        else {
            return webviewUtils_1.getWebViewHtml(this.context, {
                allowUnsafeEval: this.options.allowUnsafeEval,
                fonts: [
                    vscode_1.Uri.file(path.join("..", "..", "..", COMMON_FOLDER, "codicon.ttf"))
                ],
                relativePath: this.options.content, htmlFileName: this.options.webviewHtmlPath
            }, viewPanel.getPanel().webview);
        }
    }
    async updateContentData(domain, problem, panel) {
        const stateData = await this.renderer.render(this.context, problem, domain, this.rendererOptions);
        return panel.postMessage('updateContent', {
            data: stateData
        });
    }
    async parseProblem(problemDocument) {
        const fileInfo = await this.codePddlWorkspace.upsertAndParseFile(problemDocument);
        if (!(fileInfo === null || fileInfo === void 0 ? void 0 : fileInfo.isProblem())) {
            return undefined;
        }
        return fileInfo;
    }
    async getProblemAndDomain(problemDocument) {
        try {
            const fileInfo = await this.parseProblem(problemDocument);
            if (!fileInfo) {
                return undefined;
            }
            const problemInfo = fileInfo;
            const domainInfo = workspaceUtils_1.getDomainFileForProblem(problemInfo, this.codePddlWorkspace);
            return [domainInfo, problemInfo];
        }
        catch (ex) {
            throw new Error("No domain associated to problem.");
        }
    }
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    handleMessage(_panel, _message) {
        return false;
    }
    async handleOnLoad(panel) {
        await panel.postMessage('setIsInset', { value: panel.getPanel().isInset });
        return await this.refreshPanelContent(panel);
    }
    async handleMessageCore(panel, message) {
        console.log(`Message received from the webview: ${message.command}`);
        switch (message.command) {
            case 'close':
                panel.close();
                break;
            case 'expand':
                this.expandInset(panel);
                break;
            case 'onload':
                await this.handleOnLoad(panel);
                break;
            default:
                if (!this.handleMessage(panel, message)) {
                    console.warn('Unexpected command: ' + message.command);
                }
        }
    }
}
exports.ProblemView = ProblemView;
async function getProblemDocument(dotDocumentUri) {
    if (dotDocumentUri) {
        return await vscode_1.workspace.openTextDocument(dotDocumentUri);
    }
    else {
        if (vscode_1.window.activeTextEditor !== undefined && workspaceUtils_1.isPddl(vscode_1.window.activeTextEditor.document)) {
            return vscode_1.window.activeTextEditor.document;
        }
        else {
            return undefined;
        }
    }
}
//# sourceMappingURL=ProblemView.js.map