/* --------------------------------------------------------------------------------------------
 * Copyright (c) Jan Dolejsi. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for license information.
 * ------------------------------------------------------------------------------------------ */
'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
exports.PlanReportGenerator = void 0;
const vscode_1 = require("vscode");
const path = require("path");
const fs = require("fs");
const opn = require("open");
const pddl_workspace_1 = require("pddl-workspace");
const ai_planning_val_1 = require("ai-planning-val");
const pddl_gantt_1 = require("pddl-gantt");
const SwimLane_1 = require("./SwimLane");
const PlanReportSettings_1 = require("./PlanReportSettings");
const configuration_1 = require("../configuration/configuration");
const utils_1 = require("../utils");
const valStepErrorHandler_1 = require("../planView/valStepErrorHandler");
const DIGITS = 4;
class PlanReportGenerator {
    constructor(context, options) {
        this.context = context;
        this.options = options;
        this.planStepHeight = 20;
        this.settings = new Map();
        this.colors = ['#ff0000', '#ff4000', '#ff8000', '#ffbf00', '#ffff00', '#bfff00', '#80ff00', '#40ff00', '#00ff00', '#00ff40', '#00ff80', '#00ffbf', '#00ffff', '#00bfff', '#0080ff', '#0040ff', '#0000ff', '#4000ff', '#8000ff', '#bf00ff', '#ff00ff', '#ff00bf', '#ff0080', '#ff0040'];
    }
    async export(plans, planId) {
        const html = await this.generateHtml(plans, planId);
        const htmlFile = await ai_planning_val_1.Util.toFile("plan-report", ".html", html);
        const uri = vscode_1.Uri.parse("file://" + htmlFile);
        opn(uri.toString());
        return true; //env.openExternal(uri);
    }
    async generateHtml(plans, planId = -1) {
        const selectedPlan = planId < 0 ? plans.length - 1 : planId;
        const maxCost = Math.max(...plans.map(plan => { var _a; return (_a = plan.metric) !== null && _a !== void 0 ? _a : 0; }));
        const planSelectors = plans.map((plan, planIndex) => this.renderPlanSelector(plan, planIndex, selectedPlan, maxCost)).join(" ");
        const planSelectorsDisplayStyle = plans.length > 1 ? "flex" : "none";
        const planHtmlArr = await Promise.all(plans.map(async (plan, planIndex) => await this.renderPlan(plan, planIndex, selectedPlan)));
        const plansHtml = planHtmlArr.join("\n\n");
        const plansChartsScript = this.createPlansChartsScript(plans);
        const relativePath = path.join('views', 'planview');
        const staticPath = path.join(relativePath, 'static');
        const ganttStylesPath = path.join('node_modules', 'pddl-gantt', 'styles');
        const html = `<!DOCTYPE html>
        <head>
            <title>Plan report</title>
            <meta http-equiv="Content-Security-Policy"
                content="default-src 'none'; img-src vscode-resource: https: data:; script-src vscode-resource: https://www.gstatic.com/charts/ 'unsafe-inline'; style-src vscode-resource: https://www.gstatic.com/charts/ 'unsafe-inline';"
            />    
            ${await this.includeStyle(this.asAbsolutePath(ganttStylesPath, 'pddl-gantt.css'))}
            ${await this.includeStyle(this.asAbsolutePath(staticPath, 'plans.css'))}
            ${await this.includeScript(this.asAbsolutePath(path.join(relativePath, 'out'), 'plans.js'))}
            <script type="text/javascript" src="https://www.gstatic.com/charts/loader.js"></script>
            ${await this.includeScript(this.asAbsolutePath(staticPath, 'charts.js'))}
        </head>
        <body onload="scrollPlanSelectorIntoView(${selectedPlan})">
            <div class="planSelectors" style="display: ${planSelectorsDisplayStyle};">${planSelectors}
            </div>
        ${plansHtml}
        ${plansChartsScript}
        </body>`;
        return html;
    }
    asAbsolutePath(...paths) {
        let uri = vscode_1.Uri.file(this.context.asAbsolutePath(path.join(...paths)));
        if (!this.options.selfContained && this.options.resourceUriConverter) {
            uri = this.options.resourceUriConverter.asWebviewUri(uri);
        }
        return uri;
    }
    renderPlanSelector(plan, planIndex, selectedPlan, maxCost) {
        var _a;
        let className = "planSelector";
        if (planIndex === selectedPlan) {
            className += " planSelector-selected";
        }
        const normalizedCost = ((_a = plan.cost) !== null && _a !== void 0 ? _a : 0) / maxCost * 100;
        const costRounded = plan.cost ? plan.cost.toFixed(DIGITS) : NaN;
        const tooltip = `Plan #${planIndex}
Metric value / cost: ${plan.cost}
Makespan: ${plan.makespan}
States evaluated: ${plan.statesEvaluated}`;
        return `
        <div class="${className}" plan="${planIndex}" onclick="showPlan(${planIndex})"><span>${costRounded}</span>
            <div class="planMetricBar" style="height: ${normalizedCost}px" title="${tooltip}"></div>
        </div>`;
    }
    shouldDisplay(planStep, plan) {
        if (this.settings.has(plan)) {
            return this.settings.get(plan).shouldDisplay(planStep);
        }
        else {
            return true;
        }
    }
    async renderPlan(plan, planIndex, selectedPlan) {
        var _a, _b;
        plan = pddl_gantt_1.capitalize(plan);
        let planVisualizerPath;
        if (plan.domain) {
            const settings = await PlanReportSettings_1.PlanReportSettings.load(plan.domain.fileUri);
            planVisualizerPath = settings.getPlanVisualizerScript();
            this.settings.set(plan, settings);
        }
        const styleDisplay = planIndex === selectedPlan ? "block" : "none";
        let stateViz = '';
        if (planVisualizerPath && plan.domain) {
            const absPath = path.join(pddl_workspace_1.PddlWorkspace.getFolderPath(plan.domain.fileUri), planVisualizerPath);
            try {
                const planVisualizerText = (await vscode_1.workspace.fs.readFile(vscode_1.Uri.file(absPath))).toString();
                const planVisualizer = eval(planVisualizerText);
                const width = this.options.displayWidth;
                stateViz = (_b = (_a = planVisualizer.visualizePlanHtml) === null || _a === void 0 ? void 0 : _a.call(planVisualizer, plan, width)) !== null && _b !== void 0 ? _b : `<div class="hint">Only 'visualizePlanHtml' option is supported by the exported plan report currently. </div>`;
                stateViz = `<div class="stateView" plan="${planIndex}" style="margin: 5px; width: ${width}px; display: ${styleDisplay};">${stateViz}</div>`;
            }
            catch (ex) {
                console.warn(ex);
            }
        }
        const stepsToDisplay = plan.steps
            .filter(step => this.shouldDisplay(step, plan));
        // split this to two batches and insert helpful actions in between
        const planHeadSteps = stepsToDisplay
            .filter(step => this.isPlanHeadStep(step, plan));
        const relaxedPlanSteps = stepsToDisplay
            .filter(step => !this.isPlanHeadStep(step, plan));
        const oneIfHelpfulActionsPresent = (plan.hasHelpfulActions() ? 1 : 0);
        const relaxedPlanStepIndexOffset = planHeadSteps.length + oneIfHelpfulActionsPresent;
        const ganttChartHtml = planHeadSteps
            .map((step, stepIndex) => this.renderGanttStep(step, stepIndex, plan, planIndex)).join("\n")
            + this.renderHelpfulActions(plan, planHeadSteps.length) + '\n'
            + relaxedPlanSteps
                .map((step, stepIndex) => this.renderGanttStep(step, stepIndex + relaxedPlanStepIndexOffset, plan, planIndex)).join("\n");
        const ganttChartHeight = (stepsToDisplay.length + oneIfHelpfulActionsPresent) * this.planStepHeight;
        const ganttChart = `    <div class="gantt" plan="${planIndex}" style="margin: 5px; height: ${ganttChartHeight}px; display: ${styleDisplay};">
    ${ganttChartHtml}
        </div>`;
        let objectsHtml = '';
        if (!this.options.disableSwimLaneView && plan.domain && plan.problem) {
            const allTypeObjects = plan.domain.getConstants().merge(plan.problem.getObjectsTypeMap());
            objectsHtml = plan.domain.getTypes()
                .filter(type => type !== "object")
                .map(type => {
                const typeObjects = allTypeObjects.getTypeCaseInsensitive(type);
                return typeObjects
                    ? this.renderTypeSwimLanes(type, typeObjects.getObjects(), plan)
                    : '';
            }).join("\n");
        }
        const swimLanes = `    <div class="resourceUtilization" plan="${planIndex}" style="display: ${styleDisplay};">
        <table>
${objectsHtml}
        </table>
    </div>`;
        const valStepPath = utils_1.ensureAbsoluteGlobalStoragePath(vscode_1.workspace.getConfiguration(configuration_1.CONF_PDDL).get(configuration_1.VAL_STEP_PATH), this.context);
        const valueSeqPath = utils_1.ensureAbsoluteGlobalStoragePath(vscode_1.workspace.getConfiguration(configuration_1.CONF_PDDL).get(configuration_1.VALUE_SEQ_PATH), this.context);
        const valVerbose = vscode_1.workspace.getConfiguration(configuration_1.CONF_PDDL).get(configuration_1.VAL_VERBOSE, false);
        let lineCharts = `    <div class="lineChart" plan="${planIndex}" style="display: ${styleDisplay};margin-top: 20px;">\n`;
        let lineChartScripts = '';
        if (!this.options.disableLinePlots && plan.domain && plan.problem) {
            const groupByLifted = vscode_1.workspace.getConfiguration(configuration_1.CONF_PDDL).get(configuration_1.PLAN_REPORT_LINE_PLOT_GROUP_BY_LIFTED, true);
            const evaluator = new ai_planning_val_1.PlanFunctionEvaluator(plan, { valStepPath, valueSeqPath, shouldGroupByLifted: groupByLifted, verbose: valVerbose });
            if (evaluator.isAvailable()) {
                try {
                    const functionValues = await evaluator.evaluate();
                    functionValues.forEach((values, liftedVariable) => {
                        const chartDivId = `chart_${planIndex}_${liftedVariable.declaredName}`;
                        lineCharts += this.createLineChartDiv(chartDivId);
                        let chartTitleWithUnit = values.legend.length > 1 ? liftedVariable.name : liftedVariable.getFullName();
                        if (liftedVariable.getUnit()) {
                            chartTitleWithUnit += ` [${liftedVariable.getUnit()}]`;
                        }
                        lineChartScripts += `        drawChart('${chartDivId}', '${chartTitleWithUnit}', '', ${JSON.stringify(values.legend)}, ${JSON.stringify(values.values)}, ${this.options.displayWidth});\n`;
                    });
                    // add one plot for declared metric
                    for (let metricIndex = 0; metricIndex < plan.problem.getMetrics().length; metricIndex++) {
                        const metric = plan.problem.getMetrics()[metricIndex];
                        const metricExpression = metric.getExpression();
                        if (metricExpression instanceof pddl_workspace_1.VariableExpression) {
                            if (metricExpression.name === "total-time") {
                                continue;
                            }
                        }
                        const metricValues = await evaluator.evaluateExpression(metric.getExpression());
                        const chartDivId = `chart_${planIndex}_metric${metricIndex}`;
                        lineCharts += this.createLineChartDiv(chartDivId);
                        const chartTitleWithUnit = metric.getDocumentation()[metric.getDocumentation().length - 1];
                        lineChartScripts += `        drawChart('${chartDivId}', '${chartTitleWithUnit}', '', ['${ /*unit?*/""}'], ${JSON.stringify(metricValues.values)}, ${this.options.displayWidth});\n`;
                    }
                }
                catch (err) {
                    console.error(err);
                    const valStepPath = evaluator.getValStepPath();
                    if (valStepPath) {
                        valStepErrorHandler_1.handleValStepError(err, valStepPath);
                    }
                }
            }
            else {
                lineCharts += `<a href="command:pddl.downloadVal" title="Click to initiate download. You will be able to see what is being downloaded and from where...">Download plan validation tools (a.k.a. VAL) to see line plots of function values.</a>`;
            }
        }
        lineCharts += `\n    </div>`;
        const hint = plan.domain && plan.problem ? '' : `<div class="hint"><b>Hint:</b> Problem file was not identified for this plan. Open the associated domain and problem and visualize the plan again.</div>`;
        return `${this.options.selfContained || this.options.disableHamburgerMenu ? '' : this.renderMenu()}
${stateViz}
${ganttChart}
${swimLanes}
${lineCharts}
${hint}
        <script>function drawPlan${planIndex}Charts(){\n${lineChartScripts}}</script>
`;
    }
    createLineChartDiv(chartDivId) {
        return `        <div id="${chartDivId}" style="width: ${this.options.displayWidth + 100}px; height: ${Math.round(this.options.displayWidth / 2)}px"></div>\n`;
    }
    renderHelpfulActions(plan, planHeadLength) {
        var _a;
        if (plan.hasHelpfulActions()) {
            const fromTop = planHeadLength * this.planStepHeight;
            const fromLeft = this.toViewCoordinates(plan.now, plan);
            const text = (_a = plan.helpfulActions) === null || _a === void 0 ? void 0 : _a.map((helpfulAction, index) => this.renderHelpfulAction(index, helpfulAction)).join(', ');
            return `\n        <div class="planstep" style="top: ${fromTop}px; left: ${fromLeft}px; margin-top: 3px">▶ ${text}</div>`;
        }
        else {
            return '';
        }
    }
    renderHelpfulAction(index, helpfulAction) {
        const suffix = PlanReportGenerator.getActionSuffix(helpfulAction);
        const beautifiedName = `${helpfulAction.actionName}<sub>${suffix}</sub>`;
        return `${index + 1}. <a href="#" onclick="navigateToChildOfSelectedState('${helpfulAction.actionName}')">${beautifiedName}</a>`;
    }
    static getActionSuffix(helpfulAction) {
        switch (helpfulAction.kind) {
            case pddl_workspace_1.HappeningType.START:
                return '├';
            case pddl_workspace_1.HappeningType.END:
                return '┤';
        }
        return '';
    }
    isPlanHeadStep(step, plan) {
        return plan.now === undefined ||
            step.commitment === pddl_workspace_1.PlanStepCommitment.Committed ||
            step.commitment === pddl_workspace_1.PlanStepCommitment.EndsInRelaxedPlan;
    }
    createPlansChartsScript(plans) {
        const selectedPlan = plans.length - 1;
        return `        <script>
                google.charts.setOnLoadCallback(drawCharts);
                function drawCharts() {
                    drawPlan${selectedPlan}Charts();
                }
        </script>`;
    }
    renderTypeSwimLanes(type, objects, plan) {
        return `            <tr>
                <th>${type}</th>
                <th style="width: ${this.options.displayWidth}px"></th>
            </tr>
`
            + objects.map(obj => this.renderObjectSwimLane(obj, plan)).join('\n');
    }
    renderObjectSwimLane(obj, plan) {
        const subLanes = new SwimLane_1.SwimLane(1);
        const stepsInvolvingThisObject = plan.steps
            .filter(step => this.shouldDisplay(step, plan))
            .filter(step => this.shouldDisplayObject(step, obj, plan))
            .map(step => this.renderSwimLameStep(step, plan, obj, subLanes))
            .join('\n');
        return `            <tr>
                <td class="objectName"><div>${obj}</div></td>
                <td style="position: relative; height: ${subLanes.laneCount() * this.planStepHeight}px;">
${stepsInvolvingThisObject}
                </td>
            </tr>
`;
    }
    shouldDisplayObject(step, obj, plan) {
        var _a, _b, _c;
        if (!this.settings.has(plan)) {
            return true;
        }
        const liftedAction = (_a = plan.domain) === null || _a === void 0 ? void 0 : _a.getActions().find(a => a.getNameOrEmpty().toLowerCase() === step.getActionName().toLowerCase());
        if (!liftedAction) {
            console.debug('Unexpected plan action: ' + step.getActionName());
            return true;
        }
        let fromArgument = 0;
        do {
            const indexOfArgument = step.getObjects().indexOf(obj, fromArgument);
            fromArgument = indexOfArgument + 1;
            if (indexOfArgument > -1 && indexOfArgument < liftedAction.parameters.length) {
                const parameter = liftedAction.parameters[indexOfArgument];
                const shouldIgnoreThisArgument = (_b = this.settings.get(plan)) === null || _b === void 0 ? void 0 : _b.shouldIgnoreActionParameter((_c = liftedAction.name) !== null && _c !== void 0 ? _c : 'unnamed', parameter.name);
                if (!shouldIgnoreThisArgument) {
                    return true;
                }
            }
        } while (fromArgument > 0);
        return false;
    }
    renderSwimLameStep(step, plan, thisObj, swimLanes) {
        const actionColor = this.getActionColor(step, plan.domain);
        const leftOffset = this.computeLeftOffset(step, plan);
        const width = this.computeWidth(step, plan) + this.computeRelaxedWidth(step, plan);
        const objects = step.getObjects()
            .map(obj => obj.toLowerCase() === thisObj.toLowerCase() ? '@' : obj)
            .join(' ');
        const availableLane = swimLanes.placeNext(leftOffset, width);
        const fromTop = availableLane * this.planStepHeight + 1;
        return `
                    <div class="resourceTaskTooltip" style="background-color: ${actionColor}; left: ${leftOffset}px; width: ${width}px; top: ${fromTop}px;">${step.getActionName()} ${objects}<span class="resourceTaskTooltipText">${this.toActionTooltip(step)}</span></div>`;
    }
    renderGanttStep(step, index, plan, planIndex) {
        const actionLink = this.toActionLink(step.getActionName(), plan);
        const fromTop = index * this.planStepHeight;
        const fromLeft = this.computeLeftOffset(step, plan);
        const width = this.computeWidth(step, plan);
        const widthRelaxed = this.computeRelaxedWidth(step, plan);
        const actionColor = plan.domain ? this.getActionColor(step, plan.domain) : 'gray';
        const actionIterations = step.getIterations() > 1 ? `${step.getIterations()}x` : '';
        return `        <div class="planstep" id="plan${planIndex}step${index}" style="left: ${fromLeft}px; top: ${fromTop}px; "><div class="planstep-bar" title="${this.toActionTooltipPlain(step)}" style="width: ${width}px; background-color: ${actionColor}"></div><div class="planstep-bar-relaxed whitecarbon" style="width: ${widthRelaxed}px;"></div>${actionLink} ${step.getObjects().join(' ')} ${actionIterations}</div>`;
    }
    toActionLink(actionName, plan) {
        if (this.options.selfContained || !plan.domain) {
            return actionName;
        }
        else {
            const revealActionUri = encodeURI('command:pddl.revealAction?' + JSON.stringify([plan.domain.fileUri, actionName]));
            return `<a href="${revealActionUri}" title="Reveal '${actionName}' action in the domain file">${actionName}</a>`;
        }
    }
    toActionTooltip(step) {
        var _a;
        const durationRow = step.isDurative && step.getDuration() !== undefined ?
            `<tr><td class="actionToolTip">Duration: </td><td class="actionToolTip">${(_a = step.getDuration()) === null || _a === void 0 ? void 0 : _a.toFixed(DIGITS)}</td></tr>
            <tr><td class="actionToolTip">End: </td><td class="actionToolTip">${step.getEndTime().toFixed(DIGITS)}</td></tr>` :
            '';
        return `<table><tr><th colspan="2" class="actionToolTip">${step.getActionName()} ${step.getObjects().join(' ')}</th></tr><tr><td class="actionToolTip" style="width:50px">Start:</td><td class="actionToolTip">${step.getStartTime().toFixed(DIGITS)}</td></tr>${durationRow}</table>`;
    }
    toActionTooltipPlain(step) {
        var _a;
        const durationRow = step.isDurative && step.getDuration() !== undefined ?
            `Duration: ${(_a = step.getDuration()) === null || _a === void 0 ? void 0 : _a.toFixed(DIGITS)}, End: ${step.getEndTime().toFixed(DIGITS)}` :
            '';
        const startTime = step.getStartTime() !== undefined ?
            `, Start: ${step.getStartTime().toFixed(DIGITS)}` :
            '';
        return `${step.getActionName()} ${step.getObjects().join(' ')}${startTime} ${durationRow}`;
    }
    async includeStyle(uri) {
        if (this.options.selfContained) {
            const styleText = await fs.promises.readFile(uri.fsPath, { encoding: 'utf-8' });
            return `<style>\n${styleText}\n</style>`;
        }
        else {
            return `<link rel = "stylesheet" type = "text/css" href = "${uri.toString()}" />`;
        }
    }
    async includeScript(uri) {
        if (this.options.selfContained) {
            const scriptText = await fs.promises.readFile(uri.fsPath, { encoding: 'utf-8' });
            return `<script>\n${scriptText}\n</script>`;
        }
        else {
            return `<script src="${uri.toString()}"></script>`;
        }
    }
    computeLeftOffset(step, plan) {
        return this.toViewCoordinates(step.getStartTime(), plan);
    }
    renderMenu() {
        return `    <div class="menu" onclick="postCommand('showMenu')" title="Click to show menu options...">&#x2630;</div>`;
    }
    computePlanHeadDuration(step, plan) {
        var _a, _b, _c;
        if (plan.now === undefined) {
            return (_a = step.getDuration()) !== null && _a !== void 0 ? _a : configuration_1.DEFAULT_EPSILON;
        }
        else if (step.getEndTime() < plan.now) {
            if (step.commitment === pddl_workspace_1.PlanStepCommitment.Committed) {
                return (_b = step.getDuration()) !== null && _b !== void 0 ? _b : configuration_1.DEFAULT_EPSILON;
            }
            else {
                return 0;
            } // the end was not committed yet
        }
        else if (step.getStartTime() >= plan.now) {
            return 0;
        }
        else {
            switch (step.commitment) {
                case pddl_workspace_1.PlanStepCommitment.Committed:
                    return (_c = step.getDuration()) !== null && _c !== void 0 ? _c : configuration_1.DEFAULT_EPSILON;
                case pddl_workspace_1.PlanStepCommitment.EndsInRelaxedPlan:
                    return 0;
                case pddl_workspace_1.PlanStepCommitment.StartsInRelaxedPlan:
                    return plan.now - step.getStartTime();
                default:
                    return 0; // should not happen
            }
        }
    }
    computeWidth(step, plan) {
        // remove the part of the planStep duration that belongs to the relaxed plan
        const planHeadDuration = this.computePlanHeadDuration(step, plan);
        return Math.max(1, this.toViewCoordinates(planHeadDuration, plan));
    }
    computeRelaxedWidth(step, plan) {
        var _a;
        const planHeadDuration = this.computePlanHeadDuration(step, plan);
        // remove the part of the planStep duration that belongs to the planhead part
        const relaxedDuration = ((_a = step.getDuration()) !== null && _a !== void 0 ? _a : configuration_1.DEFAULT_EPSILON) - planHeadDuration;
        return this.toViewCoordinates(relaxedDuration, plan);
    }
    /** Converts the _time_ argument to view coordinates */
    toViewCoordinates(time, plan) {
        return (time !== null && time !== void 0 ? time : 0) / plan.makespan * this.options.displayWidth;
    }
    getActionColor(step, domain) {
        const actionIndex = domain === null || domain === void 0 ? void 0 : domain.getActions().findIndex(action => action.getNameOrEmpty().toLowerCase() === step.getActionName().toLowerCase());
        if (actionIndex === undefined || actionIndex < 0) {
            return 'gray';
        }
        else {
            return this.colors[actionIndex * 7 % this.colors.length];
        }
    }
}
exports.PlanReportGenerator = PlanReportGenerator;
//# sourceMappingURL=PlanReportGenerator.js.map