/* --------------------------------------------------------------------------------------------
 * Copyright (c) Jan Dolejsi 2021. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for license information.
 * ------------------------------------------------------------------------------------------ */
'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
exports.getDomainVisualizationConfigurationDataForPlans = exports.getDomainVisualizationConfigurationDataForPlan = exports.getDomainVisualizationConfiguration = exports.getDocumentText = void 0;
const path = require("path");
const vscode_1 = require("vscode");
const pddl_workspace_1 = require("pddl-workspace");
const PlanReportSettings_1 = require("../planReport/PlanReportSettings");
const workspaceFs_1 = require("../util/workspaceFs");
const configuration_1 = require("../configuration/configuration");
async function getDocumentText(fileUri, defaultText) {
    if (await workspaceFs_1.exists(fileUri)) {
        return (await vscode_1.workspace.fs.readFile(fileUri)).toString();
    }
    else {
        return defaultText;
    }
}
exports.getDocumentText = getDocumentText;
async function getDomainVisualizationConfiguration(plan) {
    if (!plan.domain) {
        return undefined;
    }
    const visualizationConfigurationText = await getDocumentText(PlanReportSettings_1.PlanReportSettings.toVisualizationSettings(plan.domain.fileUri));
    if (visualizationConfigurationText !== undefined) {
        return PlanReportSettings_1.PlanReportSettings.loadFromText(visualizationConfigurationText);
    }
    return undefined;
}
exports.getDomainVisualizationConfiguration = getDomainVisualizationConfiguration;
async function getDomainVisualizationConfigurationDataForPlan(plan) {
    const width = vscode_1.workspace.getConfiguration(configuration_1.CONF_PDDL).get(configuration_1.PLAN_REPORT_WIDTH, 300);
    const domainVizConfiguration = plan.domain &&
        await getDomainVisualizationConfiguration(plan);
    const planVisualizerRelativePath = domainVizConfiguration === null || domainVizConfiguration === void 0 ? void 0 : domainVizConfiguration.getPlanVisualizerScript();
    const planVisualizationScriptPath = plan.domain && planVisualizerRelativePath &&
        path.join(pddl_workspace_1.PddlWorkspace.getFolderPath(plan.domain.fileUri), planVisualizerRelativePath);
    const planVisualizationScript = planVisualizationScriptPath
        && await getDocumentText(vscode_1.Uri.file(planVisualizationScriptPath));
    return {
        plan: plan,
        domainVisualizationConfiguration: domainVizConfiguration === null || domainVizConfiguration === void 0 ? void 0 : domainVizConfiguration.settings,
        customDomainVisualizationScript: planVisualizationScript,
        width: width
    };
}
exports.getDomainVisualizationConfigurationDataForPlan = getDomainVisualizationConfigurationDataForPlan;
async function getDomainVisualizationConfigurationDataForPlans(plans) {
    if (plans.length > 0) {
        const data = await getDomainVisualizationConfigurationDataForPlan(plans[0]);
        return {
            plans: plans,
            width: data.width,
            domainVisualizationConfiguration: data.domainVisualizationConfiguration,
            customDomainVisualizationScript: data.customDomainVisualizationScript
        };
    }
    else {
        return {
            plans: plans,
            width: 0
        };
    }
}
exports.getDomainVisualizationConfigurationDataForPlans = getDomainVisualizationConfigurationDataForPlans;
//# sourceMappingURL=DomainVisualization.js.map