/* --------------------------------------------------------------------------------------------
 * Copyright (c) Jan Dolejsi. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for license information.
 * ------------------------------------------------------------------------------------------ */
'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
exports.PlanView = exports.PDDL_SAVE_AS_EXPECTED_PLAN = exports.PDDL_EXPORT_PLAN = exports.PDDL_GENERATE_PLAN_REPORT = void 0;
const vscode_1 = require("vscode");
const vscode_extension_telemetry_wrapper_1 = require("vscode-extension-telemetry-wrapper");
const path = require("path");
const ai_planning_val_1 = require("ai-planning-val");
const pddl_workspace_1 = require("pddl-workspace");
const makeSerializable = pddl_workspace_1.utils.serializationUtils.makeSerializable;
const workspaceUtils_1 = require("../workspace/workspaceUtils");
const configuration_1 = require("../configuration/configuration");
const Menu_1 = require("../Menu");
const utils_1 = require("../utils");
const valStepErrorHandler_1 = require("./valStepErrorHandler");
const DomainVisualization_1 = require("./DomainVisualization");
const webviewUtils_1 = require("../webviewUtils");
const VIEWS = "views";
const COMMON_FOLDER = path.join(VIEWS, "common");
const STATIC_CONTENT_FOLDER = path.join(VIEWS, 'planview', 'static');
exports.PDDL_GENERATE_PLAN_REPORT = 'pddl.planReport';
exports.PDDL_EXPORT_PLAN = 'pddl.exportPlan';
exports.PDDL_SAVE_AS_EXPECTED_PLAN = 'pddl.saveAsExpectedPlan';
class PlanView extends vscode_1.Disposable {
    constructor(context, codePddlWorkspace) {
        super(() => this.dispose());
        this.context = context;
        this.codePddlWorkspace = codePddlWorkspace;
        this.webviewPanels = new Map(); // todo: replace with UriMap
        context.subscriptions.push(vscode_extension_telemetry_wrapper_1.instrumentOperationAsVsCodeCommand("pddl.plan.preview", async (planUri) => {
            const planDocument = await getPlanDocument(planUri);
            if (planDocument) {
                return this.revealOrCreatePreview(planDocument, vscode_1.ViewColumn.Beside);
            }
        }));
        // When the active document is changed set the provider for rebuild
        // this only occurs after an edit in a document
        context.subscriptions.push(vscode_1.workspace.onDidChangeTextDocument((e) => {
            if (e.document.languageId === pddl_workspace_1.PLAN) {
                this.setNeedsRebuild(e.document);
            }
        }));
        context.subscriptions.push(vscode_1.workspace.onDidSaveTextDocument((doc) => {
            if (doc.languageId === pddl_workspace_1.PLAN) {
                this.setNeedsRebuild(doc);
            }
        }));
        context.subscriptions.push(vscode_1.workspace.onDidChangeConfiguration(e => {
            if (e.affectsConfiguration(configuration_1.CONF_PDDL + '.' + configuration_1.PLAN_REPORT_WIDTH)) {
                // preview width configuration was updated --> rebuild all views
                this.webviewPanels.forEach(panel => panel.setNeedsRebuild(true));
                this.rebuild();
            }
        }));
    }
    setPlannerOutput(plans, reveal) {
        const plannerOutputPanel = this.getPlannerOutputPanel();
        plannerOutputPanel.setPlans(plans);
        this.resetTimeout();
        if (plans.length > 0 && reveal) {
            plannerOutputPanel.reveal();
        }
    }
    getPlannerOutputPanel() {
        let plannerOutputPanel = this.webviewPanels.get(PlanView.PLANNER_OUTPUT_URI);
        if (!plannerOutputPanel) {
            plannerOutputPanel = this.createPreviewPanel('Planner output', PlanView.PLANNER_OUTPUT_URI, vscode_1.ViewColumn.Three);
            this.webviewPanels.set(PlanView.PLANNER_OUTPUT_URI, plannerOutputPanel);
        }
        return plannerOutputPanel;
    }
    hasPlannerOutput() {
        return this.webviewPanels.has(PlanView.PLANNER_OUTPUT_URI);
    }
    async setNeedsRebuild(planDocument) {
        const panel = this.webviewPanels.get(planDocument.uri);
        if (panel) {
            try {
                const plan = await this.parsePlanFile(planDocument);
                panel.setPlans([plan]);
            }
            catch (ex) {
                panel.setError(ex);
            }
            this.resetTimeout();
        }
    }
    resetTimeout() {
        if (this.timeout) {
            clearTimeout(this.timeout);
        }
        this.timeout = setTimeout(() => this.rebuild(), 1000);
    }
    dispose() {
        if (this.timeout) {
            clearTimeout(this.timeout);
        }
    }
    rebuild(webviewPanel) {
        this.webviewPanels.forEach(async (panel) => {
            const updateThisPanel = webviewPanel === undefined || webviewPanel === panel.getPanel();
            if (updateThisPanel && panel.getNeedsRebuild() && panel.getPanel().visible) {
                this.updateContent(panel);
            }
        });
    }
    async updateContent(previewPanel) {
        previewPanel.setNeedsRebuild(false);
        const error = previewPanel.getError();
        if (error) {
            previewPanel.getPanel().webview.postMessage({ command: 'error', message: error.message });
        }
        else {
            // todo: only send the additional plan, not all of them
            const plans = previewPanel.getPlans()
                .map(p => makeSerializable(p));
            const plansData = await DomainVisualization_1.getDomainVisualizationConfigurationDataForPlans(plans);
            previewPanel.getPanel().webview.postMessage({
                command: 'showPlans',
                data: plansData
            });
        }
    }
    async revealOrCreatePreview(doc, displayColumn) {
        let previewPanel = this.webviewPanels.get(doc.uri);
        if (previewPanel) {
            previewPanel.reveal(displayColumn);
        }
        else {
            previewPanel = this.createPreviewPanelForDocument(doc, displayColumn);
            this.webviewPanels.set(doc.uri, previewPanel);
        }
        await this.setNeedsRebuild(doc);
    }
    createPreviewPanelForDocument(doc, displayColumn) {
        const previewTitle = `Preview '${path.basename(doc.uri.fsPath)}'`;
        return this.createPreviewPanel(previewTitle, doc.uri, displayColumn);
    }
    createPreviewPanel(previewTitle, uri, displayColumn) {
        const webViewPanel = vscode_1.window.createWebviewPanel('planPreview', previewTitle, displayColumn, {
            enableFindWidget: true,
            enableCommandUris: true,
            enableScripts: true,
            localResourceRoots: [vscode_1.Uri.file(this.context.asAbsolutePath(VIEWS))]
        });
        webViewPanel.iconPath = vscode_1.Uri.file(this.context.asAbsolutePath(path.join("views", "overview", "file_type_pddl_plan.svg")));
        this.getHtml(webViewPanel.webview).then(html => webViewPanel.webview.html = html);
        const previewPanel = new PlanPreviewPanel(uri, webViewPanel);
        // when the user closes the tab, remove the panel
        previewPanel.getPanel().onDidDispose(() => this.webviewPanels.delete(uri), undefined, this.context.subscriptions);
        // when the pane becomes visible again, refresh it
        previewPanel.getPanel().onDidChangeViewState(e => this.rebuild(e.webviewPanel));
        previewPanel.getPanel().webview.onDidReceiveMessage(e => this.handleMessage(previewPanel, e), undefined, this.context.subscriptions);
        return previewPanel;
    }
    async getHtml(webview) {
        const googleCharts = vscode_1.Uri.parse("https://www.gstatic.com/charts/");
        return webviewUtils_1.getWebViewHtml(this.context, {
            relativePath: STATIC_CONTENT_FOLDER, htmlFileName: 'plans.html',
            externalImages: [vscode_1.Uri.parse('data:')],
            allowUnsafeEval: true,
            externalScripts: [googleCharts],
            externalStyles: [googleCharts],
            fonts: [
                vscode_1.Uri.file(path.join("..", "..", "..", COMMON_FOLDER, "codicon.ttf"))
            ]
        }, webview);
    }
    async parsePlanFile(planDocument) {
        const planFileInfo = await this.codePddlWorkspace.upsertAndParseFile(planDocument);
        try {
            const domainAndProblem = workspaceUtils_1.getDomainAndProblemForPlan(planFileInfo, this.codePddlWorkspace.pddlWorkspace);
            return planFileInfo.getPlan(domainAndProblem.domain, domainAndProblem.problem);
        }
        catch (ex) {
            // show a hint that domain & problem were not associated
            return new pddl_workspace_1.Plan(planFileInfo.getSteps());
        }
    }
    // eslint-disable-next-line @typescript-eslint/no-explicit-any
    handleMessage(previewPanel, message) {
        console.log(`Message received from the webview: ${message.command}`);
        switch (message.command) {
            case 'onload':
                // Webview (and its resources) finished loading
                this.updateContent(previewPanel);
                break;
            case 'linePlotDataRequest': {
                const planIndex = message.planIndex;
                this.getLinePlotData(previewPanel, planIndex).catch(utils_1.showError);
                break;
            }
            case 'finalStateDataRequest': {
                const planIndex = message.planIndex;
                this.getFinalStateData(previewPanel, planIndex).catch(utils_1.showError);
                break;
            }
            case 'selectPlan':
                const planIndex = message.planIndex;
                previewPanel.setSelectedPlanIndex(planIndex);
                break;
            case 'showMenu':
                this.showMenu(previewPanel);
                break;
            case 'revealAction':
                vscode_1.commands.executeCommand("pddl.revealAction", vscode_1.Uri.file(message.domainUri.fsPath), message.action);
                break;
            default:
                console.warn('Unexpected command: ' + message.command);
        }
    }
    async getLinePlotData(previewPanel, planIndex) {
        var _a;
        if (planIndex >= previewPanel.getPlans().length) {
            console.error(`requesting data for plan index {} while there are only {} plans`, planIndex, previewPanel.getPlans().length);
            return;
        }
        const plan = previewPanel.getPlans()[planIndex];
        const valStepPath = utils_1.ensureAbsoluteGlobalStoragePath(vscode_1.workspace.getConfiguration(configuration_1.CONF_PDDL).get(configuration_1.VAL_STEP_PATH), this.context);
        const valueSeqPath = utils_1.ensureAbsoluteGlobalStoragePath(vscode_1.workspace.getConfiguration(configuration_1.CONF_PDDL).get(configuration_1.VALUE_SEQ_PATH), this.context);
        const valVerbose = vscode_1.workspace.getConfiguration(configuration_1.CONF_PDDL).get(configuration_1.VAL_VERBOSE, false);
        try {
            if (plan.domain && plan.problem) {
                const groupByLifted = vscode_1.workspace.getConfiguration(configuration_1.CONF_PDDL).get(configuration_1.PLAN_REPORT_LINE_PLOT_GROUP_BY_LIFTED, true);
                const evaluator = new ai_planning_val_1.PlanFunctionEvaluator(plan, { valStepPath, valueSeqPath, shouldGroupByLifted: groupByLifted, verbose: valVerbose });
                if (evaluator.isAvailable()) {
                    try {
                        const functionValues = await evaluator.evaluate();
                        functionValues.forEach((values, liftedVariable) => {
                            let chartTitleWithUnit = values.legend.length > 1 ? liftedVariable.name : liftedVariable.getFullName();
                            if (liftedVariable.getUnit()) {
                                chartTitleWithUnit += ` [${liftedVariable.getUnit()}]`;
                            }
                            this.sendLinePlotData(previewPanel, {
                                planIndex: planIndex,
                                name: chartTitleWithUnit,
                                unit: liftedVariable.getUnit(),
                                legend: values.legend,
                                data: values.values
                            });
                        });
                        // add one plot for declared metric
                        // todo: use valstep's ability to handle metric directly using the $metrics variable name
                        for (let metricIndex = 0; metricIndex < plan.problem.getMetrics().length; metricIndex++) {
                            const metric = plan.problem.getMetrics()[metricIndex];
                            const metricExpression = metric.getExpression();
                            if (metricExpression instanceof pddl_workspace_1.VariableExpression) {
                                if (metricExpression.name === "total-time") {
                                    continue;
                                }
                            }
                            const metricValues = await evaluator.evaluateExpression(metric.getExpression());
                            const chartTitle = (_a = metric.getDocumentation()[metric.getDocumentation().length - 1]) !== null && _a !== void 0 ? _a : "Metric";
                            this.sendLinePlotData(previewPanel, {
                                planIndex: planIndex,
                                name: chartTitle,
                                unit: "",
                                legend: [''],
                                data: metricValues.values
                            });
                        }
                    }
                    catch (err) {
                        console.error(err);
                        const valStepPath = evaluator.getValStepPath();
                        if (valStepPath) {
                            valStepErrorHandler_1.handleValStepError(err, valStepPath);
                        }
                    }
                }
                else {
                    previewPanel.getPanel().webview.postMessage({
                        "command": "setVisibility",
                        "elementId": "downloadVal",
                        "visible": true
                    });
                }
            }
            else {
                previewPanel.getPanel().webview.postMessage({
                    "command": "setVisibility",
                    "elementId": "domainProblemAssociation",
                    "visible": true
                });
            }
        }
        finally {
            // in case there is no data to display, we must send a message that clears the loading progress indicator.
            previewPanel.getPanel().webview.postMessage({
                "command": "hideLinePlotLoadingProgress",
                "planIndex": planIndex
            });
        }
    }
    sendLinePlotData(previewPanel, data) {
        previewPanel.getPanel().webview.postMessage({
            "command": "showLinePlot",
            "data": data
        });
    }
    async getFinalStateData(previewPanel, planIndex) {
        if (planIndex >= previewPanel.getPlans().length) {
            console.error(`requesting data for plan index {} while there are only {} plans`, planIndex, previewPanel.getPlans().length);
            return;
        }
        const plan = previewPanel.getPlans()[planIndex];
        const valStepPath = utils_1.ensureAbsoluteGlobalStoragePath(vscode_1.workspace.getConfiguration(configuration_1.CONF_PDDL).get(configuration_1.VAL_STEP_PATH), this.context);
        const valVerbose = vscode_1.workspace.getConfiguration(configuration_1.CONF_PDDL).get(configuration_1.VAL_VERBOSE, false);
        if (plan.domain && plan.problem) {
            try {
                const finalStateValues = await new ai_planning_val_1.PlanEvaluator().evaluatePlan(plan, { valStepPath, verbose: valVerbose });
                if (finalStateValues) {
                    const data = {
                        finalState: finalStateValues.map(tvv => tvv.getVariableValue()),
                        planIndex: planIndex
                    };
                    previewPanel.getPanel().webview.postMessage({
                        "command": "visualizeFinalState",
                        "data": data
                    });
                }
            }
            catch (err) {
                console.error(err);
                if (valStepPath) {
                    valStepErrorHandler_1.handleValStepError(err, valStepPath);
                }
            }
        }
    }
    async showMenu(previewPanel) {
        await new Menu_1.Menu([
            {
                label: "$(browser) Generate plan report",
                detail: "Creates a self-contained HTML file and opens it in a default browser",
                command: exports.PDDL_GENERATE_PLAN_REPORT,
                args: [previewPanel.getPlans(), previewPanel.getSelectedPlanIndex()]
            },
            {
                label: "$(file) Export as .plan file...",
                detail: "Opens a file picker to confirm the name and location of the plan file.",
                command: exports.PDDL_EXPORT_PLAN,
                args: [previewPanel.getSelectedPlan()]
            },
            {
                label: "$(beaker) Creates PDDL test with expected test assertion",
                detail: "Saves this plan into PDDL Test as expected plan assertion.",
                command: exports.PDDL_SAVE_AS_EXPECTED_PLAN,
                args: [previewPanel.getSelectedPlan()]
            },
            {
                label: "$(arrow-both) Change width...",
                detail: "Select the width for the plan preview (in pixels)",
                command: configuration_1.PDDL_CONFIGURE_COMMAND,
                args: [configuration_1.CONF_PDDL + '.' + configuration_1.PLAN_REPORT_WIDTH]
            },
            {
                label: "$(arrow-both) $(browser) Change report width...",
                detail: "Select the width of the next exported report (in pixels)",
                command: configuration_1.PDDL_CONFIGURE_COMMAND,
                args: ["pddl.planReport.exportWidth"]
            }
        ], { placeHolder: 'Select an action...' }).show();
    }
}
exports.PlanView = PlanView;
PlanView.PLANNER_OUTPUT_URI = vscode_1.Uri.parse("pddl://planner/output");
async function getPlanDocument(dotDocumentUri) {
    if (dotDocumentUri) {
        return await vscode_1.workspace.openTextDocument(dotDocumentUri);
    }
    else {
        if (vscode_1.window.activeTextEditor !== undefined && workspaceUtils_1.isPlan(vscode_1.window.activeTextEditor.document)) {
            return vscode_1.window.activeTextEditor.document;
        }
        else {
            return undefined;
        }
    }
}
class PlanPreviewPanel {
    constructor(uri, panel) {
        this.uri = uri;
        this.panel = panel;
        this.needsRebuild = false;
        this.width = 200;
        this.selectedPlanIndex = 0;
        this.plans = [];
    }
    setWidth(width) {
        this.width = width;
    }
    getWidth() {
        return this.width;
    }
    setSelectedPlanIndex(selectedPlan) {
        this.selectedPlanIndex = selectedPlan;
    }
    getSelectedPlanIndex() {
        return this.selectedPlanIndex;
    }
    getSelectedPlan() {
        if (this.plans.length > 0) {
            return this.plans[this.selectedPlanIndex];
        }
        else {
            return undefined;
        }
    }
    setPlans(plans) {
        this.plans = plans;
        this.selectedPlanIndex = plans ? plans.length - 1 : 0;
        this.error = undefined;
        this.setNeedsRebuild(true);
    }
    setError(ex) {
        this.error = ex;
    }
    getError() {
        return this.error;
    }
    getPlans() {
        return this.plans;
    }
    reveal(displayColumn) {
        this.panel.reveal(displayColumn);
    }
    setNeedsRebuild(needsRebuild) {
        this.needsRebuild = needsRebuild;
    }
    getNeedsRebuild() {
        return this.needsRebuild;
    }
    getPanel() {
        return this.panel;
    }
}
//# sourceMappingURL=PlanView.js.map