/* --------------------------------------------------------------------------------------------
 * Copyright (c) Jan Dolejsi. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for license information.
 * ------------------------------------------------------------------------------------------ */
'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
exports.PlannerExecutableFactory = exports.PlannerExecutable = void 0;
/* eslint-disable @typescript-eslint/no-explicit-any */
const vscode_1 = require("vscode");
const process = require("child_process");
const treeKill = require("tree-kill");
const pddl_workspace_1 = require("pddl-workspace");
const ai_planning_val_1 = require("ai-planning-val");
/** Planner implemented as an executable process, which outputs through VS Code facilities. */
class PlannerExecutable extends pddl_workspace_1.planner.Planner {
    constructor(plannerPath, configuration, providerConfiguration) {
        var _a, _b, _c;
        super(plannerPath, configuration, providerConfiguration);
        this.configuration = configuration;
        this._exited = new vscode_1.EventEmitter();
        this.plannerSyntax = (_b = (_a = configuration.plannerSyntax) !== null && _a !== void 0 ? _a : providerConfiguration.configuration.syntax) !== null && _b !== void 0 ? _b : PlannerExecutable.DEFAULT_SYNTAX;
        this.plannerOptions = (_c = configuration.options) !== null && _c !== void 0 ? _c : '';
    }
    get onExited() {
        return this._exited.event;
    }
    dispose() {
        this._exited.dispose();
    }
    get requiresKeyboardInput() {
        return false;
    }
    get supportsSearchDebugger() {
        return true;
    }
    async plan(domainFileInfo, problemFileInfo, planParser, callbacks) {
        const domainFilePath = await ai_planning_val_1.Util.toPddlFile("domain", domainFileInfo.getText());
        const problemFilePath = await ai_planning_val_1.Util.toPddlFile("problem", problemFileInfo.getText());
        let command = this.plannerSyntax
            .replace('$(planner)', pddl_workspace_1.utils.Util.q(this.plannerPath))
            .replace('$(options)', this.plannerOptions)
            .replace('$(domain)', pddl_workspace_1.utils.Util.q(domainFilePath))
            .replace('$(problem)', pddl_workspace_1.utils.Util.q(problemFilePath));
        command += ' ' + callbacks.providePlannerOptions({ domain: domainFileInfo, problem: problemFileInfo }).join(' ');
        callbacks.handleOutput(command + '\n');
        const thisPlanner = this;
        if (vscode_1.workspace.getConfiguration("pddlPlanner").get("executionTarget") === "Terminal") {
            return new Promise((resolve) => {
                const terminal = vscode_1.window.createTerminal({ name: "Planner output", cwd: thisPlanner.configuration.workingDirectory });
                terminal.sendText(command, true);
                terminal.show(true);
                const plans = [];
                resolve(plans);
            });
        }
        return new Promise(function (resolve, reject) {
            var _a, _b;
            thisPlanner.child = process.exec(command, { cwd: thisPlanner.configuration.workingDirectory }, (error) => {
                var _a;
                planParser.onPlanFinished();
                if (error && !((_a = thisPlanner.child) === null || _a === void 0 ? void 0 : _a.killed) && !thisPlanner.planningProcessKilled) {
                    reject(error); // todo: should calle `return` here?
                }
                const plans = planParser.getPlans();
                resolve(plans); // todo: should we resolve() even if we reject()ed above?
                thisPlanner.child = undefined;
            });
            (_a = thisPlanner.child.stdout) === null || _a === void 0 ? void 0 : _a.on('data', (data) => {
                const dataString = data.toString();
                callbacks.handleOutput(dataString);
                planParser.appendBuffer(dataString);
            });
            (_b = thisPlanner.child.stderr) === null || _b === void 0 ? void 0 : _b.on('data', (data) => callbacks.handleOutput("Error: " + data));
            thisPlanner.child.on("close", (code, signal) => {
                if (code) {
                    console.log("Exit code: " + code);
                }
                if (signal) {
                    console.log("Exit Signal: " + signal);
                }
                thisPlanner._exited.fire(code);
            });
        });
    }
    /**
     * When the UI button is pressed, the planner is forced to stop.
     */
    stop() {
        if (this.child) {
            super.stop();
            // try to kill just the shell
            // this.child.kill();//'SIGINT');
            // this.child.stdin.pause();
            treeKill(this.child.pid);
        }
    }
}
exports.PlannerExecutable = PlannerExecutable;
PlannerExecutable.DEFAULT_SYNTAX = "$(planner) $(options) $(domain) $(problem)";
/** Creates instances of the PlannerExecutable, so other extensions could wrap them. */
class PlannerExecutableFactory {
    /**
     * Creates new instance of `PlannerExecutable`.
     * @param plannerPath planner path
     * @param plannerRunConfiguration run configuration
     * @param providerConfiguration provider configuration
     * @returns planner executable that VS Code will call the `plan()` method on.
     */
    createPlannerExecutable(plannerPath, plannerRunConfiguration, providerConfiguration) {
        return new PlannerExecutable(plannerPath, plannerRunConfiguration, providerConfiguration);
    }
}
exports.PlannerExecutableFactory = PlannerExecutableFactory;
//# sourceMappingURL=PlannerExecutable.js.map