/* --------------------------------------------------------------------------------------------
 * Copyright (c) Jan Dolejsi 2020. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for license information.
 * ------------------------------------------------------------------------------------------ */
'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
exports.ManifestGenerator = void 0;
const TestsManifest_1 = require("./TestsManifest");
const path_1 = require("path");
const PTestTreeDataProvider_1 = require("./PTestTreeDataProvider");
const Test_1 = require("./Test");
const vscode_1 = require("vscode");
const utils_1 = require("../utils");
class ManifestGenerator {
    constructor(pddlWorkspace, context) {
        this.pddlWorkspace = pddlWorkspace;
        this.context = context;
    }
    async generateAll() {
        const domainFiles = this.pddlWorkspace
            .getAllFilesIf(fileInfo => fileInfo.isDomain())
            .filter(domain => this.pddlWorkspace.getProblemFiles(domain).length);
        return await Promise.all(domainFiles.map(async (domain) => {
            const manifest = await this.readOrCreateManifest(domain, this.context);
            this.addProblems(manifest, domain);
            manifest.store();
            return manifest;
        }));
    }
    async createPlanAssertion(plan) {
        if (!plan.domain || !plan.problem) {
            throw new Error(`Plan has no domain or problem associated.`);
        }
        const manifest = await this.readOrCreateManifest(plan.domain, this.context);
        const { filePath: domainPath } = this.getPathAndName(plan.domain);
        const { fileName: problemFileName } = this.getPathAndName(plan.problem);
        const planFileName = `${problemFileName}_${new Date().toISOString().split(':').join('-')}.plan`;
        const planPath = path_1.join(path_1.dirname(domainPath), planFileName);
        await vscode_1.workspace.fs.writeFile(vscode_1.Uri.file(planPath), Buffer.from(plan.getText(), 'utf8'));
        const planRelativePath = path_1.relative(path_1.dirname(manifest.uri.fsPath), planPath);
        this.upsertProblemCase(plan.problem, plan.domain, manifest, [planRelativePath]);
        await manifest.store();
        return manifest;
    }
    async readOrCreateManifest(domain, context) {
        const { filePath: domainPath, fileName: domainFileName } = this.getPathAndName(domain);
        const domainFileNameWithoutExt = path_1.basename(domainPath, path_1.extname(domainPath));
        const manifestUri = vscode_1.Uri.file(path_1.join(path_1.dirname(domainPath), domainFileNameWithoutExt + PTestTreeDataProvider_1.PTestTreeDataProvider.PTEST_SUFFIX));
        if (await utils_1.fileOrFolderExists(manifestUri)) {
            const manifestText = await vscode_1.workspace.fs.readFile(manifestUri);
            const manifestJson = JSON.parse(manifestText.toString());
            return TestsManifest_1.TestsManifest.fromJSON(manifestUri.fsPath, manifestJson, context);
        }
        else {
            const manifest = new TestsManifest_1.TestsManifest(domainFileName, undefined, '', manifestUri);
            await manifest.store();
            return manifest;
        }
    }
    getPathAndName(fileInfo) {
        const domainPath = fileInfo.fileUri.fsPath;
        const domainFileName = path_1.basename(domainPath);
        return { filePath: domainPath, fileName: domainFileName };
    }
    addProblems(manifest, domain) {
        const problems = this.pddlWorkspace.getProblemFiles(domain);
        problems.forEach(problem => {
            this.upsertProblemCase(problem, domain, manifest);
        });
    }
    upsertProblemCase(problem, domain, manifest, expectedPlans) {
        const { filePath: domainPath } = this.getPathAndName(domain);
        const { filePath: problemPath } = this.getPathAndName(problem);
        const problemRelativePath = path_1.relative(path_1.dirname(domainPath), problemPath);
        const testCaseFound = manifest.testCases
            .find(c => c.getProblemUri().toString() === problem.fileUri.toString());
        if (!testCaseFound) {
            const newTestCase = new Test_1.Test(undefined, undefined, undefined, problemRelativePath, undefined, undefined, expectedPlans);
            // only add if it did not exist
            manifest.addCase(newTestCase);
            return newTestCase;
        }
        else {
            if (expectedPlans) {
                expectedPlans
                    .filter(expectedPlan => !testCaseFound.getExpectedPlans().includes(expectedPlan))
                    .forEach(expectedPlan => testCaseFound.getExpectedPlans().push(expectedPlan));
            }
            return testCaseFound;
        }
    }
}
exports.ManifestGenerator = ManifestGenerator;
//# sourceMappingURL=ManifestGenerator.js.map