/* --------------------------------------------------------------------------------------------
 * Copyright (c) Jan Dolejsi. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for license information.
 * ------------------------------------------------------------------------------------------ */
'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
exports.PTestReportView = void 0;
const vscode_1 = require("vscode");
const path = require("path");
const webviewUtils_1 = require("../webviewUtils");
const PTestCommands_1 = require("./PTestCommands");
/** Visualizes PTest results on a web view panel. */
class PTestReportView {
    constructor(context, report) {
        this.context = context;
        this.report = report;
        this.CONTENT_FOLDER = path.join("views", "ptestReport");
    }
    async showReport() {
        if (this.webViewPanel) {
            this.webViewPanel.reveal();
        }
        else {
            this.createPage();
        }
    }
    async createPage() {
        const iconUri = this.context.asAbsolutePath('images/icon.png');
        this.webViewPanel = vscode_1.window.createWebviewPanel("pddl.test.report", "Test Report", {
            viewColumn: vscode_1.ViewColumn.Active,
            preserveFocus: false
        }, {
            retainContextWhenHidden: true,
            enableFindWidget: true,
            enableCommandUris: true,
            enableScripts: true,
            localResourceRoots: [vscode_1.Uri.file(this.context.asAbsolutePath(this.CONTENT_FOLDER))]
        });
        this.webViewPanel.iconPath = vscode_1.Uri.file(iconUri);
        this.webViewPanel.onDidDispose(() => this.webViewPanel = undefined, undefined, this.context.subscriptions);
        this.webViewPanel.webview.onDidReceiveMessage(message => this.handleMessage(message), undefined, this.context.subscriptions);
        this.webViewPanel.onDidChangeViewState(() => this.updatePage());
        this.context.subscriptions.push(this.webViewPanel);
        // set up the view with relevant data
        this.updatePage();
    }
    async updatePage() {
        if (this.webViewPanel) {
            const html = await this.getHtml(this.webViewPanel.webview);
            this.webViewPanel.webview.html = html;
        }
    }
    async getHtml(webview) {
        let html = await webviewUtils_1.getWebViewHtml(this.context, {
            allowUnsafeInlineScripts: true,
            relativePath: this.CONTENT_FOLDER,
            htmlFileName: 'page.html'
        }, webview);
        const tableRowsHtml = this.report.getManifests().map(manifest => this.renderManifestRow(manifest));
        html = html.replace("TABLE", tableRowsHtml.join('\n'));
        return html;
    }
    renderManifestRow(manifest) {
        const manifestLocation = vscode_1.workspace.asRelativePath(manifest.path, true);
        const manifestLink = `<a  href="#" onclick="openManifest('${manifest.uri.toString()}')" title="Open test manifest.">&#128065;</a>`;
        const manifestHeader = `<tr><td colspan="4" class="manifestRow">${manifestLocation} ${manifestLink}</td></tr>`;
        const testRows = this.report.getTestCases(manifest).map(test => this.renderTestRow(test));
        return [manifestHeader].concat(testRows).join('\n');
    }
    renderTestRow(test) {
        var _a;
        const testResult = this.report.getTestResultOrThrow(test);
        const elapsedTime = testResult.elapsedTime ? `${(testResult.elapsedTime / 1000).toFixed(2)}` : '';
        const viewTestLink = `<a  href="#" onclick="openTest('${test.getUriOrThrow().toString()}')" title="Open test case.">&#128065;</a>`;
        return `<tr><td>${test.getLabel()} ${viewTestLink}</td><td>${testResult.outcomeChar}</td><td>${elapsedTime}</td><td>${(_a = testResult.error) !== null && _a !== void 0 ? _a : ""}</td></tr>`;
    }
    // eslint-disable-next-line @typescript-eslint/no-explicit-any
    async handleMessage(message) {
        console.log(`Message received from the webview: ${message.command}`);
        switch (message.command) {
            case 'openManifest':
                const manifestDoc = await vscode_1.workspace.openTextDocument(vscode_1.Uri.parse(message.value));
                await vscode_1.window.showTextDocument(manifestDoc, { viewColumn: vscode_1.ViewColumn.Beside });
                break;
            case 'openTest':
                vscode_1.commands.executeCommand(PTestCommands_1.PTEST_VIEW, vscode_1.Uri.parse(message.value));
                break;
            default:
                console.warn('Unexpected command: ' + message.command);
        }
    }
}
exports.PTestReportView = PTestReportView;
//# sourceMappingURL=PTestReportView.js.map