/* --------------------------------------------------------------------------------------------
 * Copyright (c) Jan Dolejsi. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for license information.
 * ------------------------------------------------------------------------------------------ */
'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
exports.PTestTreeDataProvider = exports.PTestNodeKind = void 0;
const vscode_1 = require("vscode");
const path_1 = require("path");
const pddl_workspace_1 = require("pddl-workspace");
const TestsManifest_1 = require("./TestsManifest");
const Test_1 = require("./Test");
const utils_1 = require("../utils");
var PTestNodeKind;
(function (PTestNodeKind) {
    PTestNodeKind[PTestNodeKind["Directory"] = 0] = "Directory";
    PTestNodeKind[PTestNodeKind["Manifest"] = 1] = "Manifest";
    PTestNodeKind[PTestNodeKind["Test"] = 2] = "Test";
})(PTestNodeKind = exports.PTestNodeKind || (exports.PTestNodeKind = {}));
class PTestTreeDataProvider {
    constructor(context) {
        this.context = context;
        // fire event with 'undefined' payload to refresh he entire tree
        this._onDidChange = new vscode_1.EventEmitter();
        this.testResults = new Map();
        this.treeNodeCache = new Map();
    }
    get onDidChangeTreeData() { return this._onDidChange.event; }
    refresh() {
        this.testResults.clear();
        this.treeNodeCache.clear();
        // update all visible tree nodes
        this._onDidChange.fire(undefined);
    }
    getTestOutcome(testUri) {
        var _a;
        return (_a = this.testResults.get(testUri.toString())) !== null && _a !== void 0 ? _a : Test_1.TestOutcome.UNKNOWN;
    }
    setTestOutcome(test, testOutcome) {
        this.testResults.set(test.getUriOrThrow().toString(), testOutcome);
        const node = this.findNodeByResource(test.getUriOrThrow());
        // the node may not exist, if the tree hasn't been expanded yet
        node && this._onDidChange.fire(node);
    }
    findNodeByResourceOrThrow(resource) {
        return utils_1.assertDefined(this.findNodeByResourceOrThrow(resource), `No node for ${resource.toString()}`);
    }
    findNodeByResource(resource) {
        return this.treeNodeCache.get(resource.toString());
    }
    cache(node) {
        this.treeNodeCache.set(node.resource.toString(), node);
        return node;
    }
    getTreeItem(element) {
        let icon;
        let contextValue;
        if (element.kind === PTestNodeKind.Directory) {
            icon = 'folder' + '.svg';
            contextValue = 'folder';
        }
        else if (element.kind === PTestNodeKind.Manifest) {
            icon = 'beaker' + '.svg';
            contextValue = 'manifest';
        }
        else {
            const testOutcome = this.getTestOutcome(element.resource);
            switch (testOutcome) {
                case Test_1.TestOutcome.UNKNOWN:
                    icon = 'exclamation';
                    break;
                case Test_1.TestOutcome.SUCCESS:
                    icon = 'checked';
                    break;
                case Test_1.TestOutcome.FAILED:
                    icon = 'error';
                    break;
                case Test_1.TestOutcome.SKIPPED:
                    icon = 'skipped';
                    break;
                case Test_1.TestOutcome.IN_PROGRESS:
                    icon = 'progress';
                    break;
                default:
                    icon = 'interrogation';
            }
            icon += '.svg';
            contextValue = 'test';
        }
        const isCollapsible = element.kind === PTestNodeKind.Directory || element.kind === PTestNodeKind.Manifest;
        return {
            id: element.resource.toString(),
            resourceUri: element.resource,
            collapsibleState: isCollapsible ? vscode_1.TreeItemCollapsibleState.Collapsed : void 0,
            contextValue: contextValue,
            label: element.label,
            iconPath: this.getIcon(icon),
            tooltip: element.tooltip
        };
    }
    getIcon(fileName) {
        if (!fileName) {
            return undefined;
        }
        return {
            light: this.context.asAbsolutePath(path_1.join('images', 'light', fileName)),
            dark: this.context.asAbsolutePath(path_1.join('images', 'dark', fileName))
        };
    }
    async getChildren(element) {
        if (!element) {
            if (vscode_1.workspace.workspaceFolders) {
                const manifestPromises = vscode_1.workspace.workspaceFolders
                    .map(async (wf) => await this.findManifests(wf, 1));
                const manifestUris = pddl_workspace_1.utils.Util.flatMap(await Promise.all(manifestPromises));
                const rootNodes = vscode_1.workspace.workspaceFolders
                    .filter(wf => this.containsManifest(wf.uri.fsPath, manifestUris))
                    .map(wf => this.cache({ resource: wf.uri, kind: PTestNodeKind.Directory }));
                // if empty, will show the welcome message
                return rootNodes;
            }
            else {
                return [];
            }
        }
        else {
            const parentUri = element.resource;
            const parentPath = parentUri.fsPath;
            if (PTestTreeDataProvider.isTestManifest(parentPath)) {
                const manifest = this.tryLoadManifest(parentPath);
                if (!manifest) {
                    return [];
                }
                // return test cases
                return manifest.testCases
                    .map(test => this.cache({
                    resource: test.getUriOrThrow(),
                    kind: PTestNodeKind.Test,
                    label: test.getLabel(),
                    tooltip: test.getDescription()
                }));
            }
            else {
                const nestedManifestUris = await this.findManifests(parentPath, 1000);
                const directoryContent = await vscode_1.workspace.fs.readDirectory(parentUri);
                return directoryContent.map(childAndType => {
                    const [child, childType] = childAndType;
                    const childPath = path_1.join(parentPath, child);
                    return this.toCachedNode(childPath, childType, nestedManifestUris);
                })
                    .filter(node => !!node)
                    .map(node => node);
            }
        }
    }
    toCachedNode(childPath, childType, nestedManifestUris) {
        if (childType === vscode_1.FileType.File) {
            if (PTestTreeDataProvider.isTestManifest(childPath)) {
                let label = childPath;
                const baseName = path_1.basename(childPath);
                if (baseName.length === PTestTreeDataProvider.PTEST_SUFFIX.length) {
                    label = 'Test cases';
                }
                else {
                    label = baseName.substring(0, baseName.length - PTestTreeDataProvider.PTEST_SUFFIX.length);
                }
                return this.cache({
                    resource: vscode_1.Uri.file(childPath),
                    kind: PTestNodeKind.Manifest,
                    label: label
                });
            }
            else {
                return undefined; // file, but not manifest
            }
        }
        else if (childType === vscode_1.FileType.Directory) {
            if (this.containsManifest(childPath, nestedManifestUris)) {
                return this.cache({
                    resource: vscode_1.Uri.file(childPath),
                    kind: PTestNodeKind.Directory
                });
            }
            else {
                return undefined; // does not contain any manifests
            }
        }
        else {
            return undefined; // not following symbolic links
        }
    }
    containsManifest(folderPath, manifestUris) {
        return manifestUris
            .some(uri => uri.fsPath.startsWith(folderPath));
    }
    async findManifests(folder, maxEntries = 1) {
        return await vscode_1.workspace.findFiles(new vscode_1.RelativePattern(folder, '**/*' + PTestTreeDataProvider.PTEST_SUFFIX), '.git/', maxEntries);
    }
    tryLoadManifest(manifestPath) {
        try {
            return TestsManifest_1.TestsManifest.load(manifestPath, this.context);
        }
        catch (error) {
            vscode_1.window.showErrorMessage(`Unable to load test manifest from: ${manifestPath} 
${error}`);
            return undefined;
        }
    }
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    getParent(_element) {
        throw new Error("Method not implemented.");
    }
    static isTestManifest(filePath) {
        return path_1.basename(filePath).toLowerCase().endsWith(this.PTEST_SUFFIX);
    }
}
exports.PTestTreeDataProvider = PTestTreeDataProvider;
PTestTreeDataProvider.PTEST_SUFFIX = '.ptest.json';
//# sourceMappingURL=PTestTreeDataProvider.js.map