/* --------------------------------------------------------------------------------------------
 * Copyright (c) Jan Dolejsi. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for license information.
 * ------------------------------------------------------------------------------------------ */
'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
exports.Test = exports.TestOutcome = void 0;
const vscode_1 = require("vscode");
const path_1 = require("path");
const TestsManifest_1 = require("./TestsManifest");
const pddl_workspace_1 = require("pddl-workspace");
const utils_1 = require("../utils");
var TestOutcome;
(function (TestOutcome) {
    TestOutcome[TestOutcome["UNKNOWN"] = 0] = "UNKNOWN";
    TestOutcome[TestOutcome["SUCCESS"] = 1] = "SUCCESS";
    TestOutcome[TestOutcome["FAILED"] = 2] = "FAILED";
    TestOutcome[TestOutcome["SKIPPED"] = 3] = "SKIPPED";
    TestOutcome[TestOutcome["IN_PROGRESS"] = 4] = "IN_PROGRESS";
})(TestOutcome = exports.TestOutcome || (exports.TestOutcome = {}));
const EXPECTED_PLANS = "expectedPlans";
const LABEL = "label";
const DESCRIPTION = "description";
const DOMAIN = "domain";
const PROBLEM = "problem";
const OPTIONS = "options";
const PRE_PROCESSOR = "preProcess";
const PRE_PROCESSOR_KIND = "kind";
const PRE_PROCESSOR_SCRIPT = "script";
const PRE_PROCESSOR_DATA = "data";
const PRE_PROCESSOR_ARGS = "args";
/**
 * Test definitions
 */
class Test {
    constructor(label, description, domain, problem, options, preProcessor, expectedPlans) {
        this.label = label;
        this.description = description;
        this.domain = domain;
        this.problem = problem;
        this.options = options;
        this.preProcessor = preProcessor;
        this.expectedPlans = expectedPlans;
    }
    // eslint-disable-next-line @typescript-eslint/no-explicit-any
    static fromJSON(json, context) {
        var _a;
        const label = json[LABEL];
        const description = json[DESCRIPTION];
        const domain = json[DOMAIN];
        const problem = json[PROBLEM];
        const options = json[OPTIONS];
        const expectedPlans = (_a = json[EXPECTED_PLANS]) !== null && _a !== void 0 ? _a : [];
        const preProcessSettings = json[PRE_PROCESSOR];
        let preProcessor;
        if (preProcessSettings) {
            const kind = preProcessSettings[PRE_PROCESSOR_KIND];
            switch (kind) {
                case "command":
                    preProcessor = pddl_workspace_1.CommandPreProcessor.fromJson(preProcessSettings);
                    break;
                case "python":
                    preProcessor = new pddl_workspace_1.PythonPreProcessor(context.pythonPath(), preProcessSettings[PRE_PROCESSOR_SCRIPT], preProcessSettings[PRE_PROCESSOR_ARGS]);
                    break;
                case "nunjucks":
                    preProcessor = new pddl_workspace_1.NunjucksPreProcessor(preProcessSettings[PRE_PROCESSOR_DATA], undefined, 0, false);
                    break;
                case "jinja2":
                    preProcessor = new pddl_workspace_1.Jinja2PreProcessor(context.pythonPath(), context.extensionPath, preProcessSettings[PRE_PROCESSOR_DATA]);
                    break;
            }
        }
        return new Test(label, description, domain, problem, options, preProcessor, expectedPlans);
    }
    // eslint-disable-next-line @typescript-eslint/no-explicit-any
    toJSON() {
        var _a;
        // eslint-disable-next-line @typescript-eslint/no-explicit-any
        const json = {};
        if (this.label) {
            json[LABEL] = this.label;
        }
        if (this.description) {
            json[DESCRIPTION] = this.description;
        }
        if (this.domain) {
            json[DOMAIN] = this.domain;
        }
        if (this.problem) {
            json[PROBLEM] = this.problem;
        }
        if (this.options) {
            json[OPTIONS] = this.options;
        }
        if ((_a = this.expectedPlans) === null || _a === void 0 ? void 0 : _a.length) {
            json[EXPECTED_PLANS] = this.expectedPlans;
        }
        if (this.preProcessor) {
            json[PRE_PROCESSOR] = { kind: "unsupported" };
        } // creating test cases with pre-processing is currently not supported
        return json;
    }
    /**
     * @returns index of this test case in the manifest
     */
    getIndex() {
        return this.index;
    }
    getManifest() {
        return this.manifest;
    }
    setManifest(manifest) {
        this.index = manifest.testCases.length;
        this.uri = manifest.uri.with({ fragment: this.index.toString() });
        this.manifest = manifest;
    }
    getDomain() {
        var _a, _b, _c;
        return (_c = (_a = this.domain) !== null && _a !== void 0 ? _a : (_b = this.manifest) === null || _b === void 0 ? void 0 : _b.defaultDomain) !== null && _c !== void 0 ? _c : utils_1.throwForUndefined('domain');
    }
    getDomainUri() {
        return vscode_1.Uri.file(this.toAbsolutePath(this.getDomain()));
    }
    getProblem() {
        var _a, _b, _c;
        return (_c = (_a = this.problem) !== null && _a !== void 0 ? _a : (_b = this.manifest) === null || _b === void 0 ? void 0 : _b.defaultProblem) !== null && _c !== void 0 ? _c : utils_1.throwForUndefined("problem");
    }
    getProblemUri() {
        return vscode_1.Uri.file(this.toAbsolutePath(this.getProblem()));
    }
    getUri() {
        return this.uri;
    }
    getUriOrThrow() {
        if (!this.uri) {
            throw new Error(`Test ${this.getLabel()} has no URI.`);
        }
        return this.uri;
    }
    getLabel() {
        var _a, _b, _c;
        return (_b = (_a = this.label) !== null && _a !== void 0 ? _a : this.problem) !== null && _b !== void 0 ? _b : this.getProblem() + ` (${((_c = this.index) !== null && _c !== void 0 ? _c : 0) + 1})`;
    }
    getDescription() {
        return this.description;
    }
    getOptions() {
        var _a, _b;
        return (_a = this.options) !== null && _a !== void 0 ? _a : (_b = this.manifest) === null || _b === void 0 ? void 0 : _b.defaultOptions;
    }
    getPreProcessor() {
        return this.preProcessor;
    }
    hasExpectedPlans() {
        return this.expectedPlans !== undefined && this.expectedPlans.length > 0;
    }
    getExpectedPlans() {
        var _a;
        return (_a = this.expectedPlans) !== null && _a !== void 0 ? _a : [];
    }
    toAbsolutePath(fileName) {
        if (!this.manifest) {
            throw new Error(`Test ${this.getLabel()} is not associated to a manifest.`);
        }
        return path_1.join(path_1.dirname(this.manifest.path), fileName);
    }
    static fromUri(uri, context) {
        const testIndex = parseInt(uri.fragment);
        if (Number.isFinite(testIndex)) {
            const manifest = TestsManifest_1.TestsManifest.load(uri.fsPath, context);
            return manifest.testCases[testIndex];
        }
        else {
            return undefined;
        }
    }
}
exports.Test = Test;
//# sourceMappingURL=Test.js.map