/* --------------------------------------------------------------------------------------------
 * Copyright (c) Jan Dolejsi. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for license information.
 * ------------------------------------------------------------------------------------------ */
'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
exports.MessageParser = void 0;
const pddl_workspace_1 = require("pddl-workspace");
class MessageParser {
    constructor(search, stateIdPattern) {
        this.search = search;
        this.stateIdPattern = stateIdPattern;
        this.lastStateOrder = -1;
        this.stateIdToOrder = new Map();
    }
    clear() {
        this.stateIdToOrder.clear();
        this.lastStateOrder = -1;
    }
    parseStateId(origId) {
        let assignedStateId;
        if (this.stateIdPattern) {
            this.stateIdPattern.lastIndex = 0;
            let match;
            if (match = origId.match(this.stateIdPattern)) {
                assignedStateId = parseInt(match[1]);
            }
            else {
                console.log("State ID does not conform to the pattern. Assigning own ID.");
                assignedStateId = ++this.lastStateOrder;
            }
        }
        else {
            assignedStateId = ++this.lastStateOrder;
        }
        this.stateIdToOrder.set(origId, assignedStateId);
        return assignedStateId;
    }
    // eslint-disable-next-line @typescript-eslint/no-explicit-any
    parseInitialState(state) {
        const assignedStateId = this.parseStateId(state.id);
        this.stateIdToOrder.set(state.id, assignedStateId);
        return new pddl_workspace_1.search.SearchState(assignedStateId, state.id, state.g, state.earliestTime, [], state.landmarks);
    }
    // eslint-disable-next-line @typescript-eslint/no-explicit-any
    parseState(state) {
        let assignedStateId = this.stateIdToOrder.get(state.id);
        if (assignedStateId === undefined) {
            assignedStateId = this.parseStateId(state.id);
        }
        const parentId = this.stateIdToOrder.get(state.parentId);
        // eslint-disable-next-line @typescript-eslint/no-explicit-any
        const planHead = state.planHead.map((h) => this.parseSearchHappening(h, false));
        const actionName = state.appliedAction ? this.createActionName(this.parseSearchHappening(state.appliedAction, false)) : undefined;
        return new pddl_workspace_1.search.SearchState(assignedStateId, state.id, state.g, state.earliestTime, planHead, state.landmarks, parentId, actionName);
    }
    createActionName(lastHappening) {
        let actionName = lastHappening.actionName;
        if (lastHappening.shotCounter > 0) {
            actionName += `[${lastHappening.shotCounter}]`;
        }
        switch (lastHappening.kind) {
            case pddl_workspace_1.HappeningType.START:
                actionName += '├';
                break;
            case pddl_workspace_1.HappeningType.END:
                actionName += ' ┤';
                break;
        }
        if (lastHappening.iterations > 1) {
            actionName += ` ${lastHappening.iterations}x`;
        }
        return actionName;
    }
    // eslint-disable-next-line @typescript-eslint/no-explicit-any
    parseEvaluatedState(state) {
        const assignedStateId = this.stateIdToOrder.get(state.id);
        if (assignedStateId === undefined) {
            throw new Error(`State with id ${state.id} is unknown`);
        }
        const stateFound = this.search.getState(assignedStateId);
        if (!stateFound) {
            throw new Error(`State with id ${state.id} not found.`);
        }
        if (!state.hasOwnProperty('h')) {
            return stateFound.setDeadEnd();
        }
        else {
            // eslint-disable-next-line @typescript-eslint/no-explicit-any
            const relaxedPlan = state.relaxedPlan.map((h) => this.parseSearchHappening(h, true));
            // eslint-disable-next-line @typescript-eslint/no-explicit-any
            const helpfulActions = state.helpfulActions.map((a) => this.parseHelpfulAction(a));
            return stateFound.evaluate(state.h, state.totalMakespan, helpfulActions, relaxedPlan);
        }
    }
    // eslint-disable-next-line @typescript-eslint/no-explicit-any
    parseSearchHappening(happening, isRelaxed) {
        return {
            actionName: happening.actionName,
            earliestTime: happening.earliestTime,
            shotCounter: happening.shotCounter,
            iterations: happening.iterations,
            kind: this.parseHappeningType(happening.kind),
            isRelaxed: isRelaxed
        };
    }
    // eslint-disable-next-line @typescript-eslint/no-explicit-any
    parseHelpfulAction(action) {
        return {
            actionName: action.actionName,
            kind: this.parseHappeningType(action.kind)
        };
    }
    parseHappeningType(happeningTypeAsString) {
        switch (happeningTypeAsString) {
            case pddl_workspace_1.HappeningType[pddl_workspace_1.HappeningType.END]:
                return pddl_workspace_1.HappeningType.END;
            case pddl_workspace_1.HappeningType[pddl_workspace_1.HappeningType.INSTANTANEOUS]:
                return pddl_workspace_1.HappeningType.INSTANTANEOUS;
            case pddl_workspace_1.HappeningType[pddl_workspace_1.HappeningType.START]:
                return pddl_workspace_1.HappeningType.START;
            case pddl_workspace_1.HappeningType[pddl_workspace_1.HappeningType.TIMED]:
                return pddl_workspace_1.HappeningType.TIMED;
            default:
                throw new Error("Unexpected happening type: " + happeningTypeAsString);
        }
    }
}
exports.MessageParser = MessageParser;
//# sourceMappingURL=MessageParser.js.map