/* --------------------------------------------------------------------------------------------
 * Copyright (c) Jan Dolejsi. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for license information.
 * ------------------------------------------------------------------------------------------ */
'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
exports.Search = void 0;
const vscode_1 = require("vscode");
/** Represents the running/completed search and notifies when new states are added or evaluated. */
class Search {
    constructor() {
        this._onStateAdded = new vscode_1.EventEmitter();
        this._onStateUpdated = new vscode_1.EventEmitter();
        this._onBetterState = new vscode_1.EventEmitter();
        this._onPlanFound = new vscode_1.EventEmitter();
        this.states = new Map();
        this.bestStateHeuristic = Number.MAX_VALUE;
    }
    clear() {
        this.states.clear();
        this.bestStateHeuristic = Number.MAX_VALUE;
    }
    get onStateAdded() {
        return this._onStateAdded.event;
    }
    get onStateUpdated() {
        return this._onStateUpdated.event;
    }
    get onBetterState() {
        return this._onBetterState.event;
    }
    get onPlanFound() {
        return this._onPlanFound.event;
    }
    addInitialState(state) {
        console.log('Added initial: ' + state);
        this.states.set(state.id, state);
        this._onStateAdded.fire(state);
    }
    addState(state) {
        if (this.states.has(state.id)) {
            // todo: this may need modifications to align with state ordering and memoisation
            console.log('Ignoring: ' + state);
        }
        else {
            console.log('Added: ' + state);
            this.states.set(state.id, state);
            this._onStateAdded.fire(state);
        }
    }
    update(state) {
        console.log('Updated: ' + state);
        this.states.set(state.id, state);
        this._onStateUpdated.fire(state);
        if (state.isEvaluated && !state.isDeadEnd && state.h && (state.h < this.bestStateHeuristic)) {
            this.bestStateHeuristic = state.h;
            this._onBetterState.fire(state);
        }
    }
    showPlan(goalState) {
        const plan = [];
        let state = goalState;
        while (state !== undefined) {
            state.isPlan = true;
            plan.push(state);
            state = state.parentId !== undefined ? this.states.get(state.parentId) : undefined;
        }
        this._onPlanFound.fire(plan);
        // goal state is better than previous state, because we assume H==0
        this._onBetterState.fire(goalState);
    }
    setPlan(state) {
        console.log('Plan found: ' + state);
        const goalState = state.evaluate(0, state.earliestTime, [], []);
        this.update(goalState);
        this.showPlan(goalState);
    }
    getState(stateId) {
        return this.states.get(stateId);
    }
    getStates() {
        return [...this.states.values()];
    }
}
exports.Search = Search;
//# sourceMappingURL=Search.js.map