/* --------------------------------------------------------------------------------------------
 * Copyright (c) Jan Dolejsi. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for license information.
 * ------------------------------------------------------------------------------------------ */
'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
exports.SearchDebugger = void 0;
const vscode_1 = require("vscode");
const vscode_extension_telemetry_wrapper_1 = require("vscode-extension-telemetry-wrapper");
const express = require("express");
const bodyParser = require("body-parser");
const http = require("http");
const Search_1 = require("./Search");
const MessageParser_1 = require("./MessageParser");
const MockSearch_1 = require("./MockSearch");
const SearchDebuggerView_1 = require("./SearchDebuggerView");
const configuration_1 = require("../configuration/configuration");
const PlannersConfiguration_1 = require("../configuration/PlannersConfiguration");
class SearchDebugger {
    constructor(context, pddlConfiguration) {
        this.context = context;
        this.pddlConfiguration = pddlConfiguration;
        this.server = null;
        this.port = 0; //port is randomized
        this.CONFIG_PDDL_SEARCH_DEBUGGER = "pddlSearchDebugger";
        this.CONF_DEFAULT_PORT = "defaultPort";
        this.CONF_STATE_ID_PATTERN = "stateIdPattern";
        this.CONFIG_PLANNER_OPTION = "plannerCommandLine";
        this.context.subscriptions.push(vscode_extension_telemetry_wrapper_1.instrumentOperationAsVsCodeCommand("pddl.searchDebugger.start", () => this.tryStart()));
        this.context.subscriptions.push(vscode_extension_telemetry_wrapper_1.instrumentOperationAsVsCodeCommand("pddl.searchDebugger.stop", () => this.tryStop()));
        this.context.subscriptions.push(vscode_extension_telemetry_wrapper_1.instrumentOperationAsVsCodeCommand("pddl.searchDebugger.reset", () => this.reset()));
        this.context.subscriptions.push(vscode_extension_telemetry_wrapper_1.instrumentOperationAsVsCodeCommand("pddl.searchDebugger.mock", () => this.mock()));
        // hen the planner output target changes, disable the search debugger accordingly, so it stops listening
        vscode_1.workspace.onDidChangeConfiguration(e => {
            if (e.affectsConfiguration(configuration_1.PDDL_PLANNER + '.' + PlannersConfiguration_1.EXECUTION_TARGET)) {
                const target = vscode_1.workspace.getConfiguration(configuration_1.PDDL_PLANNER).get(PlannersConfiguration_1.EXECUTION_TARGET, PlannersConfiguration_1.DEF_PLANNER_OUTPUT_TARGET);
                if (this.isRunning() && target !== PlannersConfiguration_1.PLANNER_OUTPUT_TARGET_SEARCH_DEBUGGER) {
                    this.tryStop();
                }
            }
        });
        this.view = new SearchDebuggerView_1.SearchDebuggerView(this.context);
    }
    providePlannerOptions(context) {
        if (this.isRunning()) {
            this.reset();
            if (this.view) {
                this.view.setDomainAndProblem(context.domain, context.problem);
            }
            const commandLine = vscode_1.workspace.getConfiguration(this.CONFIG_PDDL_SEARCH_DEBUGGER).get(this.CONFIG_PLANNER_OPTION);
            if (!commandLine) {
                throw new Error(`Missing planner command-line option configuration: ${this.CONFIG_PDDL_SEARCH_DEBUGGER}.${this.CONFIG_PLANNER_OPTION}`);
            }
            return commandLine.replace('$(port)', this.port.toString());
        }
        else {
            return "";
        }
    }
    reset() {
        if (this.search) {
            this.search.clear();
        }
        if (this.messageParser) {
            this.messageParser.clear();
        }
        if (this.view) {
            this.view.clear();
        }
    }
    tryStart() {
        try {
            this.startAndShow();
        }
        catch (ex) {
            vscode_1.window.showErrorMessage("Error starting search debug listener: " + ex);
        }
    }
    isRunning() {
        return this.server !== null && this.server.listening;
    }
    startAndShow() {
        if (!this.isRunning()) {
            this.startServer();
        }
        this.view.showDebugView();
        this.showStatusBarItem();
    }
    startServer() {
        if (!this.search || !this.messageParser) {
            this.search = new Search_1.Search();
            const stateIdPattern = this.getStateIdPattern();
            this.messageParser = new MessageParser_1.MessageParser(this.search, stateIdPattern);
            this.view.observe(this.search);
        }
        const app = this.createApplication(this.search, this.messageParser);
        const defaultPort = vscode_1.workspace.getConfiguration(this.CONFIG_PDDL_SEARCH_DEBUGGER).get(this.CONF_DEFAULT_PORT, 0);
        this.port = defaultPort > 0 ? defaultPort : 8000 + Math.floor(Math.random() * 1000);
        this.server = http.createServer(app);
        this.server.on('error', e => {
            vscode_1.window.showErrorMessage(e.message);
            // eslint-disable-next-line @typescript-eslint/no-explicit-any
            if (e['code'] === "EADDRINUSE") {
                this.pddlConfiguration.askConfiguration(this.CONFIG_PDDL_SEARCH_DEBUGGER + "." + this.CONF_DEFAULT_PORT);
            }
        });
        this.server.on("listening", () => {
            console.log("Search debugger listening at port " + this.port);
            this.showStatus();
        });
        this.server.on("close", () => {
            console.log("Search debugger closed");
            this.server = null;
            this.showStatus();
        });
        this.server.listen(this.port, "127.0.0.1"); // listen to the local loop-back IP address only
    }
    getStateIdPattern() {
        const stateIdPatternAsString = vscode_1.workspace.getConfiguration(this.CONFIG_PDDL_SEARCH_DEBUGGER).get(this.CONF_STATE_ID_PATTERN);
        if (!stateIdPatternAsString) {
            throw new Error(`Missing configuration: ${this.CONFIG_PDDL_SEARCH_DEBUGGER}.${this.CONF_STATE_ID_PATTERN}`);
        }
        let stateIdPattern;
        try {
            stateIdPattern = new RegExp(stateIdPatternAsString);
        }
        catch (ex) {
            console.log("Invalid stateIdPattern regular expression: " + ex);
            throw new Error(`Invalid regular expression in configuration: ${this.CONFIG_PDDL_SEARCH_DEBUGGER}.${this.CONF_STATE_ID_PATTERN}`);
            // todo: this.pddlConfiguration.askConfiguration(this.CONFIG_PDDL_SEARCH_DEBUGGER + "." + this.CONF_STATE_ID_PATTERN);
        }
        return stateIdPattern;
    }
    createApplication(search, messageParser) {
        const app = express();
        app.use(bodyParser.json());
        app.get('/about', function (_req, res) {
            res.status(200).send('Visual search debugger.');
        });
        app.post('/state/initial', function (req, res) {
            try {
                search.addInitialState(messageParser.parseInitialState(req.body));
                res.status(201).end();
            }
            catch (ex) {
                console.log(ex);
                res.status(500).end();
            }
        });
        app.post('/state', function (req, res) {
            search.addState(messageParser.parseState(req.body));
            res.status(201).end();
        });
        // todo: the next one should be a 'patch' verb for '/state' path
        app.post('/state/visitedOrWorse', function (req, res) {
            search.update(messageParser.parseState(req.body).setVisitedOrIsWorse());
            res.status(201).end();
        });
        // todo: the next one should be a 'patch' verb for '/state' path
        app.post('/state/heuristic', function (req, res) {
            search.update(messageParser.parseEvaluatedState(req.body));
            res.status(200).end();
        });
        app.post('/plan', function (req, res) {
            search.setPlan(messageParser.parseState(req.body));
            res.status(200).end();
        });
        return app;
    }
    tryStop() {
        var _a;
        try {
            this.stop();
        }
        catch (ex) {
            vscode_1.window.showErrorMessage("Error stopping search debug listener: " + ((_a = ex.message) !== null && _a !== void 0 ? _a : ex));
        }
    }
    stop() {
        if (this.server !== null) {
            this.server.close();
        }
        this.showStatus();
    }
    showStatus() {
        this.view.setDebuggerState(this.isRunning(), this.port);
        this.showStatusBarItem();
    }
    toggle() {
        if (this.isRunning()) {
            this.tryStop();
        }
        else {
            this.tryStart();
        }
    }
    mock() {
        try {
            this.view.clear();
            this.stop();
            this.startAndShow();
            new MockSearch_1.MockSearch(this.port).run();
        }
        catch (ex) {
            vscode_1.window.showErrorMessage("Error starting the mock search: " + ex);
        }
    }
    showStatusBarItem() {
        if (!this.statusBarItem) {
            // create it and show it the first time the search debugger is used
            this.statusBarItem = vscode_1.window.createStatusBarItem(vscode_1.StatusBarAlignment.Left, PlannersConfiguration_1.STATUS_BAR_PRIORITY);
            this.context.subscriptions.push(this.statusBarItem);
            this.context.subscriptions.push(vscode_extension_telemetry_wrapper_1.instrumentOperationAsVsCodeCommand(SearchDebugger.TOGGLE_COMMAND, () => this.toggle()));
            this.statusBarItem.command = SearchDebugger.TOGGLE_COMMAND;
            this.statusBarItem.show();
        }
        this.statusBarItem.text = this.isRunning() ? '$(stop-circle)' : '$(record)';
        this.statusBarItem.color = this.isRunning() ? undefined : 'red';
        const status = this.isRunning() ? `listening on port ${this.port}. Click here to stop it.` : 'OFF. Click here to start it.';
        this.statusBarItem.tooltip = `PDDL Search debugger is ${status}.`;
    }
}
exports.SearchDebugger = SearchDebugger;
SearchDebugger.TOGGLE_COMMAND = "pddl.searchDebugger.toggle";
//# sourceMappingURL=SearchDebugger.js.map