/* --------------------------------------------------------------------------------------------
 * Copyright (c) Jan Dolejsi. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for license information.
 * ------------------------------------------------------------------------------------------ */
'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
exports.SearchDebuggerView = void 0;
const vscode_1 = require("vscode");
const path = require("path");
const utils_1 = require("../utils");
const pddl_workspace_1 = require("pddl-workspace");
const configuration_1 = require("../configuration/configuration");
const DomainVisualization_1 = require("../planView/DomainVisualization");
const ai_planning_val_1 = require("ai-planning-val");
const valStepErrorHandler_1 = require("../planView/valStepErrorHandler");
const webviewUtils_1 = require("../webviewUtils");
class SearchDebuggerView {
    constructor(context) {
        this.context = context;
        this.subscriptions = [];
        this.stateChangedWhileViewHidden = false;
        this.stateLogLineCache = new Map();
        this.VIEWS = "views";
        this.STATIC_CONTENT_FOLDER = path.join(this.VIEWS, "searchview", "static");
        this.COMMON_FOLDER = path.join(this.VIEWS, "common");
    }
    isVisible() {
        return this.webViewPanel !== undefined && this.webViewPanel.visible;
    }
    observe(search) {
        this.search = search;
        // first unsubscribe from previous search
        this.subscriptions.forEach(subscription => subscription.dispose());
        this.subscriptions = [];
        this.subscriptions.push(search.onStateAdded(newState => this.addState(newState)));
        this.subscriptions.push(search.onStateUpdated(newState => this.update(newState)));
        this.subscriptions.push(search.onBetterState(betterState => this.displayBetterState(betterState)));
        this.subscriptions.push(search.onPlanFound(planStates => this.displayPlan(planStates)));
    }
    setDomainAndProblem(domain, problem) {
        this.domain = domain;
        this.problem = problem;
        this.serializableDomain = undefined;
        this.serializableProblem = undefined;
    }
    getSerializableDomain() {
        var _a;
        return (_a = this.serializableDomain) !== null && _a !== void 0 ? _a : (this.serializableDomain = this.domain && pddl_workspace_1.utils.serializationUtils.makeSerializable(this.domain));
    }
    getSerializableProblem() {
        var _a;
        return (_a = this.serializableProblem) !== null && _a !== void 0 ? _a : (this.serializableProblem = this.problem && pddl_workspace_1.utils.serializationUtils.makeSerializable(this.problem));
    }
    async showDebugView() {
        if (this.webViewPanel !== undefined) {
            this.webViewPanel.reveal();
        }
        else {
            await this.createDebugView(false);
        }
    }
    async createDebugView(showOnTop) {
        const iconUri = this.context.asAbsolutePath('images/icon.png');
        this.webViewPanel = vscode_1.window.createWebviewPanel("pddl.SearchDebugger", "Search Debugger", {
            viewColumn: vscode_1.ViewColumn.Active,
            preserveFocus: !showOnTop
        }, {
            retainContextWhenHidden: true,
            enableFindWidget: true,
            enableCommandUris: true,
            enableScripts: true,
            localResourceRoots: [vscode_1.Uri.file(this.context.asAbsolutePath("views"))]
        });
        const html = await this.getHtml(this.webViewPanel.webview);
        this.webViewPanel.webview.html = html;
        this.webViewPanel.iconPath = vscode_1.Uri.file(iconUri);
        this.webViewPanel.onDidDispose(() => this.webViewPanel = undefined, undefined, this.context.subscriptions);
        this.webViewPanel.webview.onDidReceiveMessage(message => this.handleMessage(message), undefined, this.context.subscriptions);
        this.webViewPanel.onDidChangeViewState(event => this.changedViewState(event.webviewPanel));
        this.context.subscriptions.push(this.webViewPanel); // todo: this may not be necessary
    }
    changedViewState(webViewPanel) {
        if (webViewPanel.visible) {
            this.showDebuggerState();
            if (this.stateChangedWhileViewHidden) {
                // re-send all states
                this.showAllStates();
            }
            // reset the state
            this.stateChangedWhileViewHidden = false;
        }
    }
    // eslint-disable-next-line @typescript-eslint/no-explicit-any
    async handleMessage(message) {
        console.log(`Message received from the webview: ${message.command}`);
        switch (message.command) {
            case 'onload':
                this.showDebuggerState();
                break;
            case 'stateSelected':
                try {
                    this.showStatePlan(message.stateId);
                    this.scrollStateLog(message.stateId);
                }
                catch (ex) {
                    vscode_1.window.showErrorMessage("Error while displaying state-plan: " + ex);
                }
                break;
            case 'finalStateDataRequest':
                this.getFinalStateData(message.stateId).catch(utils_1.showError);
                break;
            case 'startDebugger':
                vscode_1.commands.executeCommand("pddl.searchDebugger.start");
                this.stateLogLineCache.clear();
                break;
            case 'stopDebugger':
                vscode_1.commands.executeCommand("pddl.searchDebugger.stop");
                break;
            case 'reset':
                vscode_1.commands.executeCommand("pddl.searchDebugger.reset");
                break;
            case 'toggleStateLog':
                this.toggleStateLog();
                break;
            case 'revealAction':
                this.domain && vscode_1.commands.executeCommand("pddl.revealAction", this.domain.fileUri, message.action);
                break;
            default:
                console.warn('Unexpected command: ' + message.command);
        }
    }
    async getHtml(webview) {
        const googleCharts = vscode_1.Uri.parse("https://www.gstatic.com/charts/");
        return webviewUtils_1.getWebViewHtml(this.context, {
            relativePath: this.STATIC_CONTENT_FOLDER, htmlFileName: 'search.html',
            externalImages: [vscode_1.Uri.parse('data:')],
            allowUnsafeInlineScripts: true,
            allowUnsafeEval: true,
            externalScripts: [googleCharts],
            externalStyles: [googleCharts],
            fonts: [
                vscode_1.Uri.file(path.join("..", "..", "..", this.COMMON_FOLDER, "codicon.ttf"))
            ]
        }, webview);
    }
    setDebuggerState(on, port) {
        this.debuggerState = on;
        this.port = port;
        this.showDebuggerState();
    }
    showDebuggerState() {
        this.postMessage({
            command: "debuggerState", state: {
                running: this.debuggerState ? 'on' : 'off',
                port: this.port
            }
        });
    }
    addState(newState) {
        new Promise(() => this.postMessage({ command: 'stateAdded', state: newState }))
            .catch(reason => console.log(reason));
    }
    update(state) {
        new Promise(() => this.postMessage({ command: 'stateUpdated', state: state }))
            .catch(reason => console.log(reason));
    }
    showAllStates() {
        var _a, _b;
        const allStates = (_b = (_a = this.search) === null || _a === void 0 ? void 0 : _a.getStates()) !== null && _b !== void 0 ? _b : [];
        new Promise(() => this.postMessage({ command: 'showAllStates', state: allStates }))
            .catch(reason => console.log(reason));
    }
    displayBetterState(state) {
        var _a;
        try {
            this.showStatePlan(state.id);
        }
        catch (ex) {
            vscode_1.window.showErrorMessage((_a = ex.message) !== null && _a !== void 0 ? _a : ex);
        }
    }
    displayPlan(planStates) {
        new Promise(() => this.postMessage({ command: 'showPlan', state: planStates }))
            .catch(reason => console.log(reason));
    }
    // eslint-disable-next-line @typescript-eslint/no-explicit-any
    postMessage(message) {
        if (this.webViewPanel !== undefined) {
            this.webViewPanel.webview.postMessage(message);
            if (!this.webViewPanel.visible) {
                this.stateChangedWhileViewHidden = true;
            }
        }
    }
    async showStatePlan(stateId) {
        if (this.search === undefined) {
            return void 0;
        }
        if (stateId === null) {
            return void 0;
        }
        const state = this.search.getState(stateId);
        if (!state || !this.webViewPanel) {
            return;
        }
        const statePlan = new pddl_workspace_1.search.SearchStateToPlan(this.getSerializableDomain(), this.getSerializableProblem(), configuration_1.DEFAULT_EPSILON).convert(state);
        const plansData = await DomainVisualization_1.getDomainVisualizationConfigurationDataForPlan(statePlan);
        this.postMessage({
            command: 'showStatePlan', state: plansData
        });
    }
    async getFinalStateData(stateId) {
        if (this.search === undefined) {
            return void 0;
        }
        if (stateId === null || stateId === undefined) {
            return void 0;
        }
        const state = this.search.getState(stateId);
        if (!state || !this.webViewPanel) {
            return;
        }
        const valStepPath = utils_1.ensureAbsoluteGlobalStoragePath(vscode_1.workspace.getConfiguration(configuration_1.CONF_PDDL).get(configuration_1.VAL_STEP_PATH), this.context);
        const valVerbose = vscode_1.workspace.getConfiguration(configuration_1.CONF_PDDL).get(configuration_1.VAL_VERBOSE, false);
        const happenings = state.planHead.map(searchHappening => toHappening(searchHappening));
        if (this.domain && this.problem) {
            try {
                const finalStateValues = await new ai_planning_val_1.PlanEvaluator().evaluateHappenings(this.domain, this.problem, happenings, { valStepPath, verbose: valVerbose });
                if (finalStateValues) {
                    const data = {
                        finalState: finalStateValues.map(tvv => tvv.getVariableValue()),
                        planIndex: stateId
                    };
                    this.postMessage({
                        "command": "visualizeFinalState",
                        "state": data
                    });
                }
            }
            catch (err) {
                console.error(err);
                if (valStepPath) {
                    valStepErrorHandler_1.handleValStepError(err, valStepPath);
                }
            }
        }
    }
    clear() {
        this.postMessage({ command: 'clear', state: 'n/a' });
        this.stateLogLineCache.clear();
        this.domain = undefined;
        this.problem = undefined;
    }
    async toggleStateLog() {
        if (this.stateLogFile !== undefined) {
            this.postMessage({ command: 'stateLog', state: null });
        }
        else {
            const selectedUri = await vscode_1.window.showOpenDialog({ canSelectMany: false, defaultUri: this.stateLogFile, canSelectFolders: false });
            if (!selectedUri) {
                return;
            }
            this.stateLogFile = selectedUri[0];
            this.stateLogEditor = await vscode_1.window.showTextDocument(await vscode_1.workspace.openTextDocument(this.stateLogFile), { preserveFocus: true, viewColumn: vscode_1.ViewColumn.Beside });
            this.postMessage({ command: 'stateLog', state: this.stateLogFile.fsPath });
        }
    }
    async scrollStateLog(stateId) {
        if (!this.stateLogFile || !this.stateLogEditor || !this.search) {
            return;
        }
        const state = this.search.getState(stateId);
        if (!state) {
            return;
        }
        if (this.stateLogEditor.document.isClosed) {
            this.stateLogEditor = await vscode_1.window.showTextDocument(this.stateLogEditor.document, vscode_1.ViewColumn.Beside);
        }
        if (this.stateLogLineCache.has(state.origId)) {
            const cachedLineId = this.stateLogLineCache.get(state.origId);
            if (cachedLineId) {
                this.stateLogEditor.revealRange(new vscode_1.Range(cachedLineId, 0, cachedLineId + 1, 0), vscode_1.TextEditorRevealType.AtTop);
            }
            return;
        }
        const pattern = vscode_1.workspace.getConfiguration("pddlSearchDebugger").get("stateLogPattern", "");
        for (let lineIdx = 0; lineIdx < this.stateLogEditor.document.lineCount; lineIdx++) {
            const logLine = this.stateLogEditor.document.lineAt(lineIdx);
            const patternMatch = logLine.text.match(new RegExp(pattern));
            if (patternMatch && patternMatch[1] === state.origId) {
                this.stateLogEditor.revealRange(logLine.range, vscode_1.TextEditorRevealType.AtTop);
                this.stateLogLineCache.set(state.origId, lineIdx);
                break;
            }
        }
    }
}
exports.SearchDebuggerView = SearchDebuggerView;
/**
 * Converts `SearchHappening` to `Happening`.
 * @param searchHappening plan happening that was created as a search state progression
 */
function toHappening(searchHappening) {
    return new pddl_workspace_1.Happening(searchHappening.earliestTime, searchHappening.kind, searchHappening.actionName, searchHappening.shotCounter);
}
//# sourceMappingURL=SearchDebuggerView.js.map