/* --------------------------------------------------------------------------------------------
 * Copyright (c) Jan Dolejsi. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for license information.
 * ------------------------------------------------------------------------------------------ */
'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
exports.PlanningDomainsSessions = void 0;
const vscode_1 = require("vscode");
const vscode_extension_telemetry_wrapper_1 = require("vscode-extension-telemetry-wrapper");
const utils_1 = require("../utils");
const opn = require("open");
const SessionDocumentContentProvider_1 = require("./SessionDocumentContentProvider");
const SessionSourceControl_1 = require("./SessionSourceControl");
const SessionRepository_1 = require("./SessionRepository");
const SessionConfiguration_1 = require("./SessionConfiguration");
const SessionUriHandler_1 = require("./SessionUriHandler");
const StudentNameParser_1 = require("./StudentNameParser");
const Classroom_1 = require("./Classroom");
const workspaceFs_1 = require("../util/workspaceFs");
/**
 * Handles the life-cycle of the planning-domains sessions.
 */
class PlanningDomainsSessions {
    constructor(context) {
        this.context = context;
        /** There may be multiple sessions in one workspace stored in different workspace folders.  */
        this.sessionSourceControlRegister = new Map(); // todo: replace with UriMap
        this.sessionDocumentContentProvider = new SessionDocumentContentProvider_1.SessionDocumentContentProvider();
        this.scanWorkspaceFoldersForConfigurationFiles(context);
        this.subscribe(vscode_1.window.registerUriHandler(new SessionUriHandler_1.SessionUriHandler()));
        this.subscribe(vscode_extension_telemetry_wrapper_1.instrumentOperationAsVsCodeCommand(SessionSourceControl_1.SESSION_COMMAND_LOAD, (sessionId, workspaceUri) => {
            this.tryOpenSession(context, sessionId, workspaceUri);
        }));
        this.subscribe(vscode_1.workspace.registerTextDocumentContentProvider(SessionRepository_1.SESSION_SCHEME, this.sessionDocumentContentProvider));
        this.subscribe(vscode_extension_telemetry_wrapper_1.instrumentOperationAsVsCodeCommand("pddl.planning.domains.session.refresh", async (sourceControlPane) => {
            const sourceControl = await this.pickSourceControl(sourceControlPane);
            if (sourceControl) {
                sourceControl.refresh();
            }
        }));
        this.subscribe(vscode_extension_telemetry_wrapper_1.instrumentOperationAsVsCodeCommand(SessionSourceControl_1.SESSION_COMMAND_REFRESH_ALL, async () => {
            this.sessionSourceControlRegister.forEach(async (sourceControl) => await sourceControl.refresh());
        }));
        this.subscribe(vscode_extension_telemetry_wrapper_1.instrumentOperationAsVsCodeCommand("pddl.planning.domains.session.discard", async (sourceControlPane) => {
            const sourceControl = await this.pickSourceControl(sourceControlPane);
            if (sourceControl) {
                sourceControl.resetFilesToCheckedOutVersion();
            }
        }));
        this.subscribe(vscode_extension_telemetry_wrapper_1.instrumentOperationAsVsCodeCommand(SessionSourceControl_1.SESSION_COMMAND_CHECKOUT, async (sourceControl, newVersion) => {
            sourceControl = sourceControl || await this.pickSourceControl();
            if (sourceControl) {
                sourceControl.tryCheckout(newVersion);
            }
        }));
        this.subscribe(vscode_extension_telemetry_wrapper_1.instrumentOperationAsVsCodeCommand("pddl.planning.domains.session.commit", async (sourceControlPane) => {
            const sourceControl = await this.pickSourceControl(sourceControlPane);
            if (sourceControl) {
                sourceControl.commitAll();
            }
        }));
        this.subscribe(vscode_extension_telemetry_wrapper_1.instrumentOperationAsVsCodeCommand("pddl.planning.domains.session.open", async (sourceControlPane) => {
            const sourceControl = await this.pickSourceControl(sourceControlPane);
            if (sourceControl) {
                this.openInBrowser(sourceControl.getSession()).catch(utils_1.showError);
            }
        }));
        this.subscribe(vscode_extension_telemetry_wrapper_1.instrumentOperationAsVsCodeCommand("pddl.planning.domains.session.duplicate", async (sourceControlPane) => {
            const sourceControl = await this.pickSourceControl(sourceControlPane);
            if (sourceControl) {
                this.duplicateAsWritable(sourceControl).catch(utils_1.showError);
            }
        }));
        this.subscribe(vscode_extension_telemetry_wrapper_1.instrumentOperationAsVsCodeCommand("pddl.planning.domains.session.share", async (sourceControlPane) => {
            const sourceControl = await this.pickSourceControl(sourceControlPane);
            if (sourceControl) {
                this.shareByEmail(sourceControl.getSession(), 'someone@somewhere.else', false).catch(utils_1.showError);
            }
        }));
        this.subscribe(vscode_extension_telemetry_wrapper_1.instrumentOperationAsVsCodeCommand("pddl.planning.domains.session.generateClassroom", async (sourceControlPane) => {
            const sourceControl = await this.pickSourceControl(sourceControlPane);
            if (sourceControl) {
                this.generateClassroom(sourceControl);
            }
        }));
        this.subscribe(vscode_1.workspace.onDidChangeWorkspaceFolders(e => {
            var _a;
            try {
                // initialize new source control for manually added workspace folders
                e.added.forEach(wf => {
                    this.initializeFromConfigurationFile(wf, context);
                });
            }
            catch (ex) {
                vscode_1.window.showErrorMessage((_a = ex.message) !== null && _a !== void 0 ? _a : ex);
            }
            finally {
                // dispose source control for removed workspace folders
                e.removed.forEach(wf => {
                    this.unregisterSessionSourceControl(wf.uri);
                });
            }
        }));
    }
    subscribe(disposable) {
        this.context.subscriptions.push(disposable);
    }
    async pickSourceControl(sourceControlPane) {
        if (sourceControlPane) {
            const rootUri = sourceControlPane.rootUri;
            return rootUri ? this.sessionSourceControlRegister.get(rootUri) : undefined;
        }
        // todo: when/if the SourceControl exposes a 'selected' property, use that instead
        if (this.sessionSourceControlRegister.size === 0) {
            return undefined;
        }
        else if (this.sessionSourceControlRegister.size === 1) {
            return [...this.sessionSourceControlRegister.values()][0];
        }
        else {
            const picks = [...this.sessionSourceControlRegister.values()].map(fsc => new RepositoryPick(fsc));
            if (vscode_1.window.activeTextEditor) {
                const activeWorkspaceFolder = vscode_1.workspace.getWorkspaceFolder(vscode_1.window.activeTextEditor.document.uri);
                const activeSourceControl = activeWorkspaceFolder && this.sessionSourceControlRegister.get(activeWorkspaceFolder.uri);
                const activeIndex = utils_1.firstIndex(picks, pick => pick.sessionSourceControl === activeSourceControl);
                // if there is an active editor, move its folder to be the first in the pick list
                if (activeIndex > -1) {
                    picks.unshift(...picks.splice(activeIndex, 1));
                }
            }
            const pick = await vscode_1.window.showQuickPick(picks, { placeHolder: 'Select repository' });
            return pick && pick.sessionSourceControl;
        }
    }
    async tryOpenSession(context, sessionId, workspaceUri) {
        var _a;
        try {
            await this.openSession(context, sessionId, workspaceUri);
        }
        catch (ex) {
            vscode_1.window.showErrorMessage((_a = ex.message) !== null && _a !== void 0 ? _a : ex);
            console.log(ex);
        }
    }
    async openSession(context, sessionId, workspaceUri) {
        var _a;
        if (workspaceUri && this.sessionSourceControlRegister.has(workspaceUri)) {
            vscode_1.window.showErrorMessage("Another session was already open in this workspace folder. Close it first, or select a different folder and try again.");
            return;
        }
        if (!sessionId) {
            sessionId = (_a = (await vscode_1.window.showInputBox({ prompt: 'Paste Planning.Domains session hash', placeHolder: 'hash e.g. XOacXgN1V7' }))) !== null && _a !== void 0 ? _a : '';
        }
        const mode = (await SessionRepository_1.checkSession(sessionId))[0];
        // show the file explorer with the new files
        vscode_1.commands.executeCommand("workbench.view.explorer");
        const workspaceFolder = await this.selectWorkspaceFolder(workspaceUri, sessionId, mode);
        if (!workspaceFolder) {
            return;
        } // canceled by user
        // unregister previous source control for this folder, if any
        this.unregisterSessionSourceControl(workspaceFolder.uri);
        // register source control
        const sessionSourceControl = await SessionSourceControl_1.SessionSourceControl.fromSessionId(sessionId, mode, context, workspaceFolder, true);
        this.registerSessionSourceControl(sessionSourceControl, context);
    }
    registerSessionSourceControl(sessionSourceControl, context) {
        // update the session document content provider with the latest content
        this.sessionDocumentContentProvider.updated(sessionSourceControl.getWorkspaceFolder(), sessionSourceControl.getSession());
        // every time the repository is updated with new session version, notify the content provider
        sessionSourceControl.onRepositoryChange(session => this.sessionDocumentContentProvider.updated(sessionSourceControl.getWorkspaceFolder(), session));
        this.sessionSourceControlRegister.set(sessionSourceControl.getWorkspaceFolder().uri, sessionSourceControl);
        context.subscriptions.push(sessionSourceControl);
        if (!this.refreshTimer) {
            this.refreshTimer = setInterval(() => this.updateStatusBar(), 60000);
        }
    }
    unregisterSessionSourceControl(folderUri) {
        if (this.sessionSourceControlRegister.has(folderUri)) {
            // the folder was already under source control
            const previousSourceControl = this.sessionSourceControlRegister.get(folderUri);
            previousSourceControl.dispose();
            this.sessionSourceControlRegister.delete(folderUri);
        }
        if (this.sessionSourceControlRegister.size === 0 && this.refreshTimer) {
            clearInterval(this.refreshTimer);
            this.refreshTimer = undefined;
        }
    }
    updateStatusBar() {
        this.sessionSourceControlRegister.forEach(sourceControl => {
            try {
                sourceControl.refreshStatusBar();
            }
            catch (ex) {
                console.log(ex);
            }
        });
    }
    /**
     * When the extension is loaded for a workspace that contains the session source control configuration file, initialize the source control.
     * @param context extension context
     */
    scanWorkspaceFoldersForConfigurationFiles(context) {
        if (!vscode_1.workspace.workspaceFolders) {
            return;
        }
        [...vscode_1.workspace.workspaceFolders]
            .sort((f1, f2) => f1.name.localeCompare(f2.name))
            .forEach(async (folder) => {
            try {
                await this.initializeFromConfigurationFile(folder, context);
            }
            catch (err) {
                const output = vscode_1.window.createOutputChannel("Planner output");
                output.appendLine(err);
                output.show();
            }
        });
    }
    async initializeFromConfigurationFile(folder, context) {
        const sessionFolder = await SessionConfiguration_1.isSessionFolder(folder);
        if (sessionFolder) {
            const sessionConfiguration = await SessionConfiguration_1.readSessionConfiguration(folder);
            const sessionSourceControl = await SessionSourceControl_1.SessionSourceControl.fromConfiguration(sessionConfiguration, folder, context, sessionConfiguration.versionDate === undefined);
            this.registerSessionSourceControl(sessionSourceControl, context);
        }
    }
    /**
     * Selects or validates the pre-determined workspace folder to clone into.
     * @param folderUri pre-determined folder
     * @param sessionId session ID
     * @param mode read/write mode
     */
    async selectWorkspaceFolder(folderUri, sessionId, mode) {
        let selectedFolder;
        let workspaceFolderUri;
        let workspaceFolderIndex;
        let folderOpeningMode;
        const sessionConfiguration = SessionConfiguration_1.toSessionConfiguration(sessionId, mode);
        if (folderUri) {
            // is this a currently open workspace folder?
            const currentlyOpenWorkspaceFolder = vscode_1.workspace.getWorkspaceFolder(folderUri);
            if (currentlyOpenWorkspaceFolder && currentlyOpenWorkspaceFolder.uri.fsPath === folderUri.fsPath) {
                selectedFolder = currentlyOpenWorkspaceFolder;
                workspaceFolderIndex = selectedFolder.index;
                workspaceFolderUri = selectedFolder.uri;
            }
            else {
                if (await workspaceFs_1.doesNotExist(folderUri)) {
                    await vscode_1.workspace.fs.createDirectory(folderUri); /* 0o777 */
                }
                else if (workspaceFs_1.isNotDirectory(folderUri)) {
                    vscode_1.window.showErrorMessage("Selected path is not a directory.");
                    return undefined;
                }
                workspaceFolderUri = folderUri;
            }
        }
        if (!workspaceFolderUri) {
            const folderPicks = [newFolderPick];
            if (vscode_1.workspace.workspaceFolders && vscode_1.workspace.workspaceFolders.length > 0) {
                folderPicks.push(newWorkspaceFolderPick);
                // for all workspace folders...
                for (const wf of vscode_1.workspace.workspaceFolders) {
                    // ... add all their nested folders
                    const content = (await vscode_1.workspace.fs.readDirectory(wf.uri))
                        .filter(child => child[1] === vscode_1.FileType.Directory)
                        .map(child => child[0]); // get name
                    folderPicks.push(new ExistingWorkspaceFolderPick(wf, content));
                }
            }
            const selectedFolderPick = folderPicks.length === 1 ?
                folderPicks[0] :
                await vscode_1.window.showQuickPick(folderPicks, {
                    canPickMany: false, ignoreFocusOut: true, placeHolder: 'Pick workspace folder to create files in.'
                });
            if (!selectedFolderPick) {
                return undefined;
            }
            if (selectedFolderPick instanceof ExistingWorkspaceFolderPick) {
                selectedFolder = selectedFolderPick.workspaceFolder;
                workspaceFolderIndex = selectedFolder.index;
                workspaceFolderUri = selectedFolder.uri;
            }
            folderOpeningMode = selectedFolderPick.folderOpeningMode;
        }
        if (!workspaceFolderUri && !selectedFolder) {
            const folderUris = await vscode_1.window.showOpenDialog({ canSelectFolders: true, canSelectFiles: false, canSelectMany: false, openLabel: 'Select folder' });
            if (!folderUris) {
                return undefined;
            }
            workspaceFolderUri = folderUris[0];
            // was such workspace folder already open?
            workspaceFolderIndex = vscode_1.workspace.workspaceFolders
                && vscode_1.workspace.workspaceFolders.findIndex(wf => wf.uri.toString() === (workspaceFolderUri === null || workspaceFolderUri === void 0 ? void 0 : workspaceFolderUri.toString()));
        }
        if (!workspaceFolderUri) {
            throw new Error("failed assertion: workspace folder must have been selected");
        }
        if (!await this.clearWorkspaceFolder(workspaceFolderUri)) {
            return undefined;
        }
        // save folder configuration
        await SessionConfiguration_1.saveConfiguration(workspaceFolderUri, sessionConfiguration);
        if (folderOpeningMode === FolderOpeningMode.AddToWorkspace || folderOpeningMode === undefined) {
            const workSpacesToReplace = typeof workspaceFolderIndex === 'number' && workspaceFolderIndex > -1 ? 1 : 0;
            if (workspaceFolderIndex === undefined || workspaceFolderIndex < 0) {
                workspaceFolderIndex = 0;
            }
            // replace or insert the workspace
            if (workspaceFolderUri) {
                vscode_1.workspace.updateWorkspaceFolders(workspaceFolderIndex, workSpacesToReplace, { uri: workspaceFolderUri });
            }
        }
        else if (folderOpeningMode === FolderOpeningMode.OpenFolder) {
            vscode_1.commands.executeCommand("vscode.openFolder", workspaceFolderUri);
        }
        return selectedFolder;
    }
    async clearWorkspaceFolder(workspaceFolderUri) {
        if (!workspaceFolderUri) {
            return undefined;
        }
        // check if the workspace is empty, or clear it
        const existingWorkspaceFiles = (await vscode_1.workspace.fs.readDirectory(workspaceFolderUri))
            .map(child => child[0]); // take file name, not the type
        if (existingWorkspaceFiles.length > 0) {
            const answer = await vscode_1.window.showQuickPick(["Yes", "No"], { placeHolder: `Remove ${existingWorkspaceFiles.length} file(s) from the folder ${workspaceFolderUri.fsPath} before cloning the remote repository?` });
            if (!answer) {
                return false;
            }
            if (answer === "Yes") {
                existingWorkspaceFiles
                    .forEach(async (filename) => await vscode_1.workspace.fs.delete(vscode_1.Uri.joinPath(workspaceFolderUri, filename), { recursive: true, useTrash: true }));
            }
        }
        return true;
    }
    async openDocumentInColumn(fileName, column) {
        const uri = vscode_1.Uri.file(fileName);
        // assuming the file was saved, let's open it in a view column
        const doc = await vscode_1.workspace.openTextDocument(uri);
        await vscode_1.window.showTextDocument(doc, { viewColumn: column });
    }
    async openInBrowser(session) {
        const sessionUri = this.createBrowserUri(session, true);
        return vscode_1.env.openExternal(vscode_1.Uri.parse(sessionUri));
    }
    async shareByEmail(session, email, readWrite) {
        const subject = `Planning.Domains session ${readWrite ? session.writeHash : session.hash}`;
        const body = `Open session in your browser: ${this.createBrowserUri(session, readWrite)}
Open session in Visual Studio Code: ${this.createVSCodeUri(session, readWrite)}`;
        const mailTo = `mailto:${email}?subject=${encodeURIComponent(subject)}&body=${encodeURIComponent(body).replace('%23', '#')}`;
        return opn(mailTo);
    }
    createBrowserUri(session, readWrite) {
        let sessionUri = "http://editor.planning.domains/";
        if (session.writeHash && readWrite) {
            sessionUri += "#edit_session=" + session.writeHash;
        }
        else {
            sessionUri += "#read_session=" + session.hash;
        }
        return sessionUri;
    }
    createVSCodeUri(session, readWrite) {
        let sessionUri = "vscode://jan-dolejsi.pddl/planning.domains/session/";
        if (session.writeHash && readWrite) {
            sessionUri += "edit/" + session.writeHash;
        }
        else {
            sessionUri += session.hash;
        }
        return sessionUri;
    }
    async duplicateAsWritable(sourceControl) {
        const folderUris = await vscode_1.window.showOpenDialog({ canSelectFolders: true, canSelectFiles: false, canSelectMany: false, openLabel: 'Select folder for duplicated session files' });
        if (!folderUris) {
            return undefined;
        }
        await sourceControl.duplicateAsWritable(folderUris[0], true);
    }
    async generateClassroom(templateSourceControl) {
        const studentNameParser = new StudentNameParser_1.StudentNameParser();
        const studentNameInput = await vscode_1.window.showInputBox({
            ignoreFocusOut: true, prompt: 'List the student names/emails',
            placeHolder: 'Example: John Doe; student1@domain.com; Alice von Wunderland <alice@wunderland-blah-blah.com>',
            validateInput: input => studentNameParser.validateClassroomNames(input)
        });
        if (!studentNameInput) {
            return;
        }
        const students = studentNameParser.parse(studentNameInput);
        const studentSessionsPromises = students
            .map(async (student) => await this.duplicateClassroomSession(templateSourceControl, student));
        const studentSessions = await Promise.all(studentSessionsPromises);
        const workspacePath = await new Classroom_1.Classroom(templateSourceControl, studentSessions).createWorkspace();
        // email students their session address
        const emailPromises = studentSessions
            .filter(studentSession => !!studentSession.identity.email)
            .map(session => this.shareByEmail(session.sessionConfiguration, session.identity.email, true));
        await Promise.all(emailPromises);
        // display summary of sessions
        const sessionSummaryCsv = studentSessions
            .map(session => `${session.identity.getEffectiveName()}, ${session.identity.email}, ${session.sessionConfiguration.writeHash}, ${this.createBrowserUri(session.sessionConfiguration, true)}`)
            .join('\n');
        const summaryDoc = await vscode_1.workspace.openTextDocument({ language: 'csv', content: sessionSummaryCsv });
        vscode_1.window.showTextDocument(summaryDoc);
        await summaryDoc.save();
        vscode_1.commands.executeCommand('vscode.openFolder', vscode_1.Uri.file(workspacePath));
    }
    /** duplicate the session */
    async duplicateClassroomSession(templateSourceControl, student) {
        const sessionPath = Classroom_1.Classroom.getSessionPath(templateSourceControl, student);
        const studentSessionHash = await templateSourceControl.duplicateAsWritable(vscode_1.Uri.file(sessionPath), false);
        return new Classroom_1.StudentSession(student, SessionConfiguration_1.toSessionConfiguration(studentSessionHash, SessionConfiguration_1.SessionMode.READ_WRITE));
    }
}
exports.PlanningDomainsSessions = PlanningDomainsSessions;
class RepositoryPick {
    constructor(sessionSourceControl) {
        this.sessionSourceControl = sessionSourceControl;
    }
    get label() {
        return this.sessionSourceControl.getSourceControl().label;
    }
    get description() {
        return this.sessionSourceControl.getWorkspaceFolder().name;
    }
}
class WorkspaceFolderPick {
    constructor(_label, folderOpeningMode) {
        this._label = _label;
        this.folderOpeningMode = folderOpeningMode;
    }
    get label() {
        return this._label;
    }
}
class ExistingWorkspaceFolderPick extends WorkspaceFolderPick {
    constructor(workspaceFolder, content) {
        super(workspaceFolder.name, FolderOpeningMode.AddToWorkspace);
        this.workspaceFolder = workspaceFolder;
        this.content = content;
    }
    get description() {
        return this.workspaceFolder.uri.fsPath;
    }
    get detail() {
        return this.content.length ?
            `${this.content.length} files/directories may need to be removed..` :
            ''; // this should not happen
    }
}
class NewWorkspaceFolderPick extends WorkspaceFolderPick {
    constructor(label, folderOpeningMode) {
        super(label, folderOpeningMode);
    }
}
var FolderOpeningMode;
(function (FolderOpeningMode) {
    FolderOpeningMode[FolderOpeningMode["AddToWorkspace"] = 0] = "AddToWorkspace";
    FolderOpeningMode[FolderOpeningMode["OpenFolder"] = 1] = "OpenFolder";
})(FolderOpeningMode || (FolderOpeningMode = {}));
const newWorkspaceFolderPick = new NewWorkspaceFolderPick("Select/create a local folder to add to this workspace...", FolderOpeningMode.AddToWorkspace);
const newFolderPick = new NewWorkspaceFolderPick("Select/create a local folder...", FolderOpeningMode.OpenFolder);
//# sourceMappingURL=PlanningDomainsSessions.js.map