/* --------------------------------------------------------------------------------------------
 * Copyright (c) Jan Dolejsi. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for license information.
 * ------------------------------------------------------------------------------------------ */
'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
exports.StudentName = exports.StudentNameParser = void 0;
/** Parses student email, name or both  */
class StudentNameParser {
    constructor() {
        this.EMAIL_PATTERN = /^(([^<>()\[\]\\.,;:\s@"]+(\.[^<>()\[\]\\.,;:\s@"]+)*)|(".+"))@((\[[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}])|(([a-zA-Z\-0-9]+\.)+[a-zA-Z]{2,}))$/;
        this.NAME_PATTERN = /^[\w \-\.]+$/;
        this.NAME_AND_EMAIL_PATTERN = /(^[\w \-\.]+)\s*(<(.+)>)?$/;
    }
    parse(input) {
        return input.split(';')
            .map(name => name.trim())
            .filter(name => name.length) // skip empty names e.g. after a trailing semicolon
            .map(name => this.parseName(name))
            .filter(student => !!student)
            .map(student => student);
    }
    validateClassroomNames(input) {
        const invalidNames = input.split(';')
            .map(name => name.trim())
            .filter(name => name.length > 0) // skip empty names e.g. after a trailing semicolon
            .filter(name => !this.parseName(name));
        if (invalidNames.length) {
            return 'Invalid names: ' + invalidNames.join(', ');
        }
        else {
            return null; // this is the expected value for 'input is valid'
        }
    }
    parseName(name) {
        this.EMAIL_PATTERN.lastIndex = 0;
        this.NAME_PATTERN.lastIndex = 0;
        this.NAME_AND_EMAIL_PATTERN.lastIndex = 0;
        let match;
        if (match = name.match(this.EMAIL_PATTERN)) {
            return new StudentName(match[0], match[0]); // it is valid
        }
        else if (match = name.match(this.NAME_PATTERN)) {
            return new StudentName(match[0]); // it is valid, but email address was not included
        }
        else if (match = name.match(this.NAME_AND_EMAIL_PATTERN)) {
            const emailPart = match[3].trim();
            this.EMAIL_PATTERN.lastIndex = 0;
            if (emailPart.match(this.EMAIL_PATTERN) === null) {
                return null;
            }
            return new StudentName(match[1].trim(), match[3].trim());
        }
        return null; // it is invalid
    }
}
exports.StudentNameParser = StudentNameParser;
class StudentName {
    constructor(name, email) {
        this.name = name;
        this.email = email;
    }
    getEffectiveName() {
        return this.name;
    }
    /**
     * Returns sanitized student name, so a folder may be created from it.
     */
    getEffectivePath() {
        return this.name.replace(' ', '_').replace('@', '_at_');
    }
}
exports.StudentName = StudentName;
//# sourceMappingURL=StudentNameParser.js.map