/* --------------------------------------------------------------------------------------------
 * Copyright (c) Jan Dolejsi 2019. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for license information.
 * ------------------------------------------------------------------------------------------ */
'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
exports.ModelHierarchyProvider = void 0;
const vscode_1 = require("vscode");
const SymbolUtils_1 = require("./SymbolUtils");
const pddl_workspace_1 = require("pddl-workspace");
const pddl_workspace_2 = require("pddl-workspace");
const pddl_workspace_3 = require("pddl-workspace");
const utils_1 = require("../utils");
const workspaceUtils_1 = require("../workspace/workspaceUtils");
const pddl_workspace_4 = require("pddl-workspace");
const pddl_workspace_5 = require("pddl-workspace");
class ModelHierarchyProvider {
    constructor(context, pddlWorkspace) {
        this.pddlWorkspace = pddlWorkspace;
        this.dirtyEditors = new Set();
        this.timeout = undefined;
        this.decorations = new Map();
        this.symbolUtils = new SymbolUtils_1.SymbolUtils(pddlWorkspace);
        vscode_1.window.onDidChangeActiveTextEditor(editor => this.scheduleDecoration(editor), null, context.subscriptions);
        pddlWorkspace.pddlWorkspace.on(pddl_workspace_5.PddlWorkspace.UPDATED, updatedFile => {
            if (updatedFile instanceof pddl_workspace_3.DomainInfo) {
                vscode_1.window.visibleTextEditors
                    .filter(editor => editor.document.uri.toString() === updatedFile.fileUri.toString())
                    .forEach(editor => this.scheduleDecoration(editor));
            }
        });
        vscode_1.window.visibleTextEditors.forEach(editor => this.scheduleDecoration(editor));
    }
    scheduleDecoration(editor) {
        if (editor && editor.visibleRanges.length && workspaceUtils_1.isPddl(editor.document)) {
            this.triggerDecorationRefresh(editor);
        }
    }
    triggerDecorationRefresh(editor) {
        this.dirtyEditors.add(editor);
        if (this.timeout) {
            clearTimeout(this.timeout);
            this.timeout = undefined;
        }
        this.timeout = setTimeout(() => this.refreshDirtyEditors(), 1000);
    }
    refreshDirtyEditors() {
        const currentlyDirtyEditors = new Set(this.dirtyEditors);
        this.dirtyEditors.clear();
        currentlyDirtyEditors
            .forEach(editor => this.updateDecoration(editor));
    }
    updateDecoration(editor) {
        var _a;
        if (editor.visibleRanges.length === 0) {
            return;
        }
        const fileInfo = this.pddlWorkspace.pddlWorkspace.getFileInfo(utils_1.toURI(editor.document.uri));
        if (fileInfo instanceof pddl_workspace_3.DomainInfo) {
            const domainInfo = fileInfo;
            const allVariables = domainInfo.getFunctions().concat(domainInfo.getPredicates());
            //todo: add derived variables
            (_a = this.decorations.get(editor)) === null || _a === void 0 ? void 0 : _a.forEach(d => d.dispose());
            this.decorations.delete(editor);
            const decorations = allVariables.map(v => this.decorateVariable(v, editor, domainInfo))
                .filter(dec => !!dec)
                .map(dec => dec);
            this.decorations.set(editor, decorations);
        }
    }
    decorateVariable(variable, editor, domainInfo) {
        const symbolInfo = this.symbolUtils.getSymbolInfo(editor.document, utils_1.toPosition(variable.getLocation().start).translate({ characterDelta: 1 }));
        if (symbolInfo instanceof SymbolUtils_1.VariableInfo) {
            const references = this.symbolUtils.findSymbolReferences(editor.document, symbolInfo, false);
            const pddlFileInfo = this.pddlWorkspace.getFileInfo(editor.document);
            if (!pddlFileInfo) {
                return undefined;
            }
            if (references !== undefined && domainInfo !== undefined) {
                const referenceInfos = this.getReferences(references, domainInfo, symbolInfo, editor.document);
                const readCount = referenceInfos
                    .filter(ri => [pddl_workspace_1.VariableReferenceKind.READ, pddl_workspace_1.VariableReferenceKind.READ_OR_WRITE].includes(ri.kind))
                    .length;
                const writeReferences = referenceInfos
                    .filter(ri => [pddl_workspace_1.VariableReferenceKind.WRITE, pddl_workspace_1.VariableReferenceKind.READ_OR_WRITE].includes(ri.kind));
                const writeEffectReferences = writeReferences
                    .filter(ri => (ri instanceof pddl_workspace_1.VariableEffectReferenceInfo))
                    .map(ri => ri);
                const increaseCount = writeEffectReferences.filter(ri => ri.effect instanceof pddl_workspace_4.parser.IncreaseEffect).length;
                const decreaseCount = writeEffectReferences.filter(ri => ri.effect instanceof pddl_workspace_4.parser.DecreaseEffect).length;
                const scaleUpCount = writeEffectReferences.filter(ri => ri.effect instanceof pddl_workspace_4.parser.ScaleUpEffect).length;
                const scaleDownCount = writeEffectReferences.filter(ri => ri.effect instanceof pddl_workspace_4.parser.ScaleDownEffect).length;
                const assignCount = writeEffectReferences.filter(ri => ri.effect instanceof pddl_workspace_4.parser.AssignEffect).length;
                const makeTrueCount = writeEffectReferences.filter(ri => ri.effect instanceof pddl_workspace_4.parser.MakeTrueEffect).length;
                const makeFalseCount = writeEffectReferences.filter(ri => ri.effect instanceof pddl_workspace_4.parser.MakeFalseEffect).length;
                const decorationText = [];
                const hoverText = [];
                if (readCount) {
                    decorationText.push(`${readCount}👁`);
                    hoverText.push(`${readCount}x read`);
                }
                if (increaseCount) {
                    decorationText.push(`${increaseCount}↗`);
                    hoverText.push(`${increaseCount}x increased`);
                }
                if (decreaseCount) {
                    decorationText.push(`${decreaseCount}↘`);
                    hoverText.push(`${decreaseCount}x decreased`);
                }
                if (scaleUpCount) {
                    decorationText.push(`${scaleUpCount}⤴`);
                    hoverText.push(`${scaleUpCount}x scaled up`);
                }
                if (scaleDownCount) {
                    decorationText.push(`${scaleDownCount}⤵`);
                    hoverText.push(`${scaleDownCount}x scaled down`);
                }
                if (assignCount) {
                    decorationText.push(`${assignCount}≔`);
                    hoverText.push(`${assignCount}x assigned`);
                }
                if (makeTrueCount) {
                    decorationText.push(`${makeTrueCount}☑`); // ⊨
                    hoverText.push(`${makeTrueCount}x made true`);
                }
                if (makeFalseCount) {
                    decorationText.push(`${makeFalseCount}☒`); // ⊭ 
                    hoverText.push(`${makeFalseCount}x made false`);
                }
                const rest = referenceInfos.length - readCount - increaseCount - decreaseCount - scaleUpCount - scaleDownCount - assignCount - makeTrueCount - makeFalseCount;
                if (rest) {
                    decorationText.push(`${rest}?`);
                    hoverText.push(`${rest}x unrecognized`);
                }
                return this.decorate(editor, decorationText.join(' '), hoverText.join('\n\n'), symbolInfo.location.range);
            }
        }
        return undefined;
    }
    decorate(editor, decorationText, hoverText, range) {
        const decorationType = vscode_1.window.createTextEditorDecorationType({
            after: {
                contentText: decorationText,
                textDecoration: "; color: gray; margin-left: 10px" //font-size: 10px; ; opacity: 0.5
            }
        });
        editor.setDecorations(decorationType, [{ range: range, hoverMessage: hoverText }]);
        return decorationType;
    }
    async provideHover(document, position, token) {
        if (token.isCancellationRequested) {
            return undefined;
        }
        await this.symbolUtils.assertFileParsed(document);
        const symbolInfo = this.symbolUtils.getSymbolInfo(document, position);
        if (symbolInfo instanceof SymbolUtils_1.VariableInfo) {
            const references = this.symbolUtils.findSymbolReferences(document, symbolInfo, false);
            const pddlFileInfo = this.pddlWorkspace.getFileInfo(document);
            if (!pddlFileInfo) {
                return undefined;
            }
            const domainInfo = this.pddlWorkspace.pddlWorkspace.asDomain(pddlFileInfo);
            if (references !== undefined && domainInfo !== undefined) {
                const referenceInfos = this.getReferences(references, domainInfo, symbolInfo, document);
                const documentation = this.createReferenceDocumentation(referenceInfos);
                return new vscode_1.Hover(documentation, symbolInfo.hover.range);
            }
            else {
                return undefined;
            }
        }
        else {
            return undefined;
        }
    }
    getReferences(references, domainInfo, symbolInfo, document) {
        return references
            .filter(r => r.uri.toString() === domainInfo.fileUri.toString()) // limit this to the domain file only
            .map(r => new pddl_workspace_1.ModelHierarchy(domainInfo).getReferenceInfo(symbolInfo.variable, document.offsetAt(r.range.start) + 1));
    }
    createReferenceDocumentation(referenceInfos) {
        const documentation = new vscode_1.MarkdownString(`**References**\n`);
        this.addAccessKindDocumentation(documentation, referenceInfos, 'Read', pddl_workspace_1.VariableReferenceKind.READ);
        this.addAccessKindDocumentation(documentation, referenceInfos, 'Write', pddl_workspace_1.VariableReferenceKind.WRITE);
        this.addAccessKindDocumentation(documentation, referenceInfos, 'Read or write', pddl_workspace_1.VariableReferenceKind.READ_OR_WRITE);
        this.addAccessKindDocumentation(documentation, referenceInfos, 'Unrecognized', pddl_workspace_1.VariableReferenceKind.UNRECOGNIZED);
        if (referenceInfos.length === 0) {
            documentation.appendText('\nNo references.');
        }
        return documentation;
    }
    addAccessKindDocumentation(documentation, referenceInfos, label, kind) {
        const accessReferences = referenceInfos.filter(ri => ri.kind === kind);
        if (accessReferences.length > 0) {
            documentation.appendText('\n' + label + ' access:\n');
            this.createAccessKindDocumentation(accessReferences, documentation);
        }
    }
    createAccessKindDocumentation(referenceInfos, documentation) {
        referenceInfos.forEach(ri => { var _a; return documentation.appendMarkdown(`\n- \`${ri.structure.getNameOrEmpty()}\` ${ri.getTimeQualifier()} ${ri.part}`).appendCodeblock((_a = ri.relevantCode) !== null && _a !== void 0 ? _a : '', pddl_workspace_2.PDDL); });
    }
}
exports.ModelHierarchyProvider = ModelHierarchyProvider;
//# sourceMappingURL=ModelHierarchyProvider.js.map