/* --------------------------------------------------------------------------------------------
 * Copyright (c) Jan Dolejsi. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for license information.
 * ------------------------------------------------------------------------------------------ */
'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
exports.SuggestionProvider = void 0;
const vscode_1 = require("vscode");
const MissingRequirements_1 = require("./MissingRequirements");
const pddl_workspace_1 = require("pddl-workspace");
const pddl_workspace_2 = require("pddl-workspace");
const pddl_workspace_3 = require("pddl-workspace");
const PTestCommands_1 = require("../ptest/PTestCommands");
const Test_1 = require("../ptest/Test");
const TestsManifest_1 = require("../ptest/TestsManifest");
const path_1 = require("path");
const UndeclaredVariable_1 = require("./UndeclaredVariable");
/**
 * Provides code actions for PDDL files.
 */
class SuggestionProvider {
    constructor(workspace) {
        this.workspace = workspace;
    }
    async provideCodeActions(document, range, context, token) {
        if (token.isCancellationRequested) {
            return [];
        }
        const fileInfo = await this.workspace.upsertFile(document);
        if (fileInfo === undefined) {
            throw new Error(`Not a PDDL file: ` + document.uri.toString());
        }
        const syntaxTree = new pddl_workspace_2.parser.PddlSyntaxTreeBuilder(fileInfo.getText()).getTree();
        const insertSnippetCodeActions = context.diagnostics
            .filter(diagnostic => diagnostic.code === SuggestionProvider.CONTENT_NOT_RECOGNIZED)
            .map(diagnostic => this.createSnippetSuggestions(document, diagnostic, range, fileInfo, syntaxTree));
        if (token.isCancellationRequested) {
            return [];
        }
        const missingRequirement = context.diagnostics
            .filter(diagnostic => diagnostic.message.match(MissingRequirements_1.MissingRequirements.undeclaredRequirementDiagnosticPattern))
            .map(diagnostic => this.createMissingRequirementAction(document, diagnostic, fileInfo))
            .filter(action => !!action)
            .map(action => action);
        if (token.isCancellationRequested) {
            return [];
        }
        const undeclaredVariable = context.diagnostics
            .filter(diagnostic => diagnostic.message.match(UndeclaredVariable_1.UndeclaredVariable.undeclaredVariableDiagnosticPattern))
            .map(diagnostic => this.createUndeclaredVariableAction(document, diagnostic, fileInfo))
            .filter(action => !!action)
            .map(action => action);
        if (token.isCancellationRequested) {
            return [];
        }
        const problemSnippets = this.createProblemActions(fileInfo, document, range, context);
        return pddl_workspace_1.utils.Util.flatMap(insertSnippetCodeActions).concat(missingRequirement).concat(undeclaredVariable).concat(problemSnippets);
    }
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    createProblemActions(fileInfo, document, range, _context) {
        if (!fileInfo.isProblem()) {
            return [];
        }
        if (!(range instanceof vscode_1.Range)) {
            return [];
        }
        const problemInfo = fileInfo;
        const preProcessor = problemInfo.getPreParsingPreProcessor();
        if (range.isSingleLine && preProcessor) {
            const focussedLine = document.lineAt(range.start.line);
            if (focussedLine.text.trim().match(/^;;\s*!pre-parsing:/)) {
                const title = "Preview generated problem file";
                const codeAction = new vscode_1.CodeAction(title, vscode_1.CodeActionKind.Empty);
                codeAction.command = { title: title, command: PTestCommands_1.PTEST_VIEW_PROBLEM, arguments: [this.createTest(document, preProcessor)] };
                return [codeAction];
            }
        }
        return [];
    }
    createTest(document, preProcessor) {
        const test = new Test_1.Test(preProcessor.getLabel(), "", "this should not be used", path_1.basename(document.uri.fsPath), "", preProcessor, []);
        test.setManifest(new TestsManifest_1.TestsManifest('unused', 'unused', 'unused', document.uri)); // only the uri is needed
        return test;
    }
    createSnippetSuggestions(document, diagnostic, range, fileInfo, syntaxTree) {
        const isWhitespaceOnly = syntaxTree.getRootNode().getChildren()
            .every(node => node.isType(pddl_workspace_2.parser.PddlTokenType.Comment) || node.isType(pddl_workspace_2.parser.PddlTokenType.Whitespace));
        const selectedNode = syntaxTree.getNodeAt(document.offsetAt(range.start));
        const isInsideWhitespace = selectedNode &&
            (selectedNode.isType(pddl_workspace_2.parser.PddlTokenType.Whitespace)
                || selectedNode.isType(pddl_workspace_2.parser.PddlTokenType.Document));
        const codeActions = [];
        if (fileInfo.isUnknownPddl() && isInsideWhitespace && isWhitespaceOnly) {
            {
                const title = 'Insert PDDL domain snippet...';
                const action = new vscode_1.CodeAction(title, vscode_1.CodeActionKind.QuickFix);
                action.diagnostics = [diagnostic];
                action.command = { command: 'editor.action.insertSnippet', arguments: [{ 'langId': pddl_workspace_3.PDDL, 'name': 'domain' }], title: title };
                action.isPreferred = true;
                codeActions.push(action);
            }
            {
                const title = 'Insert PDDL problem snippet...';
                const action = new vscode_1.CodeAction(title, vscode_1.CodeActionKind.QuickFix);
                action.diagnostics = [diagnostic];
                action.command = { command: 'editor.action.insertSnippet', arguments: [{ 'langId': pddl_workspace_3.PDDL, 'name': 'problem' }], title: title };
                codeActions.push(action);
            }
        }
        return codeActions;
    }
    createMissingRequirementAction(document, diagnostic, fileInfo) {
        const missingRequirementsDelegate = new MissingRequirements_1.MissingRequirements(fileInfo);
        const requirementName = missingRequirementsDelegate.getRequirementName(diagnostic.message);
        if (!requirementName) {
            return undefined;
        }
        const edit = missingRequirementsDelegate.createEdit(document, requirementName);
        const title = 'Add missing requirement ' + requirementName;
        const action = new vscode_1.CodeAction(title, vscode_1.CodeActionKind.QuickFix);
        action.edit = edit;
        action.diagnostics = [diagnostic];
        action.isPreferred = true;
        return action;
    }
    createUndeclaredVariableAction(document, diagnostic, fileInfo) {
        const undeclaredVariableDelegate = new UndeclaredVariable_1.UndeclaredVariable(fileInfo);
        const variableNode = undeclaredVariableDelegate.getVariable(diagnostic, document);
        if (!variableNode) {
            return undefined;
        }
        const [variable, node] = variableNode;
        const [edit, type] = undeclaredVariableDelegate.createEdit(document, variable, node);
        let sectionName;
        switch (type) {
            case UndeclaredVariable_1.VariableType.Function:
                sectionName = "function";
                break;
            case UndeclaredVariable_1.VariableType.Predicate:
                sectionName = "predicate";
                break;
            default:
                return undefined;
        }
        const title = `Add undeclared ${sectionName} (${variable.getFullName()})`;
        const action = new vscode_1.CodeAction(title, vscode_1.CodeActionKind.QuickFix);
        action.edit = edit;
        action.diagnostics = [diagnostic];
        action.isPreferred = true;
        return action;
    }
}
exports.SuggestionProvider = SuggestionProvider;
SuggestionProvider.CONTENT_NOT_RECOGNIZED = 'CONTENT_NOT_RECOGNIZED';
SuggestionProvider.providedCodeActionKinds = [
    vscode_1.CodeActionKind.QuickFix
];
//# sourceMappingURL=SuggestionProvider.js.map