/* --------------------------------------------------------------------------------------------
 * Copyright (c) Jan Dolejsi. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for license information.
 * ------------------------------------------------------------------------------------------ */
'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
exports.SymbolRenameProvider = void 0;
const vscode_1 = require("vscode");
const SymbolUtils_1 = require("./SymbolUtils");
class SymbolRenameProvider {
    constructor(pddlWorkspace) {
        this.symbolUtils = new SymbolUtils_1.SymbolUtils(pddlWorkspace);
    }
    async provideRenameEdits(document, position, newName, token) {
        if (token.isCancellationRequested) {
            return undefined;
        }
        await this.symbolUtils.assertFileParsed(document);
        const symbolInfo = this.symbolUtils.getSymbolInfo(document, position);
        if (!symbolInfo || !this.canRename(symbolInfo)) {
            throw new Error("This cannot be renamed.");
        }
        if (!newName.match(/^\w[-\w]*$/g)) {
            throw new Error(`This is not a valid PDDL name: ${newName}`);
        }
        const references = this.symbolUtils.findSymbolReferences(document, symbolInfo, true);
        if (!references) {
            return undefined;
        }
        const origName = document.getText(this.getWordRangeAtPosition(document, position));
        const workspaceEdits = new vscode_1.WorkspaceEdit();
        references.forEach(reference => {
            const oldName = this.findDocument(reference.uri).getText(reference.range);
            const replacementName = oldName.replace(origName, newName);
            workspaceEdits.replace(reference.uri, reference.range, replacementName);
        });
        return workspaceEdits;
    }
    getWordRangeAtPosition(document, position) {
        return document.getWordRangeAtPosition(position, /\w[-\w]*/g);
    }
    findDocument(fileUri) {
        const documentFound = vscode_1.workspace.textDocuments.find(textDoc => textDoc.uri.toString() === fileUri.toString());
        if (!documentFound) {
            throw new Error("Document not found in the workspace: " + fileUri.toString());
        }
        return documentFound;
    }
    canRename(symbolInfo) {
        return symbolInfo instanceof SymbolUtils_1.VariableInfo
            || symbolInfo instanceof SymbolUtils_1.TypeInfo
            || symbolInfo instanceof SymbolUtils_1.ParameterInfo;
    }
    async prepareRename(document, position, token) {
        await this.symbolUtils.assertFileParsed(document);
        if (token.isCancellationRequested) {
            return undefined;
        }
        const symbolInfo = this.symbolUtils.getSymbolInfo(document, position);
        if (!symbolInfo || !this.canRename(symbolInfo)) {
            throw new Error("This cannot be renamed.");
        }
        return this.getWordRangeAtPosition(document, position);
    }
}
exports.SymbolRenameProvider = SymbolRenameProvider;
//# sourceMappingURL=SymbolRenameProvider.js.map