/* --------------------------------------------------------------------------------------------
 * Copyright (c) Jan Dolejsi. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for license information.
 * ------------------------------------------------------------------------------------------ */
'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
exports.ValDownloader = void 0;
const vscode_1 = require("vscode");
const vscode_extension_telemetry_wrapper_1 = require("vscode-extension-telemetry-wrapper");
const path = require("path");
const os = require("os");
const pddl_workspace_1 = require("pddl-workspace");
const ai_planning_val_1 = require("ai-planning-val");
const configuration_1 = require("../configuration/configuration");
const valCommand_1 = require("./valCommand");
const utils_1 = require("../utils");
class ValDownloader extends ai_planning_val_1.ValDownloader {
    constructor(context) {
        super();
        this.context = context;
        this.binaryStorage = this.context.globalStoragePath;
        this.valVersionPath = path.join(this.getValPath(), "VAL.version");
    }
    registerCommands() {
        this.context.subscriptions.push(vscode_extension_telemetry_wrapper_1.instrumentOperationAsVsCodeCommand(valCommand_1.VAL_DOWNLOAD_COMMAND, async (options) => {
            var _a, _b;
            try {
                const userAgreesToDownload = (_a = options === null || options === void 0 ? void 0 : options.bypassConsent) !== null && _a !== void 0 ? _a : await this.promptForConsent();
                if (!userAgreesToDownload) {
                    return;
                }
                await this.downloadConfigureAndCleanUp();
            }
            catch (ex) {
                vscode_1.window.showErrorMessage((_b = ex.message) !== null && _b !== void 0 ? _b : ex);
            }
        }));
        return this;
    }
    async downloadDelegate(url, zipPath, message) {
        return await vscode_1.window.withProgress({ location: vscode_1.ProgressLocation.Window, title: message }, () => {
            return super.downloadDelegate(url, zipPath, message);
        });
    }
    /** Directory where VAL binaries are to be downloaded locally. */
    getValPath() {
        return path.join(this.binaryStorage, ValDownloader.VAL_DIR);
    }
    asAbsoluteStoragePath(relativePath) {
        return path.join(this.binaryStorage, relativePath);
    }
    async promptForConsent() {
        const download = "Download";
        const message = `Please confirm to download [VAL tools](${ValDownloader.VAL_REPO}) build from ${ValDownloader.VAL_BINARY_PROJECT}?`;
        const answer = await vscode_1.window.showInformationMessage(message, { modal: true }, download);
        return answer === download;
    }
    async downloadConfigureAndCleanUp() {
        const wasValInstalled = await this.isInstalled();
        const previousVersion = wasValInstalled ? await this.readVersion() : undefined;
        let newVersion;
        try {
            newVersion = await this.downloadAndConfigure();
        }
        finally {
            // clean previous version
            if (wasValInstalled && previousVersion && newVersion) {
                if (previousVersion.buildId !== newVersion.buildId) {
                    console.log(`The ${previousVersion.version} and the ${newVersion.version} differ, cleaning-up the old version.`);
                    const filesAbsPaths = previousVersion.files.map(f => this.asAbsoluteStoragePath(path.join(ValDownloader.VAL_DIR, f)));
                    await ValDownloader.deleteAll(filesAbsPaths);
                }
            }
        }
        return newVersion;
    }
    getLatestStableValBuildId() {
        var _a;
        return (_a = vscode_1.workspace.getConfiguration(configuration_1.CONF_PDDL).get(configuration_1.VALIDATOR_VERSION)) !== null && _a !== void 0 ? _a : -1;
    }
    async downloadAndConfigure() {
        const buildId = this.getLatestStableValBuildId();
        await pddl_workspace_1.utils.afs.mkdirIfDoesNotExist(this.binaryStorage, 0o755);
        const newValVersion = await this.download(buildId, this.getValPath());
        const wasValInstalled = await this.isInstalled();
        const previousVersion = wasValInstalled ? await this.readVersion() : undefined;
        await this.writeVersion(newValVersion);
        await this.updateConfigurationPaths(newValVersion, previousVersion);
        return newValVersion;
    }
    async isInstalled() {
        return await pddl_workspace_1.utils.afs.exists(this.valVersionPath);
    }
    async readVersion() {
        return ai_planning_val_1.readValManifest(this.valVersionPath);
    }
    async writeVersion(valVersion) {
        return ai_planning_val_1.writeValManifest(this.valVersionPath, valVersion);
    }
    /**
     * Configures the val tool paths
     * @param newValVersion val version just downloaded
     * @param oldValVersion val version from which we are upgrading
     */
    async updateConfigurationPaths(newValVersion, oldValVersion) {
        const fileToConfig = new Map();
        fileToConfig.set("Parser", configuration_1.PARSER_EXECUTABLE_OR_SERVICE);
        fileToConfig.set("Validate", configuration_1.CONF_PDDL + '.' + configuration_1.VALIDATION_PATH);
        fileToConfig.set("ValueSeq", configuration_1.CONF_PDDL + '.' + configuration_1.VALUE_SEQ_PATH);
        fileToConfig.set("ValStep", configuration_1.CONF_PDDL + '.' + configuration_1.VAL_STEP_PATH);
        for (const toolName of fileToConfig.keys()) {
            const oldToolPath = findValToolPath(oldValVersion, toolName);
            const newToolPath = findValToolPath(newValVersion, toolName);
            if (!newToolPath) {
                console.log(`Tool ${toolName} not found in the downloaded archive.`);
                return;
            }
            const oldToolAbsPath = oldToolPath && this.asAbsoluteStoragePath(path.join(ValDownloader.VAL_DIR, oldToolPath));
            const newToolAbsPath = this.asAbsoluteStoragePath(path.join(ValDownloader.VAL_DIR, newToolPath));
            const configKey = fileToConfig.get(toolName);
            if (configKey && newToolPath) {
                await this.updateConfigurationPath(toolName, configKey, newToolAbsPath, oldToolAbsPath);
            }
        }
    }
    /**
     * Updates the configuration path for the configuration key, unless it was explicitly set by the user.
     * @param toolName name of the tool being configured
     * @param configKey configuration key in the form prefix.postfix
     * @param newToolPath the location of the currently downloaded/unzipped tool
     * @param prevDownloadToolPath the location of the previously downloaded/unzipped tool (if known)
     */
    async updateConfigurationPath(toolName, configKey, newToolPath, prevDownloadToolPath) {
        const configurationInspect = vscode_1.workspace.getConfiguration().inspect(configKey);
        if (!configurationInspect) {
            console.log("configuration not declared: " + configKey);
            return;
        }
        const normConfiguredGlobalValue = this.normalizePathIfValid(
        // for backward compatibility
        utils_1.ensureAbsoluteGlobalStoragePath(configurationInspect.globalValue, this.context));
        const normPrevDownloadToolPath = this.normalizePathIfValid(prevDownloadToolPath);
        // was the configuration value empty?
        if (normConfiguredGlobalValue === undefined
            // or did it match the previous download?
            // (i.e.it came from the previous download and the user did not change the configuration)
            || normConfiguredGlobalValue === normPrevDownloadToolPath
            // or does the path not exist anyway?
            // || prevDownloadToolPath && !await utils.afs.exists(this.asAbsoluteStoragePath(prevDownloadToolPath))
            || await this.shouldOverwrite(toolName, normConfiguredGlobalValue, newToolPath)) {
            // Overwrite it!
            return await vscode_1.workspace.getConfiguration().update(configKey, newToolPath, vscode_1.ConfigurationTarget.Global);
        }
    }
    async shouldOverwrite(toolName, yourConfiguredPath, newToolPath) {
        const overwrite = "Overwrite";
        const message = `VAL tool '${toolName}' was already configured:\n\nYour configuration: ${yourConfiguredPath}\n\nJust downloaded: ${newToolPath}\n\nWish to use the downloaded tool instead?`;
        const answer = await vscode_1.window.showInformationMessage(message, { modal: true }, overwrite, "Keep");
        return answer === overwrite;
    }
    normalizePathIfValid(pathToNormalize) {
        if (!pathToNormalize) {
            return pathToNormalize;
        }
        let normalizedPath = path.normalize(pathToNormalize);
        if (os.platform() === 'darwin' && normalizedPath.includes(' ')) {
            normalizedPath = normalizedPath.replace(' ', '\\ ');
        }
        return normalizedPath;
    }
    async isNewValVersionAvailable() {
        const isInstalled = await this.isInstalled();
        if (!isInstalled) {
            return false;
        }
        const latestStableValBuildId = this.getLatestStableValBuildId();
        const installedVersion = await this.readVersion();
        return latestStableValBuildId > installedVersion.buildId;
    }
}
exports.ValDownloader = ValDownloader;
/** Directory where VAL binaries are to be downloaded locally. */
ValDownloader.VAL_DIR = "val";
/**
 * Finds the path of given VAL tool in the given version.
 * @param valVersion VAL version manifest (if known)
 * @param toolName tool name for which we are looking for its path
 * @returns corresponding path, or _undefined_ if the _valVersion_ argument is null or undefined
 */
function findValToolPath(valVersion, toolName) {
    if (!valVersion) {
        return undefined;
    }
    const pattern = new RegExp("\\b" + toolName + "(?:\\.exe)?$");
    return valVersion.files.find(filePath => pattern.test(filePath));
}
//# sourceMappingURL=ValDownloader.js.map