/* --------------------------------------------------------------------------------------------
 * Copyright (c) Jan Dolejsi 2021. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for license information.
 * ------------------------------------------------------------------------------------------ */
'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
exports.asWebviewUri = exports.getWebViewHtml = void 0;
const vscode_1 = require("vscode");
const fs = require("fs");
const path = require("path");
async function getWebViewHtml(extensionContext, options, webview) {
    const htmlPath = extensionContext.asAbsolutePath(path.join(options.relativePath, options.htmlFileName));
    const templateHtml = (await fs.promises.readFile(htmlPath, { encoding: "utf-8", flag: 'r' })).toString();
    // generate nonce for secure calling of javascript
    const nonce = !options.allowUnsafeInlineScripts ? generateNonce() : undefined;
    // be sure that the template has a placeholder for Content Security Policy
    const cspPlaceholderPattern = /<!--\s*CSP\s*-->/i;
    if (!templateHtml.match(cspPlaceholderPattern) || templateHtml.includes('http-equiv="Content-Security-Policy"')) {
        throw new Error(`Template does not contain CSP placeholder or contains rogue CSP.`);
    }
    let html = templateHtml.replace(/<(script|img|link) ([^>]*)(src|href)="([^"]+)"/g, (sourceElement, elementName, middleBits, attribName, attribValue) => {
        if (isAbsoluteWebview(attribValue)) {
            return sourceElement;
        }
        const resource = getWebviewUri(extensionContext, options.relativePath, attribValue, webview);
        const nonceAttr = elementName.toLowerCase() === "script" && attribName.toLowerCase() === "src" && nonce ? `nonce="${nonce}" ` : "";
        return `<${elementName} ${middleBits !== null && middleBits !== void 0 ? middleBits : ""}${nonceAttr}${attribName}="${resource}"`;
    });
    if (webview) {
        cspPlaceholderPattern.lastIndex = 0;
        html = html.replace(cspPlaceholderPattern, createContentSecurityPolicy(extensionContext, webview, options, nonce));
    }
    return html;
}
exports.getWebViewHtml = getWebViewHtml;
function isAbsoluteWebview(attribValue) {
    return attribValue.match(/^(http[s]?|data):/i) !== null;
}
function getWebviewUri(extensionContext, relativePath, fileName, webview) {
    return asWebviewUri(vscode_1.Uri.joinPath(extensionContext.extensionUri, relativePath, fileName), webview);
}
function getAbsoluteWebviewUri(extensionContext, webview, options, uri) {
    if (uri.scheme === "file") {
        return getWebviewUri(extensionContext, options.relativePath, uri.fsPath.replace(/^\/..\//, '../'), webview);
    }
    else {
        return uri;
    }
}
function getAbsoluteWebviewUrisSSV(extensionContext, webview, options, uris) {
    var _a;
    return (_a = uris === null || uris === void 0 ? void 0 : uris.map(uri => getAbsoluteWebviewUri(extensionContext, webview, options, uri).toString()).join(" ")) !== null && _a !== void 0 ? _a : "";
}
function asWebviewUri(localUri, webview) {
    var _a;
    return (_a = webview === null || webview === void 0 ? void 0 : webview.asWebviewUri(localUri)) !== null && _a !== void 0 ? _a : localUri.with({ scheme: "vscode-resource" });
}
exports.asWebviewUri = asWebviewUri;
function createContentSecurityPolicy(extensionContext, webview, options, nonce) {
    const externalStyles = getAbsoluteWebviewUrisSSV(extensionContext, webview, options, options.externalStyles);
    const externalScripts = getAbsoluteWebviewUrisSSV(extensionContext, webview, options, options.externalScripts);
    const externalImages = getAbsoluteWebviewUrisSSV(extensionContext, webview, options, options.externalImages);
    const fonts = getAbsoluteWebviewUrisSSV(extensionContext, webview, options, options.fonts);
    const unsafeInline = "'unsafe-inline'";
    const scriptUnsafeInline = options.allowUnsafeInlineScripts ? unsafeInline : '';
    const scriptUnsafeEval = options.allowUnsafeEval ? "'unsafe-eval'" : '';
    const styleUnsafeInline = options.disableUnsafeInlineStyle ? '' : unsafeInline;
    const nonceCsp = nonce ? `'nonce-${nonce}'` : '';
    return `<meta http-equiv="Content-Security-Policy"
\t\tcontent="default-src 'none'; ` +
        `img-src ${webview.cspSource} ${externalImages} https:; ` +
        `font-src ${webview.cspSource} ${fonts};` +
        `script-src ${webview.cspSource} ${externalScripts} ${scriptUnsafeInline} ${scriptUnsafeEval} ${nonceCsp}; ` +
        `style-src ${webview.cspSource} ${externalStyles} ${styleUnsafeInline};"
\t/>`;
}
function generateNonce() {
    let text = '';
    const possible = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789';
    for (let i = 0; i < 32; i++) {
        text += possible.charAt(Math.floor(Math.random() * possible.length));
    }
    return text;
}
//# sourceMappingURL=webviewUtils.js.map